#include "Clock.h"
#include "Engine.h"


void Clock::Spawn(Vec3 p, Vec3 f, Vec3 r, Vec3 u)
{
//	gLog.OutPut("Spawning Clock\n");
	SetDefaultAttribs(p, f,r,u);
	mPos = mOPos = mPos + u*0.1f;

	mMins = mPos - Vec3(0.25, 0.25, 0.25);
	mMaxs = mPos + Vec3(0.25, 0.25, 0.25);
	fRock = 0;

	shadowRight = mRight;
	shadowFwd = mForward;
	shadowUp = mUp;
}
void Clock::Update()
{
	mOPos = mPos;
	mOForward = mForward;
	mORight = mRight;
	mOUp = mUp;

	if(gpEng->mPlayer.mbTimeOut || gpEng->mPlayer.mbStageWon)
		return;

	fRock += 0.15f;

	mForward = ArbitraryRotate(mForward,DEG2RAD(2.5f),mUp);
	mForward = ArbitraryRotate(mForward,DEG2RAD(sin(fRock)),mRight);
	mRight = ArbitraryRotate(mRight,DEG2RAD(2.5f),mUp);
	mRight = ArbitraryRotate(mRight,DEG2RAD(sin(fRock)),mRight);
	mUp = ArbitraryRotate(mUp,DEG2RAD(2.5f),mUp);
	mUp = ArbitraryRotate(mUp,DEG2RAD(sin(fRock)),mRight);

	if(!mpPlatform || mpPlatform->bVisible)
	if(gpEng->mPlayer.mPos.x >= mPos.x-1)
	if(gpEng->mPlayer.mPos.x <= mPos.x+1)
	if(gpEng->mPlayer.mPos.y >= mPos.y-1)
	if(gpEng->mPlayer.mPos.y <= mPos.y+1)
	if(gpEng->mPlayer.mPos.z >= mPos.z-1)
	if(gpEng->mPlayer.mPos.z <= mPos.z+1)
	if(Distance(mPos, gpEng->mPlayer.mPos)<0.35)//this could be slow, should do a bounds check first
	{
		PlaySample(CHNL_PICKUP1, gpEng->mSnd[SND_PICKUP1],false);
//		gpEng->mfStageTime += 10;
//		if(gpEng->mfStageTime > 61)
//			gpEng->mfStageTime = 61;
		gpEng->fPauseClockTime+=10;
		gpEng->mbPauseClock = true;
		if(gpEng->fGreyScaleClockStart == 0)
		{
			MasterMusicFadeout(200);
			gpEng->mClockMusic.fPlayedTime = gpEng->fMusicTimer;
			gpEng->mMainMusic.fPlayedTime = gpEng->fMusicTimer;
			if(gpEng->bBonusStage)
				gpEng->mBonusMusic.fPlayedTime = gpEng->fMusicTimer;
		//	PlayMusicFadeIn(gpEng->mClockMusic,true,300,true);
		}
		gpEng->fGreyScaleClockStart+=10;
		gpEng->fGreyScaleClock+=10;
		bRemove = true;
		Vec2 out = UTIL_GL::GL2D::ProjectPointInWindow(mPos,gpEng->mFrustum.proj, gpEng->mFrustum.model);
		TextParticle T("+10",out,Vec2(0,-4),0.985f,2.5f,Vec3(0,0.5f,1),Vec2(1,1),0);
		gpEng->mTextParty.push_back(T);
		gpEng->miScore += 250;
	}

}
void Clock::Render(const float interp, int pass)
{
	if(mpPlatform && !mpPlatform->bVisible)
		return;

	Vec3 p = UTIL_Misc::Interpolate(mPos, mOPos, interp);
	Vec3 f = UTIL_Misc::Interpolate(mForward, mOForward, interp);
	Vec3 r = UTIL_Misc::Interpolate(mRight, mORight, interp);
	Vec3 u = UTIL_Misc::Interpolate(mUp, mOUp, interp);
	if(mpPlatform)
		p += UTIL_Misc::Interpolate(mpPlatform->mPos, mpPlatform->mOPos, interp)-mpPlatform->mPos;
/*	glBegin(GL_LINES);
	glColor3f(1,0,0);
	glVertex3fv((p).v);
	glVertex3fv((p+u*0.5f).v);
//	glColor3f(0,1,0);
//	glVertex3fv((p-r*0.5f).v);
//	glVertex3fv((p+r*0.5f).v);
//	glColor3f(0,0,1);
//	glVertex3fv((p).v);
//	glVertex3fv((p+f*0.5f).v);
	glEnd();
//*/
	glPushMatrix();
	glTranslatef(p.x,p.y,p.z);
	float mat[16];
	mat[0] = f.x;
	mat[1] = f.y;
	mat[2] = f.z;
	mat[3] = 0;
	mat[4] = r.x;
	mat[5] = r.y;
	mat[6] = r.z;
	mat[7] = 0;
	mat[8] = u.x;
	mat[9] = u.y;
	mat[10] = u.z;
	mat[11] = 0;
	mat[12] = 0;
	mat[13] = 0;
	mat[14] = 0;
	mat[15] = 1;
	float alpha = 0.45;
	if(mpPlatform)
		alpha = mpPlatform->iFadeTime>=0?UTIL_Misc::TimeRamp(0,0.45,0,100,mpPlatform->iFadeTime):0;
	glMultMatrixf(mat);
	glRotatef(180,1,0,0);
	glScalef(0.15,0.15,0.15);

	glColor4f(mColor.x,mColor.y,mColor.z,1);
	gpEng->mObjects[MDL_CLOCK].RenderBuffer(true);
//*/
	glColor4f(1,1,1,1);

	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);

	glDepthMask(GL_TRUE);
	glDisable(GL_BLEND);
	glPopMatrix();
}
void Clock::RenderShadow(const float interp)
{
//	return;
	Vec3 p = UTIL_Misc::Interpolate(mPos, mOPos, interp);
	Vec3 f = UTIL_Misc::Interpolate(mForward, mOForward, interp);
	Vec3 r = UTIL_Misc::Interpolate(mRight, mORight, interp);
	Vec3 u = shadowUp;

	Vec3 pos = p;
	for(int n=0; n<80; n++)
	{
		pos = p + u *((float)n/10.0f);
		if(gpEng->GetMapTile(pos))
		{
			Block* b= gpEng->GetMapTile(pos);
/*			float	X = u.x!=0?
				(u.x<0?(int)pos.x+0.75f:(int)pos.x+0.25f):pos.x;
			float	Y = u.y!=0?
				(u.y<0?(int)pos.y+0.75f:(int)pos.y+0.25f):pos.y;
			float	Z = u.z!=0?
				(u.z<0?(int)pos.z+0.75f:(int)pos.z+0.25f):pos.z;
*/
			float X = u.x!=0?
				(u.x<0?b->pos.x+0.25f:b->pos.x-0.25f):pos.x;
			float Y = u.y!=0?
				(u.y<0?b->pos.y+0.25f:b->pos.y-0.25f):pos.y;
			float Z = u.z!=0?
				(u.z<0?b->pos.z+0.25f:b->pos.z-0.25f):pos.z;

			pos = Vec3(X,Y,Z);//&/
			pos -= u * 0.27f;
			if(b->mbPlatform)
			{
				Vec3 shadOff = UTIL_Misc::Interpolate(b->pPlat->mPos, b->pPlat->mOPos, interp)-b->pPlat->mPos;
				shadOff = Reflect(shadOff,mRight,0);
				shadOff = Reflect(shadOff,mForward,0);
				pos += shadOff;
			}
			break;
		}
	}//*/
	glPushMatrix();
	glTranslatef(pos.x,pos.y,pos.z);
	float mat[16];
	mat[0] = f.x;
	mat[1] = f.y;
	mat[2] = f.z;
	mat[3] = 0;
	mat[4] = r.x;
	mat[5] = r.y;
	mat[6] = r.z;
	mat[7] = 0;
	mat[8] = u.x;
	mat[9] = u.y;
	mat[10] = u.z;
	mat[11] = 0;
	mat[12] = 0;
	mat[13] = 0;
	mat[14] = 0;
	mat[15] = 1;
	glMultMatrixf(mat);
	glScalef(0.15,0.15,0);
	glRotatef(180,1,0,0);

	gpEng->mObjects[MDL_CLOCK].RenderBuffer(false);
	glPopMatrix();
}
void Clock::RenderDepth(const float interp)
{
	if(mpPlatform && !mpPlatform->bVisible)
		return;

	Vec3 p = UTIL_Misc::Interpolate(mPos, mOPos, interp);
	Vec3 f = UTIL_Misc::Interpolate(mForward, mOForward, interp);
	Vec3 r = UTIL_Misc::Interpolate(mRight, mORight, interp);
	Vec3 u = UTIL_Misc::Interpolate(mUp, mOUp, interp);
	if(mpPlatform)
		p += UTIL_Misc::Interpolate(mpPlatform->mPos, mpPlatform->mOPos, interp)-mpPlatform->mPos;
	glPushMatrix();
	glTranslatef(p.x,p.y,p.z);
	float mat[16];
	mat[0] = f.x;
	mat[1] = f.y;
	mat[2] = f.z;
	mat[3] = 0;
	mat[4] = r.x;
	mat[5] = r.y;
	mat[6] = r.z;
	mat[7] = 0;
	mat[8] = u.x;
	mat[9] = u.y;
	mat[10] = u.z;
	mat[11] = 0;
	mat[12] = 0;
	mat[13] = 0;
	mat[14] = 0;
	mat[15] = 1;
	glMultMatrixf(mat);
	glRotatef(180,1,0,0);
	glScalef(0.15,0.15,0.15);
	gpEng->mObjects[MDL_CLOCK].RenderBuffer(false);
//*/
	glPopMatrix();

}
Entity* Clock::Clone() const
{
	return new Clock(*this);
}
