#include "EDFormats.h"
#include "Engine.h"
#include "Entity.h"
#include "Gemstone.h"
#include "Spikes.h"
#include "Star.h"
#include "Hoop.h"
#include "Switch.h"
#include "Laser.h"
#include "Teleporter.h"

char FORMAT[6] = "MAP01";


bool Cube::Load(CFileIO& fIO)
{
	if(!fIO.ReadBinary(&Pos.x))
		return false;
	if(!fIO.ReadBinary(&Pos.y))
		return false;
	if(!fIO.ReadBinary(&Pos.z))
		return false;

	for(int j=0; j<6; j++)
	{
		if(!fIO.ReadBinary(&Faces[j]))
			return false;
	}

	return true;
}
bool CubeEnt::Load(CFileIO& fIO)
{
	if(!fIO.ReadBinary(&bPlayOnce))
		return false;
	if(!fIO.ReadBinary(&bReturn))
		return false;
	if(!fIO.ReadBinary(&bStartOn))
		return false;
	if(!fIO.ReadBinary(&bTriggerMessage))
		return false;
	if(!fIO.ReadBinary(&bTriggerTouch))
		return false;
	if(!fIO.ReadBinary(&bVisible))
		return false;
	if(!fIO.ReadBinary(&fMoveTime))
		return false;
	if(!fIO.ReadBinary(&fWaitDelay))
		return false;
	if(!fIO.ReadBinary(&mMaxs.x))
		return false;
	if(!fIO.ReadBinary(&mMaxs.y))
		return false;
	if(!fIO.ReadBinary(&mMaxs.z))
		return false;
	if(!fIO.ReadBinary(&mMins.x))
		return false;
	if(!fIO.ReadBinary(&mMins.y))
		return false;
	if(!fIO.ReadBinary(&mMins.z))
		return false;
	if(!fIO.ReadBinary(&mMoveOffset.x))
		return false;
	if(!fIO.ReadBinary(&mMoveOffset.y))
		return false;
	if(!fIO.ReadBinary(&mMoveOffset.z))
		return false;
	if(!fIO.ReadBinary(&name))
		return false;
	if(!fIO.ReadBinary(&Target))
		return false;
	if(!fIO.ReadBinary(&TriggerMessage))
		return false;
	if(!fIO.ReadBinary(&mPushVector.x))
		return false;
	if(!fIO.ReadBinary(&mPushVector.y))
		return false;
	if(!fIO.ReadBinary(&mPushVector.z))
		return false;
	if(!fIO.ReadBinary(&mbJump))
		return false;
	if(!fIO.ReadBinary(&mbDamage))
		return false;
	return true;
}
bool Ent::Load(CFileIO& fIO)
{
	if(!fIO.ReadBinary(&iType))
		return false;
	if(!fIO.ReadBinary(&Color.x))
		return false;
	if(!fIO.ReadBinary(&Color.y))
		return false;
	if(!fIO.ReadBinary(&Color.z))
		return false;
	if(!fIO.ReadBinary(&Pos.x))
		return false;
	if(!fIO.ReadBinary(&Pos.y))
		return false;
	if(!fIO.ReadBinary(&Pos.z))
		return false;
	if(!fIO.ReadBinary(&Forward.x))
		return false;
	if(!fIO.ReadBinary(&Forward.y))
		return false;
	if(!fIO.ReadBinary(&Forward.z))
		return false;
	if(!fIO.ReadBinary(&Right.x))
		return false;
	if(!fIO.ReadBinary(&Right.y))
		return false;
	if(!fIO.ReadBinary(&Right.z))
		return false;
	if(!fIO.ReadBinary(&Up.x))
		return false;
	if(!fIO.ReadBinary(&Up.y))
		return false;
	if(!fIO.ReadBinary(&Up.z))
		return false;

	if(iType == ENT_SPIKES || iType == ENT_ARROW)
	{
		if(!fIO.ReadBinary(&fStartDelay))
			return false;
		if(!fIO.ReadBinary(&fHoldDelay))
			return false;
	}
	if(iType == ENT_BUTTON || iType == ENT_TELEPORT)
	{
		if(!fIO.ReadBinary(&name))
			return false;
		if(!fIO.ReadBinary(&Target))
			return false;
	}
	if(iType == ENT_LASER)
	{
		if(!fIO.ReadBinary(&name))
			return false;
		if(!fIO.ReadBinary(&fStartDelay))
			return false;
	}

	for(int n=0; n<3; n++)
	{
		if(fabs(Forward[n]) > fabs(1.0) || fabs(Forward[n])<fabs(-1.0))
			Forward[n] = 0.0;
		if(fabs(Right[n]) > fabs(1.0) || fabs(Right[n])<fabs(-1.0))
			Right[n] = 0.0;
		if(fabs(Up[n]) > fabs(1.0) || fabs(Up[n])<fabs(-1.0))
			Up[n] = 0.0;
	}
	return true;
}

bool Engine::LoadMap(char* name, bool persist)
{
	gLog.OutPut("Loading map: ");
	gLog.OutPut(name);
	gLog.OutPut("\n");

	CFileIO fIO;
	if(!fIO.GetFile(name,"rb"))
		return false;
	char format[6];
	int numCubes;
	int numCubeEnts;
	int numEnts;
	int n;
	mgPersistanceCounter = 0;//reset this so we can clear out any objects that shouldnt be spawned
	miStageStarCount = 0;//we need to know how many stars are in the stage
	std::list<Cube> cubes;
	std::list<CubeEnt> cubeEnts;
	std::list<Ent> ents;

	if(!fIO.ReadBinary(&format))
		return false;

	if(strcmp(format, FORMAT))
	{
		gLog.OutPut("Map format differs..\n");
		return false;
	}

	if(!fIO.ReadBinary(&numCubes))
		return false;
	if(!fIO.ReadBinary(&numCubeEnts))
		return false;
	if(!fIO.ReadBinary(&numEnts))
		return false;

	int MINX=65, MINY=65, MINZ=65;
	int MAXX=0, MAXY=0, MAXZ=0;
	for(n=0; n<numCubes; n++)
	{
		Cube c;
		if(!c.Load(fIO))
		{
			gLog.OutPut("Error loading cube!\n");
			return false;
		}
		if(c.Pos.x<MINX)
			MINX=c.Pos.x;
		if(c.Pos.y<MINY)
			MINY=c.Pos.y;
		if(c.Pos.z<MINZ)
			MINZ=c.Pos.z;

		if(c.Pos.x>MAXX)
			MAXX=c.Pos.x;
		if(c.Pos.y>MAXY)
			MAXY=c.Pos.y;
		if(c.Pos.z>MAXZ)
			MAXZ=c.Pos.z;
		cubes.push_back(c);
	}
	gLog.OutPut("Loaded blocks.\n");

	for(n=0; n<numCubeEnts; n++)
	{
		CubeEnt c;
		if(!c.Load(fIO))
		{
			gLog.OutPut("Error loading cube entity!\n");
			return false;
		}
		if(c.mMins.x<MINX)
			MINX=c.mMins.x;
		if(c.mMins.y<MINY)
			MINY=c.mMins.y;
		if(c.mMins.z<MINZ)
			MINZ=c.mMins.z;

		if(c.mMaxs.x>MAXX)
			MAXX=c.mMaxs.x;
		if(c.mMaxs.y>MAXY)
			MAXY=c.mMaxs.y;
		if(c.mMaxs.z>MAXZ)
			MAXZ=c.mMaxs.z;
		cubeEnts.push_back(c);
	}
	gLog.OutPut("Loaded platforms.\n");

	Vec3 Center = Vec3((int)((MAXX-MINX)/2), (int)((MAXY-MINY)/2), (int)((MAXZ-MINZ)/2)) +Vec3(MINX,MINY,MINZ);
	Vec3 ShiftMap = Vec3(32,32,32)-Center;

	Vec3 CamSize = Vec3((int)((MAXX-MINX)), (int)((MAXY-MINY)), (int)((MAXZ-MINZ)));
	mPrevCamZoomDistance = CamSize.x;
	if(CamSize.y> mPrevCamZoomDistance)
		mPrevCamZoomDistance = CamSize.y;
	if(CamSize.z> mPrevCamZoomDistance)
		mPrevCamZoomDistance = CamSize.z;

	if(persist)
	for(n=0; n<numEnts; n++)
	{
		Ent c;
		if(!c.Load(fIO))
		{
			gLog.OutPut("Error loading entity!\n");
			return false;
		}
		c.Pos += ShiftMap;
		ents.push_back(c);
	}
	gLog.OutPut("Loaded ents.\n");

	ClearMap();
	//generate map from loaded data
	iColoredBlocks = 0;
	if(!cubes.empty())
	{
		std::list<Cube>::iterator c;
		for(c=cubes.begin(); c!=cubes.end(); c++)
		{
			Block b;
			b.pos = (*c).Pos + Vec3(0.5, 0.5, 0.5) + ShiftMap;
			b.scale = 1;
			for(n=0; n<6; n++)
			b.Faces[n] = (*c).Faces[n];
			b.bTagged = false;
			b.bSolid = true;
			b.mbPlatform = false;
			b.iScaleDelay = 35;
			b.bPass = false;
			b.iSpecial = 0;
			b.iColortime = 0;
			b.bColored = false;
			bool isSpecial = false;
			for(int J=0;J<6;J++)
			{
				if((*c).Faces[J]==SURF_ICE)
				{
					isSpecial = true;
					b.iSpecial = SURF_ICE;
				}
				else if((*c).Faces[J]==SURF_BREAK)
					isSpecial = true;
//				else if((*c).Faces[J]==SURF_JUMP)
//					isSpecial = true;
			}
			if(!isSpecial)
				gpEng->iColoredBlocks++;
			mMapBlocks.push_back(b);
			cMap[(int)(b.pos.z)*64*64+(int)(b.pos.y)*64+(int)(b.pos.x)]=&mMapBlocks.back();
		}
	}
	gLog.OutPut("Generated tiles.\n");

	if(!persist)
		return true;


	miStageGemCount = 0;
	mbPerfect = false;
	miPerfectTimer= 0;
	mbCollectedGem = false;
	mGameEnts.remove_if(CleanEnts);
	if(!ents.empty())
	{
		std::list<Ent>::iterator e;
		for(e=ents.begin(); e!=ents.end(); e++)
		{
			switch((*e).iType)
			{
			case ENT_PLR:
				mPlayer.Spawn((*e).Pos, (*e).Forward, (*e).Right, (*e).Up);
				mCameraPos = mOCameraPos = mPlayer.mPos - mPlayer.mForward * (15.5f*mfCamZoom) - mPlayer.mUp * (7.75f*mfCamVertical) + mPlayer.mRight * 30;
				break;

			//case ENT_GEM:
			default:
				{
					Entity* ent = EntFactory((*e).iType);
					if(ent)
					{
						if((*e).iType == ENT_GEM)
							miStageGemCount++;

						if((*e).iType == ENT_SPIKES)
						{
							((Spikes*)ent)->fStartDelay = (*e).fStartDelay;
							((Spikes*)ent)->fHoldDelay = (*e).fHoldDelay;
						}
						else if((*e).iType == ENT_ARROW)
						{
							((Spikes*)ent)->fStartDelay = (*e).fStartDelay;
						}
						else if((*e).iType == ENT_BUTTON)
						{
							strcpy(((Switch*)ent)->TargetName, (*e).Target);
							strcpy(((Switch*)ent)->Name, (*e).name);
						}
						else if((*e).iType == ENT_TELEPORT)
						{
							strcpy(((Teleporter*)ent)->TargetName, (*e).Target);
							strcpy(((Teleporter*)ent)->Name, (*e).name);
						}
						else if((*e).iType == ENT_LASER)
						{
							strcpy(((Laser*)ent)->Name, (*e).name);
							((Laser*)ent)->IsToggled = (*e).fStartDelay>0?true:false;
						}
						ent->Spawn((*e).Pos, (*e).Forward, (*e).Right, (*e).Up);
						ent->mColor = (*e).Color;
						mGameEnts.push_back(ent);
					}
					break;
				}
			}
		}
	}
	gLog.OutPut("Generated ents.\n");

	if(!this->bTutorialGame)
	{
	//remove an persistence entities here before platforms are created.
	std::list<Entity*>::iterator it;
	std::list<unsigned int>::iterator pt;
	for(pt=miStagePersistance[miSelectedStageBall].begin(); pt!=miStagePersistance[miSelectedStageBall].end(); pt++)
	{
		for(it=mGameEnts.begin(); it!=mGameEnts.end(); it++)
		{
			if((*it)->iBaseType != ENT_GEM && (*it)->iBaseType != ENT_STAR)
				continue;

			if((*it)->iBaseType == ENT_GEM)
			if(((Gemstone*)(*it))->iPersistanceID == (*pt))
			{
				(*it)->bRemove = true;
				miStageGemCount--;
			//	gLog.OutPut("Persistent gem found and tagged!\n");
			}
			if((*it)->iBaseType == ENT_STAR)
			if(((Star*)(*it))->iPersistanceID == (*pt))
			{
				((Star*)(*it))->bCollected = true;
				((Star*)(*it))->pStarPS->fStarColor = Vec3(0.25f,0.25f,0.25f);
			//	gLog.OutPut("Persistent star found and tagged!\n");
			}
		}
	}
	}
	mGameEnts.remove_if(PruneEnts);

	if(!cubeEnts.empty())
	{
		std::list<CubeEnt>::iterator c;
		for(c=cubeEnts.begin(); c!=cubeEnts.end(); c++)
		{
			Platform* p = new Platform;
			p->iLogicDebugStep = -1;
			p->iMaxMoveTime = (*c).fMoveTime * (1.0 / TIME_STEP);
			p->mbMoveDir = true;
			p->mMoveDest = (*c).mMoveOffset;
			p->iMaxWaitTime = (*c).fWaitDelay * (1.0 / TIME_STEP);
			p->bPlayOnce = (*c).bPlayOnce;
			p->bActivated = (*c).bStartOn;
			p->bReturnStart = (*c).bReturn;
			p->bVisible = (*c).bVisible;
			p->bTriggerTouch = (*c).bTriggerTouch;
			strcpy(p->TargetName, (*c).Target);
			strcpy(p->Name, (*c).name);
			strcpy(p->TriggerMessage, (*c).TriggerMessage);
			mPlatforms.push_back(p);
			mPlatforms.back()->Generate((*c).mMins + ShiftMap, (*c).mMaxs + ShiftMap);
			if(mPlatforms.back()->PointInPlat(mPlayer.mPos))
			{
				mPlayer.mpPlatform = mPlatforms.back();
				mPlatforms.back()->mbPlayerIsRiding = true;
			}
		}
	}
 	gLog.OutPut("Generated Plats.\n");

	gLog.OutPut("Done\n");
	return true;
}