#include "Engine.h"
#include "SDL_KeyMap.h"
#include "Serializer.h"
bool gbAnistropy = true;
bool gVsync = false;
bool gAntiAliasing = false;
int gAASamples = 0;
int samples = 0;

bool bRecordDemo = false;
bool bPlayDemo = false;
bool bVideoDemo = false;
char szDemoName[32];
Engine* gpEng = NULL;

bool betaTest = false;
int iDemoVal=0;
bool bForceFallback = false;

extern int DOF;
extern int FUR;
extern int SHADOW;
extern int SUPER_SAMPLING;
extern int SHOW_FPS;
extern int BLUR;
extern int LOGGING;

/********************************************************/
/* check for required opengl extensions					*/
/********************************************************/
bool Engine::InitializeExtensions()
{
	gLog.OutPut("\n[Initializing Extensions]\n");
	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		gLog.OutPut("Failed To Init GLEW!\n");
		return false;
	}
	gLog.OutPut("GLEW Ready!\n");

	//npots
	if(bForceFallback)
		gpEng->mbFallBackMode = true;
	else
	{
		gLog.OutPut("Testing GL_ARB_texture_rectangle support.\n");
		if (!glewIsSupported("GL_ARB_texture_rectangle"))
		{
			gLog.OutPut("GL_ARB_texture_rectangle not supported!\n");
			//return false;
			gpEng->mbFallBackMode = true;
		}
		else
		{
			gLog.OutPut("GL_ARB_texture_rectangle supported!\n");
			mbFallBackMode = false;
		}//*/
	}

	//fbo's
	gLog.OutPut("Testing GL_EXT_framebuffer_object support.\n");
	if (!glewIsSupported("GL_EXT_framebuffer_object"))
	{
		gLog.OutPut("GL_EXT_framebuffer_object not supported!\n");
		return false;
	}
	gLog.OutPut("GL_EXT_framebuffer_object supported!\n");

	//vertex programs
	gLog.OutPut("Testing GL_ARB_vertex_program support.\n");
	if (!glewIsSupported("GL_ARB_vertex_program"))
	{
		gLog.OutPut("GL_ARB_vertex_program not supported!\n");
		return false;
	}
	gLog.OutPut("GL_ARB_vertex_program supported!\n");

	//fragment programs
	gLog.OutPut("Testing GL_ARB_fragment_program support.\n");
	if (!glewIsSupported("GL_ARB_fragment_program"))
	{
		gLog.OutPut("GL_ARB_fragment_program not supported!\n");
		return false;
	}
	gLog.OutPut("GL_ARB_fragment_program supported!\n");


	//anistropy
	if(gbAnistropy)
	{
	gLog.OutPut("Testing GL_EXT_texture_filter_anisotropic support.\n");
	if (!glewIsSupported("GL_EXT_texture_filter_anisotropic"))
	{
		gLog.OutPut("GL_EXT_texture_filter_anisotropic not supported!\n");
		gbAnistropy = false;
		return false;
	}
	else
	{
		gLog.OutPut("GL_EXT_texture_filter_anisotropic supported!\n");
		glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &gMaxAnisotropy);
		char sz[32];
		sprintf(sz,"Max Anistropy: %.1f\n",gMaxAnisotropy);
		gAnisotropy = gMaxAnisotropy;
		gLog.OutPut(sz);
	}
	}
	const GLubyte* strgl = glGetString(GL_SHADING_LANGUAGE_VERSION);
	if(strgl!=0)
	{
		gLog.OutPut((char*)strgl);
		gLog.OutPut("\n");
		if(!strncmp("1.10",(char*)strgl,4))
		{
			gLog.OutPut("GLSL 1.20 not supported, switching to fallback mode.\n");
			gpEng->mbFallBackMode = true;
		}
		else if(!strncmp("1.00",(char*)strgl,4))
		{
			gLog.OutPut("GLSL 1.20 not supported, switching to fallback mode.\n");
			gpEng->mbFallBackMode = true;
		}
	}
	gLog.OutPut("Complete...\n\n");
	return true;
}

/**************************************************************/
/* engine constructor, init vars, load one time data		  */
/**************************************************************/
Engine::Engine(int w, int h, bool fscreen)
{

	gpEng = this;
	LOGGING?
		gLog.SetLogState(1):
		gLog.SetLogState(0);

	if(!UTIL_SDL::InitSDL("Screwball",w,h,32,fscreen,gAntiAliasing?gAASamples:0))
		return;
	if(!InitializeExtensions())
		return;
	if(!gVsync)
	{
		PFNWGLSWAPINTERVALEXTPROC wglSwapIntervalEXT  = (PFNWGLSWAPINTERVALEXTPROC) wglGetProcAddress("wglSwapIntervalEXT");
		if(wglSwapIntervalEXT)
		{
			wglSwapIntervalEXT(0);
			gLog.OutPut("Vsync Disabled.\n");
		}
	}
	else
	{
		PFNWGLSWAPINTERVALEXTPROC wglSwapIntervalEXT  = (PFNWGLSWAPINTERVALEXTPROC) wglGetProcAddress("wglSwapIntervalEXT");
		if(wglSwapIntervalEXT)
		{
			wglSwapIntervalEXT(1);
			gLog.OutPut("Vsync Enabled.\n");
		}
	}
	glClear(GL_COLOR_BUFFER_BIT);

	bJoystick = UTIL_SDL::InitJoystick();
	SDL_GL_SwapBuffers();
	if(!InitializeAudio(128))
		return;
	//MaserMusicVolume(0);

	if(bForceFallback)
		mbFallBackMode = true;
	winWidth = w;
	winHeight = h;
	bTextInput = false;
	bCfgJoystick = false;
	mbPaused = false;
	mbQuit = false;
	LoadInput();
	UTIL_Misc::Seed(99999);
	miInternalClock = 0;
//	mSurfaces.reserve(1000);
	miWorld = 0;
	bBonusStage = false;
	iBonusStage = 0;
	GCOLTIME=0;
	mbDOF = DOF?true:false;
	mbFUR = FUR?true:false;
	mbBlur = BLUR?true:false;
	mbShadows = SHADOW?true:false;
	mbShowFPS = SHOW_FPS?true:false;
	//SDL_WM_GrabInput(SDL_GRAB_ON);

	InitDevil();

	if(betaTest)
	{
		bRecordDemo = true;
		CFileIO fIO;
		int dem = 0;
		if(fIO.GetFile("Demo.dat","rt"))
		{
			gSerializer.ReadVariable("Demo.dat","DEMO",iDemoVal);
			DeleteFile("Demo.dat");
			dem = iDemoVal+1;
			gSerializer.ReadVariable("Demo.dat","DEMO",dem);
		}
		else
			gSerializer.ReadVariable("Demo.dat","DEMO",dem);
		sprintf(szDemoName, "Data/Demo/Dem%i.dem",iDemoVal);
	}


	if(mbFallBackMode)
		samples = 0;
	if(samples>4)samples = 4;
	if(samples<0)samples = 0;
	iSuperSamples = samples;
	switch(iSuperSamples)
	{
	case 1: w *= 1.25f; h *= 1.25f; break;
	case 2: w *= 1.5f; h *= 1.5f; break;
	case 3: w *= 1.75f; h *= 1.75f; break;
	case 4: w *= 2; h *= 2; break;
	}
	if(mbFallBackMode)
	{
		int texW = 256;
		int texH = 256;

		while(texW<w)
		{
			texW*=2;
		}
		while(texH<h)
		{
			texH*=2;
		}
		if(!InitializeFBO(mMainFBO,"Main",texW,texH,true,true,true,true,false,GL_TEXTURE_2D,2))
			return;
		if(!InitializeFBO(mMainFBO2,"Main",texW,texH,true,true,true,true,false,GL_TEXTURE_2D,2))
			return;
		if(!InitializeFBO(mFadeFBO,"Fade",texW,texH,true,true,true,true,false,GL_TEXTURE_2D,2))
			return;
		glDisable(GL_TEXTURE_2D);
	}
	else
	{

		if(!InitializeFBO(mMainFBO,"Main",w,h,true,true,true,true,false,GL_TEXTURE_RECTANGLE_ARB,2))
			return;
		if(!InitializeFBO(mMainFBO2,"Main",w,h,true,true,true,true,false,GL_TEXTURE_RECTANGLE_ARB,2))
			return;
		if(!InitializeFBO(mFadeFBO,"Fade",w,h,true,true,true,true,false,GL_TEXTURE_RECTANGLE_ARB,2))
			return;
		glDisable(GL_TEXTURE_RECTANGLE_ARB);
		glDisable(GL_TEXTURE_2D);
	}
	if(!mFont1.LoadFont("Fonts.psd","FontShadow.psd"))
		return;
	UTIL_GL::GL2D::SetOrtho(640,480);
	mFont1.SetAlignment(ALIGN_LEFT);
	mFont1.SetColor(1,1,1,1);
	mFont1.SetScale(0.5f,0.5f);
	mFont1.Print("LOADING..",520,448);
	SDL_GL_SwapBuffers();

	UTIL_GL::GetGLError();

	if(!LoadConstantData())
		return;
	//ClearMap();

	ChangeState(STATE_SPLASH, 0);
	StartSplash();
//	ChangeState(STATE_CREDITS, 0);
//	StartCredits();


	if(bPlayDemo)
	{
//		ChangeState(STATE_GAMESELECT, 0);
//		StartGameSelect();
		if(!LoadBetaRecord(szDemoName,mBetaRecording))
		{
			gLog.OutPut("Failed to load demo: ");
			gLog.OutPut(szDemoName);
			gLog.OutPut("!\n");
		}
	}
	if(bRecordDemo)
	{
		gLog.OutPut("Recording demo: ");
		gLog.OutPut(szDemoName);
		gLog.OutPut("!\n");
	}
	mTimer.Reset();

	Pump();

}
/***********************************/
/* init image libraray			   */
/***********************************/
void Engine::InitDevil()
{
	gLog.OutPut("\n[Initializing Devil]\n");
	ilInit();
	iluInit();
	ilutInit();
	gLog.OutPut("Complete...\n\n");
}
/****************************************************/
/* engine destructor, any last minute cleanup here  */
/****************************************************/
Engine::~Engine()
{
	gLog.OutPut("Engine: Cleanup.\n");
	if(bRecordDemo)
	{
		if(!SaveBetaRecord(szDemoName,mBetaRecording))
		{
			gLog.OutPut("Failed to save beta recording: ");
			gLog.OutPut(szDemoName);
			gLog.OutPut("!\n");
		}
		else
		{
			gLog.OutPut("Saved beta recording: ");
			gLog.OutPut(szDemoName);
			gLog.OutPut("\n");
		}
	}
	mTexMgr.CleanUp();
	CleanupFBO(mFadeFBO);
	CleanupFBO(mMainFBO);
//	SaveInput();
//  Opcode::CloseOpcode();
}
/**************************************************************/
/* Pump, the main loop of the engine. Everything happens here */
/**************************************************************/
void Engine::Pump()
{
	mMouseMotion = Vec2(0,0);
	mTimer.Reset();
	char FPS[32];
	accumulator=0;
	SDL_Event event;
	SDL_EnableUNICODE(0);
	while(!mbQuit)
	{
		mTimer.Update();
		accumulator += mTimer.GetDelta();
		while(accumulator>=TIME_STEP)
		{
			if(!bPlayDemo)
			while(SDL_PollEvent(&event)>0)
			{

				switch(event.type)
				{
					case SDL_KEYDOWN:
						if(bTextInput)
							TextInput(event.key.keysym);
						else
						KeyStateAlltered(event.key.keysym.sym,true);
					break;
					case SDL_KEYUP:
					KeyStateAlltered(event.key.keysym.sym,false);
					break;
					//joystick buttons
					case SDL_JOYBUTTONDOWN:
						{
						//	if(!bJoystickSupported)
						//		break;
							int b = event.jbutton.button;
							if(b<20)
								KeyStateAlltered(JOYSTICK_BUTTON+b, true);				
						}
						break;
					case SDL_JOYBUTTONUP:
						{
						//	if(!bJoystickSupported)
						//		break;
							int b = event.jbutton.button;
							if(b<20)
								KeyStateAlltered(JOYSTICK_BUTTON+b, false);				
						}
						break;//*/
					//joystick motion
					case SDL_JOYAXISMOTION:
					//	if(!bJoystickSupported)
					//		break;
						CheckJoystickMotion(event);
						break;//*/

					case SDL_QUIT:
						mbQuit = true;
					break;

/*					case SDL_MOUSEBUTTONDOWN:
					KeyStateAlltered(event.button.button,true);
					break;
					case SDL_MOUSEBUTTONUP:
					KeyStateAlltered(event.button.button,false);
					break;

				/*	case SDL_MOUSEMOTION:
						mMouseMotion.x = event.motion.xrel;
						mMouseMotion.y = event.motion.yrel;
						break;//*/
				}
			}
			//SDL_GetMouseState(&MouseX,&MouseY);
//	_ASSERTE( _CrtCheckMemory( ) );
			if(bRecordDemo)
				RecordBetaState(mBetaRecording);
			UpdateState();
//	_ASSERTE( _CrtCheckMemory( ) );
			if(bPlayDemo)
				UpdateBetaRecordState(mBetaRecording);
//	_ASSERTE( _CrtCheckMemory( ) );
			(this->*pUpdateFunc)();
			miInternalClock++;
//	_ASSERTE( _CrtCheckMemory( ) );

			KeysPostUpdate();
			accumulator-=TIME_STEP;

			if(bVideoDemo)
			{
				static unsigned int imagecounter=0;
				imagecounter++;
				(this->*pRenderFunc)(1);
				SDL_GL_SwapBuffers();

				glPushMatrix();
				ilEnable(IL_FILE_OVERWRITE);
				ILuint ImageName;
				ilGenImages(1, &ImageName);
				ilBindImage(ImageName);
				ilutGLScreen();
				char file[64];
				sprintf(file,"%s%.8i.jpeg",szDemoName,imagecounter);
				ilSaveImage(file);
				ilDeleteImages(1, &ImageName);
				glPopMatrix();
				SDL_Delay(1);
			}
		}

	//_ASSERTE( _CrtCheckMemory( ) );
		if(!bVideoDemo)
		{
		const float interp = mInterp = accumulator/TIME_STEP;
		(this->*pRenderFunc)(mbPaused || !mMenus.empty()?1:interp);
		//UTIL_GL::GetGLError();
	//_ASSERTE( _CrtCheckMemory( ) );

		if(mbShowFPS)
		{
			UTIL_GL::GL2D::SetOrtho(640,480);
			glDisable(GL_DEPTH_TEST);
			mFont1.DisableShadow();
			mFont1.SetColor(1,1,1,1);
			mFont1.SetScale(0.5f,0.5f);
			mFont1.SetBlendMode(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
			sprintf(FPS,"FPS %i",mTimer.GetFPS());
			mFont1.SetAlignment(ALIGN_LEFT);
			mFont1.Print(FPS,15,5);
			glDisable(GL_BLEND);
		}

		SDL_GL_SwapBuffers();
		SDL_Delay(1);
		}
	}
}

/***********************************************/
/* clear map blocks, initialize vars		   */
/***********************************************/
void Engine::ClearMap()
{
//	memset(cMap,0,sizeof(Block*)*64*64*64);
	for(int X=0;X<64; X++)
	for(int Y=0;Y<64; Y++)
	for(int Z=0;Z<64; Z++)
	{
		cMap[Z*64*64+Y*64+X] = NULL;
	}
	mMapBlocks.clear();
	mSpecialBlocks.clear();
	
//	mPlayer.Spawn(Vec3(8.5f,7.5f,8.5f),Vec2(0,0));

	mfCamZoom = 2;
	mfCamVertical = 2;
	iCameraClipCounter = 0;

	mFrustum.Setup(70,1.3f,0.1f,20);
	mfCloudScroll = mfOCloudScroll = 0;
}
/*****************************************************/
/* Cleanup map data									 */
/*****************************************************/
void Engine::CleanupMap()
{
	mSurfaces.clear();
	mGlowSurfaces.clear();
	mSpecialBlocks.clear();
	mMapBlocks.clear();
	mGameEnts.remove_if(CleanEnts);
	mPartEng.CleanUp();
	std::list<Platform*>::iterator p;
	for(p=mPlatforms.begin(); p!=mPlatforms.end(); p++)
	{
		(*p)->Cleanup();
		delete (*p);
		(*p) = NULL;
	}
	mPlatforms.clear();
}
/*******************************************************/
/* given a point, find a tile that overlaps			   */
/*******************************************************/
Block* Engine::GetMapTile(Vec3 p)
{
	if(p.x<0 || p.x >=MAP_DIMENSIONS)
		return 0;
	if(p.y<0 || p.y >=MAP_DIMENSIONS)
		return 0;
	if(p.z<0 || p.z >=MAP_DIMENSIONS)
		return 0;

	//todo need to test moving platforms here also
	if(!mPlatforms.empty())
	{
		std::list<Platform*>::iterator e;
		for(e=mPlatforms.begin(); e!=mPlatforms.end(); e++)
		{
			if((*e)->PointInPlat(p))
			{
				//TODO get correct tile pos
				Block * b = (*e)->GetTileInPlat(p);
				if(b)
				{
					if(b->bPass)
						return 0;
					if(b->scale>0)
						return b;
				}
			}
		}
	}
	Block* b = cMap[(int)(p.z)*MAP_DIMENSIONS*MAP_DIMENSIONS+(int)(p.y)*MAP_DIMENSIONS+(int)(p.x)];
	if(b)
	{
		if(b->bPass)
			return 0;
		if(b->scale>0)
		return b;
	}
	return 0;
}
/****************************************************/
/* Load data that is constant to the game, and only */
/* needs to be loaded one-time						*/
/****************************************************/
bool Engine::LoadConstantData()
{
	//***pass through shaders***
	if(!LoadShader("pass through", vp_pass, fp_pass, mShdrPass))
		return false;
	if(!LoadShader("pass grey", vp_pass, fp_passGrey, mShdrPassGrey))
		return false;
	if(!LoadShader("rect", vp_pass, fp_passrect, mShdrPassRect))
		return false;
	if(!LoadShader("rect & scale", vp_pass, fp_passrectscale, mShdrPassRectScale))
		return false;
	if(!LoadShader("rect & scale grey", vp_pass, fp_passrectscaleGrey, mShdrPassRectScaleGrey))
		return false;
//		if(!LoadShader("Velocity", vp_pass, fp_Velocity, mShdrVelocity))
//			return false;
	if(!LoadShader("Fur", vp_furpass, fp_Fur, mShdrFur))
		return false;
	if(!mbFallBackMode)
	{
		if(!LoadShader("DOFH", vp_pass, fp_DofH, mShdrDOFH))
			return false;
		if(!LoadShader("DOFV", vp_pass, fp_DofV, mShdrDOFV))
			return false;
		if(!LoadShader("Radial", vp_pass, fp_Radial, mShdrRadial))
			return false;
		if(!LoadShader("FadeFx1", vp_pass, fp_FadeFx1, mShdrFadeFx1))
			return false;
		if(!LoadShader("FadeFx2", vp_pass, fp_FadeFx2, mShdrFadeFx2))
			return false;
		if(!LoadShader("DistortRect", vp_pass, fp_Distortrect, mShdrDistortRect))
			return false;
//		if(!LoadShader("Super2x", vp_pass, fp_SuperSampleRect2x, mShdrSuperSample2xRect))
//			return false;
	}
	else
	{
		if(!LoadShader("FadeFx1", vp_pass, fp_FadeFx1Fallback, mShdrFadeFx1))
			return false;
		if(!LoadShader("FadeFx2", vp_pass, fp_FadeFx2FallBack, mShdrFadeFx2))
			return false;
		if(!LoadShader("Distort", vp_pass, fp_Distort, mShdrDistort))
			return false;
	}

	if(!LoadShader("lighting", vp_ppdirlight, fp_ppdirlight, mShdrPPDirLight))
		return false;
	if(!LoadShader("lightLerp", vp_ppdirlightLerp, fp_ppdirlight, mShdrPPDirLightLerp))
		return false;
	if(!LoadShader("Ball light", vp_pBallLight, fp_pBallLight, mShdrPBallLight))
		return false;
	if(!LoadShader("PassLerp", vp_passLerp, fp_colorpass, mShdrPassLerp))
		return false;


	unsigned int ID = 0;

	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX","UI.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mSprites[SPR_UI].Split(ID,4,8,256,512);
	if(!(mGfxImg[GFX_LOADING1] = mTexMgr.LoadTexture(true,"Data/GFX","Loading1.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	
	RenderLoadingBar(0);
	if(!(mGfxImg[GFX_HEART] = mTexMgr.LoadTexture(true,"Data/GFX","Heart.tga",GL_NEAREST, GL_NEAREST, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
		return false;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX","Clock.tga",GL_NEAREST, GL_NEAREST, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
		return false;
	mSprites[SPR_CLOCK].Split(ID,2,1,256,128);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX","ClockDigits.tga",GL_NEAREST, GL_NEAREST, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
		return false;
	mSprites[SPR_CLOCK_DIGITS].Split(ID,4,3,512,512);


	if(!mObjects[MDL_SPHERE].Load("Data/Models","ball.obj",1.0))
		return false;
	if(!mObjects[MDL_HALFBALL].Load("Data/Models","halfball.obj",1.0))
		return false;
	if(!mObjects[MDL_SPHERE2].Load("Data/Models","ball2.obj",1.0))
		return false;
	if(!mObjects[MDL_STICKER].Load("Data/Models","ballSticker.obj",1.0))
		return false;
	if(!mObjects[MDL_BALL].Load("Data/Models","Sphere.obj",1.0))
		return false;
	if(!mObjects[MDL_GEM].Load("Data/Models","Gem.obj",1.0))
		return false;
	if(!mObjects[MDL_SPIKES].Load("Data/Models","Spikes.obj",3.0))
		return false;
	if(!mObjects[MDL_STAR].Load("Data/Models","Star.obj",2.5))
		return false;
	if(!mObjects[MDL_CLOCK].Load("Data/Models","Clock.obj",1.25))
		return false;

	RenderLoadingBar(10);

	char szObj[64];
	for(int n=1; n<17; n++)
	{
		sprintf(szObj,"Slinky_%.6i.obj",n);
		if(!mObjects[MDL_SLINKY01+(n-1)].Load("Data/Models",szObj,2.75))
			return false;
		mObjects[MDL_SLINKY01+(n-1)].BuildVertexArray();
	RenderLoadingBar(10+ (2*n));
	}
	RenderLoadingBar(50);

	if(!mObjects[MDL_ARROW].Load("Data/Models","Arrow.obj",2.5))
		return false;
	if(!mObjects[MDL_ROTORBLADE].Load("Data/Models","RotorBlades.obj",1.5))
		return false;
	if(!mObjects[MDL_1UP].Load("Data/Models","1Up.obj",0.75f))
		return false;
	RenderLoadingBar(60);
	if(!mObjects[MDL_BUTTON].Load("Data/Models","Button.obj",0.5f))
		return false;
	if(!mObjects[MDL_BUTTON_RIM].Load("Data/Models","ButtonRim.obj",0.5f))
		return false;
	if(!mObjects[MDL_TELEPORT].Load("Data/Models","Teleport.obj",3.0f))
		return false;
	if(!mObjects[MDL_TELEPORT_PAD].Load("Data/Models","TeleportPad.obj",3.0f))
		return false;
	RenderLoadingBar(70);
	if(!mObjects[MDL_LASER].Load("Data/Models","Laser.obj",2.0f))
		return false;
	if(!mObjects[MDL_LASER_GLOW].Load("Data/Models","LaserGlow.obj",2.0f))
		return false;
	mObjects[MDL_SPHERE].BuildVertexArray();
	mObjects[MDL_SPHERE2].BuildVertexArray();
	mObjects[MDL_STICKER].BuildVertexArray();
	mObjects[MDL_GEM].BuildVertexArray();
	mObjects[MDL_BALL].BuildVertexArray();
	mObjects[MDL_HALFBALL].BuildVertexArray();
	mObjects[MDL_SPIKES].BuildVertexArray();
	mObjects[MDL_STAR].BuildVertexArray();
	mObjects[MDL_CLOCK].BuildVertexArray();
	mObjects[MDL_ARROW].BuildVertexArray();
	mObjects[MDL_ROTORBLADE].BuildVertexArray();
	mObjects[MDL_1UP].BuildVertexArray();
	mObjects[MDL_BUTTON].BuildVertexArray();
	mObjects[MDL_BUTTON_RIM].BuildVertexArray();
	mObjects[MDL_TELEPORT].BuildVertexArray();
	mObjects[MDL_TELEPORT_PAD].BuildVertexArray();
	mObjects[MDL_LASER].BuildVertexArray();
	mObjects[MDL_LASER_GLOW].BuildVertexArray();


	RenderLoadingBar(80);

	/********************/
	/* BALL GFX			*/
	/********************/
	mBallTypes[0].iColLayers.push_back(0);
	mBallTypes[0].iColLayers.push_back(20);
	mBallTypes[0].iColLayers.push_back(5);
	mBallTypes[0].iColLayers.push_back(30);
	mBallTypes[0].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball1_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[0].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball1_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[0].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball1_3.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[0].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball1_4.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[0].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball1_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[0].idBump = ID;

	mBallTypes[1].iColLayers.push_back(20);
	mBallTypes[1].iColLayers.push_back(0);
	mBallTypes[1].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball2_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[1].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball2_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[1].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball2_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[1].idBump = ID;

	mBallTypes[2].iColLayers.push_back(10);
	mBallTypes[2].iColLayers.push_back(30);
	mBallTypes[2].iMdlType = MDL_SPHERE2;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball3_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[2].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball3_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[2].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball3_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[2].idBump = ID;

	RenderLoadingBar(85);

	mBallTypes[3].iColLayers.push_back(20);
	mBallTypes[3].iColLayers.push_back(5);
	mBallTypes[3].iColLayers.push_back(15);
	mBallTypes[3].iColLayers.push_back(25);
	mBallTypes[3].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball4_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[3].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball4_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[3].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball4_3.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[3].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball4_4.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[3].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball4_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[3].idBump = ID;

	mBallTypes[4].iColLayers.push_back(19);
	mBallTypes[4].iColLayers.push_back(1);
	mBallTypes[4].iColLayers.push_back(20);
	mBallTypes[4].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball5_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[4].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball5_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[4].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball5_3.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[4].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball5_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[4].idBump = ID;

	mBallTypes[5].iColLayers.push_back(10);
	mBallTypes[5].iColLayers.push_back(20);
	mBallTypes[5].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball6_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[5].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball6_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[5].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball6_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[5].idBump = ID;

	mBallTypes[6].iColLayers.push_back(4);
	mBallTypes[6].iColLayers.push_back(0);
	mBallTypes[6].iColLayers.push_back(20);
	mBallTypes[6].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball7_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[6].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball7_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[6].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball7_3.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[6].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball7_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[6].idBump = ID;

	RenderLoadingBar(90);

	mBallTypes[7].iColLayers.push_back(20);
	mBallTypes[7].iColLayers.push_back(34);
	mBallTypes[7].iColLayers.push_back(1);
	mBallTypes[7].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball8_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[7].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball8_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[7].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball8_3.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[7].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball8_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[7].idBump = ID;

	mBallTypes[8].iColLayers.push_back(0);
	mBallTypes[8].iColLayers.push_back(10);
	mBallTypes[8].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball9_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[8].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball9_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[8].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball9_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[8].idBump = ID;

	mBallTypes[9].iColLayers.push_back(5);
	mBallTypes[9].iColLayers.push_back(30);
	mBallTypes[9].iMdlType = MDL_SPHERE;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball10_1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[9].iLayers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball10_2.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mBallTypes[9].iLayers.push_back(ID);
//	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX/Balls","Ball10_nm.tga",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//		return false;
//	mBallTypes[9].idBump = ID;
	RenderLoadingBar(88);

	if(!(MenuImgs[GFXMENU_CUSTOMIZE] = mTexMgr.LoadTexture(true,"Data/GFX","Customize.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_GETREADY] = mTexMgr.LoadTexture(true,"Data/GFX","GetReady.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_FALLOUT] = mTexMgr.LoadTexture(true,"Data/GFX","FallOut.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_FAIL] = mTexMgr.LoadTexture(true,"Data/GFX","LostBall.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_TIMEOUT] = mTexMgr.LoadTexture(true,"Data/GFX","TimeOut.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_CLEAR] = mTexMgr.LoadTexture(true,"Data/GFX","AreaClear.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_GAMEOVER] = mTexMgr.LoadTexture(true,"Data/GFX","GameOver.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_PERFECT] = mTexMgr.LoadTexture(true,"Data/GFX","Perfect.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_WORLD1] = mTexMgr.LoadTexture(true,"Data/GFX","SlotWorld1.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_WORLD2] = mTexMgr.LoadTexture(true,"Data/GFX","SlotWorld2.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_WORLD3] = mTexMgr.LoadTexture(true,"Data/GFX","SlotWorld3.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_WORLD4] = mTexMgr.LoadTexture(true,"Data/GFX","SlotWorld4.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_WORLD5] = mTexMgr.LoadTexture(true,"Data/GFX","SlotWorld5.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_BONUS_STAGE] = mTexMgr.LoadTexture(true,"Data/GFX","BonusStage.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(MenuImgs[GFXMENU_SPLASH] = mTexMgr.LoadTexture(true,"Data/GFX","Screwball.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX","Particles.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mSprites[SPR_PARTICLES].Split(ID,4,1,256,64);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX","ParticlesAlpha.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mSprites[SPR_PARTICLESALPHA].Split(ID,4,1,256,64);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/GFX","UIMenu.tga",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
		return false;
	mSprites[SPR_MENU].Split(ID,8,8,256,256);

	if(!(mGfxImg[GFX_BONUSTILES_NM] = mTexMgr.LoadTexture(true,"Data/GFX","BonusTiles_nm.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(mGfxImg[GFX_BONUSTILES] = mTexMgr.LoadTexture(true,"Data/GFX","BonusTiles.png",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mSprites[SPR_BONUSTILES].Split(mGfxImg[GFX_BONUSTILES],8,2,1024,128);
	if(!(mGfxImg[GFX_GRID] = mTexMgr.LoadTexture(true,"Data/GFX","Grid.png",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	if(!(mGfxImg[GFX_BUSH1] = mTexMgr.LoadTexture(true,"Data/GFX","SplashBush1.png",GL_NEAREST, GL_NEAREST, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
		return false;
	if(!(mGfxImg[GFX_BUSH2] = mTexMgr.LoadTexture(true,"Data/GFX","SplashBush2.png",GL_NEAREST, GL_NEAREST, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
		return false;
	if(!(mGfxImg[GFX_BUSH3] = mTexMgr.LoadTexture(true,"Data/GFX","SplashBush3.png",GL_NEAREST, GL_NEAREST, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
		return false;
	if(!(mGfxImg[GFX_STAR_GLOW] = mTexMgr.LoadTexture(true,"Data/GFX","StarGlow.png",GL_NEAREST, GL_NEAREST, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
		return false;



	//************** Materials *******************//
	if(!(mMaterials[MAT_FLAT] = mMatMgr.LoadMaterial("Data/Materials","Flat")))
		return false;
	if(!(mMaterials[MAT_WORLD1] = mMatMgr.LoadMaterial("Data/Materials","World1")))
		return false;
	if(!(mMaterials[MAT_WORLD2] = mMatMgr.LoadMaterial("Data/Materials","World2")))
		return false;
	if(!(mMaterials[MAT_WORLD3] = mMatMgr.LoadMaterial("Data/Materials","World3")))
		return false;
	if(!(mMaterials[MAT_WORLD4] = mMatMgr.LoadMaterial("Data/Materials","World4")))
		return false;
	if(!(mMaterials[MAT_WORLD5] = mMatMgr.LoadMaterial("Data/Materials","World5")))
		return false;
	if(!(mMaterials[MAT_BONUS] = mMatMgr.LoadMaterial("Data/Materials","Bonus")))
		return false;
	if(!(mMaterials[MAT_TELEPORT] = mMatMgr.LoadMaterial("Data/Materials","Teleport")))
		return false;

	//hud colors, rainbow gem
	mHudColors.push_back(Vec3(1,0,0));
	mHudColors.push_back(Vec3(1,1,0));
	mHudColors.push_back(Vec3(0,1,0));
	mHudColors.push_back(Vec3(0,1,1));
	mHudColors.push_back(Vec3(0,0,1));
	mHudColors.push_back(Vec3(1,0,1));

	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker1.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker2.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker3.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker4.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker5.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker6.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker7.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker8.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker9.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker10.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker11.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker12.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker13.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker14.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);
	if(!(ID = mTexMgr.LoadTexture(true,"Data/Stickers","Sticker15.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return false;
	mStickers.push_back(ID);

	RenderLoadingBar(90);

	if(!LoadWorldData())
		return false;

	RenderLoadingBar(92);

	if(!LoadSample("Data/Sound/BallClip.wav",mSnd[SND_BALL_CLIP]))
		return false;
	if(!LoadSample("Data/Sound/Jump1.wav",mSnd[SND_BALL_JUMP]))
		return false;
	if(!LoadSample("Data/Sound/Jump2.wav",mSnd[SND_BALL_JUMP_PAD]))
		return false;
	if(!LoadSample("Data/Sound/BallExplode.wav",mSnd[SND_BALL_EXPLODE]))
		return false;
	if(!LoadSample("Data/Sound/Cheering.wav",mSnd[SND_CHEER]))
		return false;
	if(!LoadSample("Data/Sound/UI_Bounce.wav",mSnd[SND_UI_BOUNCE]))
		return false;
	if(!LoadSample("Data/Sound/UI_Move.wav",mSnd[SND_UI_MOVE]))
		return false;
	if(!LoadSample("Data/Sound/UI_Select.wav",mSnd[SND_UI_SELECT]))
		return false;
	if(!LoadSample("Data/Sound/UI_Select2.wav",mSnd[SND_UI_SELECT2]))
		return false;
	if(!LoadSample("Data/Sound/UI_Backup.wav",mSnd[SND_UI_BACKUP]))
		return false;
	if(!LoadSample("Data/Sound/UI_Whoosh.wav",mSnd[SND_UI_WHOOSH]))
		return false;
	if(!LoadSample("Data/Sound/UI_Whoosh2.wav",mSnd[SND_UI_WHOOSH2]))
		return false;
	if(!LoadSample("Data/Sound/UI_Whoosh3.wav",mSnd[SND_UI_WHOOSH3]))
		return false;
	if(!LoadSample("Data/Sound/Fade1.wav",mSnd[SND_FADE1]))
		return false;
	if(!LoadSample("Data/Sound/Fade2.wav",mSnd[SND_FADE2]))
		return false;
	RenderLoadingBar(94);

	if(!LoadSample("Data/Sound/GetReady.wav",mSnd[SND_VOICE_GETREADY]))
		return false;
	if(!LoadSample("Data/Sound/Go.wav",mSnd[SND_VOICE_GO]))
		return false;
	if(!LoadSample("Data/Sound/Ten.wav",mSnd[SND_VOICE_TEN]))
		return false;
	if(!LoadSample("Data/Sound/Nine.wav",mSnd[SND_VOICE_NINE]))
		return false;
	if(!LoadSample("Data/Sound/Eight.wav",mSnd[SND_VOICE_EIGHT]))
		return false;
	if(!LoadSample("Data/Sound/Seven.wav",mSnd[SND_VOICE_SEVEN]))
		return false;
	if(!LoadSample("Data/Sound/Six.wav",mSnd[SND_VOICE_SIX]))
		return false;
	if(!LoadSample("Data/Sound/Five.wav",mSnd[SND_VOICE_FIVE]))
		return false;
	if(!LoadSample("Data/Sound/Four.wav",mSnd[SND_VOICE_FOUR]))
		return false;
	if(!LoadSample("Data/Sound/Three.wav",mSnd[SND_VOICE_THREE]))
		return false;
	if(!LoadSample("Data/Sound/Two.wav",mSnd[SND_VOICE_TWO]))
		return false;
	if(!LoadSample("Data/Sound/One.wav",mSnd[SND_VOICE_ONE]))
		return false;
	RenderLoadingBar(96);

	if(!LoadSample("Data/Sound/LostBall.wav",mSnd[SND_VOICE_LOSTBALL]))
		return false;
	if(!LoadSample("Data/Sound/DropOut.wav",mSnd[SND_VOICE_DROPOUT]))
		return false;
	if(!LoadSample("Data/Sound/BonusStage.wav",mSnd[SND_VOICE_BONUS_STAGE]))
		return false;
	if(!LoadSample("Data/Sound/Teleport1.wav",mSnd[SND_TELEPORT1]))
		return false;
	if(!LoadSample("Data/Sound/Teleport2.wav",mSnd[SND_TELEPORT2]))
		return false;
	if(!LoadSample("Data/Sound/Button1.wav",mSnd[SND_BUTTON]))
		return false;
	if(!LoadSample("Data/Sound/Pickup2.wav",mSnd[SND_PICKUP1]))
		return false;
	if(!LoadSample("Data/Sound/Pickup1.wav",mSnd[SND_PICKUP2]))
		return false;
	if(!LoadSample("Data/Sound/Pop.wav",mSnd[SND_BALL_POP]))
		return false;
	if(!LoadSample("Data/Sound/Spikes.wav",mSnd[SND_SPIKES]))
		return false;
	if(!LoadSample("Data/Sound/Crumble.wav",mSnd[SND_CRUMBLE]))
		return false;
	if(!LoadSample("Data/Sound/Arrow.wav",mSnd[SND_ARROW]))
		return false;
	if(!LoadSample("Data/Sound/HitSound1.wav",mSnd[SND_HIT]))
		return false;

	RenderLoadingBar(98);

	if(!LoadMusic("Data/Sound/Selection.ogg",mAltMainMusic))
		return false;
	if(!LoadMusic("Data/Sound/Clock.ogg",mClockMusic))
		return false;
	if(!LoadMusic("Data/Sound/AreaCleared.ogg",mClearedMusic))
		return false;
	if(!LoadMusic("Data/Sound/Bonus.ogg",mBonusMusic))
		return false;
	if(!LoadMusic("Data/Sound/Track_2.ogg",mSplashMusic))
		return false;

	RenderLoadingBar(100);

	return true;
}
/*********************************************/
/* load a shader						     */
/*********************************************/
bool Engine::LoadShader(char* name, char* szVp, char* szFp, int& outShdr)
{
	gLog.OutPut("Compilling shader: ");
	gLog.OutPut(name);
	gLog.OutPut(".\n");
	unsigned int vp,fp;
	if(!(fp = mShdrMgr.CreateShader(GL_FRAGMENT_SHADER_ARB,szFp)))
		return false;
	if(!(vp = mShdrMgr.CreateShader(GL_VERTEX_SHADER_ARB,szVp)))
		return false;
	outShdr = mShdrMgr.CreateProgram();
	mShdrMgr.AttachShader(vp);
	mShdrMgr.AttachShader(fp);
	mShdrMgr.LinkProgram();
	mShdrMgr.BindProgram(0);
	return true;
}
/*************************************/
/* Load world specific data here	 */
/*************************************/
extern bool DEMO_BUILD;
bool Engine::LoadWorldData()
{
	for(int x=0; x<4; x++)
		for(int y=0; y<6; y++)
			imgSkyBox[x][y] = 0;

	if(DEMO_BUILD)
	{
		miWorld = miOWorld = 0;
	}

	//FreeMusic(mMainMusic);
	if(miWorld == 0)//candy land
	{
		if(!(mGfxImg[GFX_TILES_NM] = mTexMgr.LoadTexture(true,"Data/GFX","Tiles_nm.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","Tiles.png",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_TILES].Split(mGfxImg[GFX_TILES],8,8,1024,1024);
		if(!(mGfxImg[GFX_FUR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W1FurTiles.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_FUR_VECTOR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W1FurTilesVector.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_FUR_TILES].Split(mGfxImg[GFX_FUR_VECTOR_TILES],8,8,1024,1024);

		if(!(imgSkyBox[0][0] = mTexMgr.LoadTexture(false,"Data/GFX","W1L1D.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[0][1] = mTexMgr.LoadTexture(false,"Data/GFX","W1L1F.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[0][2] = mTexMgr.LoadTexture(false,"Data/GFX","W1L1R.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[0][3] = mTexMgr.LoadTexture(false,"Data/GFX","W1L1B.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[0][4] = mTexMgr.LoadTexture(false,"Data/GFX","W1L1L.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][1] = mTexMgr.LoadTexture(false,"Data/GFX","W1L2F.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][2] = mTexMgr.LoadTexture(false,"Data/GFX","W1L2R.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][3] = mTexMgr.LoadTexture(false,"Data/GFX","W1L2B.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][4] = mTexMgr.LoadTexture(false,"Data/GFX","W1L2L.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][1] = mTexMgr.LoadTexture(false,"Data/GFX","W1L3F.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][2] = mTexMgr.LoadTexture(false,"Data/GFX","W1L3R.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][3] = mTexMgr.LoadTexture(false,"Data/GFX","W1L3B.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][4] = mTexMgr.LoadTexture(false,"Data/GFX","W1L3L.png",GL_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][0] = mTexMgr.LoadTexture(false,"Data/GFX","W1L4D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][1] = mTexMgr.LoadTexture(false,"Data/GFX","W1L4F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][2] = mTexMgr.LoadTexture(false,"Data/GFX","W1L4R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][3] = mTexMgr.LoadTexture(false,"Data/GFX","W1L4B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][4] = mTexMgr.LoadTexture(false,"Data/GFX","W1L4L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][5] = mTexMgr.LoadTexture(false,"Data/GFX","W1L4U.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[0][5] = mTexMgr.LoadTexture(false,"Data/GFX","Clouds1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
			return false;

		if(!LoadMusic("Data/Sound/Track_1.ogg",mMainMusic))
			return false;
	}

	if(miWorld == 1)//forest
	{
		if(!(mGfxImg[GFX_TILES_NM] = mTexMgr.LoadTexture(true,"Data/GFX","W2Tiles_nm.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W2Tiles.png",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_TILES].Split(mGfxImg[GFX_TILES],8,8,1024,1024);
		if(!(mGfxImg[GFX_FUR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W2FurTiles.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_FUR_VECTOR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W1FurTilesVector.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_FUR_TILES].Split(mGfxImg[GFX_FUR_VECTOR_TILES],8,8,1024,1024);

		if(!(imgSkyBox[1][0] = mTexMgr.LoadTexture(false,"Data/GFX","W2L2D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][1] = mTexMgr.LoadTexture(false,"Data/GFX","W2L2F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][2] = mTexMgr.LoadTexture(false,"Data/GFX","W2L2R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][3] = mTexMgr.LoadTexture(false,"Data/GFX","W2L2B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][4] = mTexMgr.LoadTexture(false,"Data/GFX","W2L2L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;

		if(!(imgSkyBox[2][1] = mTexMgr.LoadTexture(false,"Data/GFX","W2L3F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][3] = mTexMgr.LoadTexture(false,"Data/GFX","W2L3B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][0] = mTexMgr.LoadTexture(false,"Data/GFX","W2L4D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][1] = mTexMgr.LoadTexture(false,"Data/GFX","W2L4F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][2] = mTexMgr.LoadTexture(false,"Data/GFX","W2L4R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][3] = mTexMgr.LoadTexture(false,"Data/GFX","W2L4B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][4] = mTexMgr.LoadTexture(false,"Data/GFX","W2L4L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][5] = mTexMgr.LoadTexture(false,"Data/GFX","W2L4U.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;

		if(!LoadMusic("Data/Sound/Track_1.ogg",mMainMusic))
			return false;
	}
	if(miWorld == 2)//egyptian
	{
		if(!(mGfxImg[GFX_TILES_NM] = mTexMgr.LoadTexture(true,"Data/GFX","W3Tiles_nm.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W3Tiles.png",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_TILES].Split(mGfxImg[GFX_TILES],8,8,1024,1024);
		if(!(mGfxImg[GFX_FUR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W3FurTiles.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_FUR_VECTOR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W1FurTilesVector.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_FUR_TILES].Split(mGfxImg[GFX_FUR_VECTOR_TILES],8,8,1024,1024);

		if(!(imgSkyBox[1][1] = mTexMgr.LoadTexture(false,"Data/GFX","W3L1F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][2] = mTexMgr.LoadTexture(false,"Data/GFX","W3L1R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][3] = mTexMgr.LoadTexture(false,"Data/GFX","W3L1B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;

		if(!(imgSkyBox[2][0] = mTexMgr.LoadTexture(false,"Data/GFX","W3L1D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][1] = mTexMgr.LoadTexture(false,"Data/GFX","W3L2F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][2] = mTexMgr.LoadTexture(false,"Data/GFX","W3L2R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][3] = mTexMgr.LoadTexture(false,"Data/GFX","W3L2B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][4] = mTexMgr.LoadTexture(false,"Data/GFX","W3L2L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;

		if(!(imgSkyBox[3][0] = mTexMgr.LoadTexture(false,"Data/GFX","W3L3D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][1] = mTexMgr.LoadTexture(false,"Data/GFX","W3L3F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][2] = mTexMgr.LoadTexture(false,"Data/GFX","W3L3R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][3] = mTexMgr.LoadTexture(false,"Data/GFX","W3L3B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][4] = mTexMgr.LoadTexture(false,"Data/GFX","W3L3L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][5] = mTexMgr.LoadTexture(false,"Data/GFX","W3L3U.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!LoadMusic("Data/Sound/Track_1.ogg",mMainMusic))
			return false;
	}
	if(miWorld == 3)//snow
	{
		if(!(mGfxImg[GFX_TILES_NM] = mTexMgr.LoadTexture(true,"Data/GFX","W4Tiles_nm.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W4Tiles.png",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_TILES].Split(mGfxImg[GFX_TILES],8,8,1024,1024);
		if(!(mGfxImg[GFX_FUR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W3FurTiles.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_FUR_VECTOR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W1FurTilesVector.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_FUR_TILES].Split(mGfxImg[GFX_FUR_VECTOR_TILES],8,8,1024,1024);

		if(!(imgSkyBox[1][0] = mTexMgr.LoadTexture(false,"Data/GFX","W4L1D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][1] = mTexMgr.LoadTexture(false,"Data/GFX","W4L1F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][2] = mTexMgr.LoadTexture(false,"Data/GFX","W4L1R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][3] = mTexMgr.LoadTexture(false,"Data/GFX","W4L1B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[1][4] = mTexMgr.LoadTexture(false,"Data/GFX","W4L1L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;

		if(!(imgSkyBox[2][0] = mTexMgr.LoadTexture(false,"Data/GFX","W4L2D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][1] = mTexMgr.LoadTexture(false,"Data/GFX","W4L2F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][2] = mTexMgr.LoadTexture(false,"Data/GFX","W4L2R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][3] = mTexMgr.LoadTexture(false,"Data/GFX","W4L2B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[2][4] = mTexMgr.LoadTexture(false,"Data/GFX","W4L2L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;

		if(!(imgSkyBox[3][0] = mTexMgr.LoadTexture(false,"Data/GFX","W4L3D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][1] = mTexMgr.LoadTexture(false,"Data/GFX","W4L3F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][2] = mTexMgr.LoadTexture(false,"Data/GFX","W4L3R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][3] = mTexMgr.LoadTexture(false,"Data/GFX","W4L3B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][4] = mTexMgr.LoadTexture(false,"Data/GFX","W4L3L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][5] = mTexMgr.LoadTexture(false,"Data/GFX","W4L3U.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[0][5] = mTexMgr.LoadTexture(false,"Data/GFX","Clouds1.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!LoadMusic("Data/Sound/Track_1.ogg",mMainMusic))
			return false;
	}
	if(miWorld == 4)//space
	{
		if(!(mGfxImg[GFX_TILES_NM] = mTexMgr.LoadTexture(true,"Data/GFX","W5Tiles.tga",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W5Tiles.png",GL_NEAREST_MIPMAP_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_TILES].Split(mGfxImg[GFX_TILES],8,8,1024,1024);
		if(!(mGfxImg[GFX_FUR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W3FurTiles.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		if(!(mGfxImg[GFX_FUR_VECTOR_TILES] = mTexMgr.LoadTexture(true,"Data/GFX","W1FurTilesVector.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return false;
		mSprites[SPR_FUR_TILES].Split(mGfxImg[GFX_FUR_VECTOR_TILES],8,8,1024,1024);

		if(!(imgSkyBox[3][0] = mTexMgr.LoadTexture(false,"Data/GFX","W5L1D.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][1] = mTexMgr.LoadTexture(false,"Data/GFX","W5L1F.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][2] = mTexMgr.LoadTexture(false,"Data/GFX","W5L1R.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][3] = mTexMgr.LoadTexture(false,"Data/GFX","W5L1B.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][4] = mTexMgr.LoadTexture(false,"Data/GFX","W5L1L.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!(imgSkyBox[3][5] = mTexMgr.LoadTexture(false,"Data/GFX","W5L1U.png",GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE)))
			return false;
		if(!LoadMusic("Data/Sound/Track_1.ogg",mMainMusic))
			return false;
	}
	return true;
}
/**********************************/
/* clean up world specific data   */
/**********************************/
void Engine::CleanupWorldData()
{
	gLog.OutPut("Cleaning World Data.\n");
	if(miWorld == 0 || miWorld == 1 || miWorld == 2)
	{
		char texname[64];
		gLog.OutPut("Clearing TILES NM\n");
		if(mTexMgr.GetTextureName(mGfxImg[GFX_TILES_NM],texname))
			mTexMgr.FreeTexture(texname);
		gLog.OutPut("Clearing TILES\n");
		if(mTexMgr.GetTextureName(mGfxImg[GFX_TILES],texname))
			mTexMgr.FreeTexture(texname);
		gLog.OutPut("Clearing TILES FUR\n");
		if(mTexMgr.GetTextureName(mGfxImg[GFX_FUR_TILES],texname))
			mTexMgr.FreeTexture(texname);
		gLog.OutPut("Clearing TILES VECTOR\n");
		if(mTexMgr.GetTextureName(mGfxImg[GFX_FUR_VECTOR_TILES],texname))
			mTexMgr.FreeTexture(texname);
		gLog.OutPut("Clearing skybox\n");
		for(int x=0; x<4; x++)
			for(int y=0; y<6; y++)
				if(mTexMgr.GetTextureName(imgSkyBox[x][y],texname))
					mTexMgr.FreeTexture(texname);
	}
	gLog.OutPut("Cleared!\n");
}
/************************************/
/* render a cube tile				*/
/************************************/
void Engine::RenderCube(Block b, Vec3 pos, Vec3 surfTranslate, float scale, GLint AtLocmins, GLint AtLocmaxs)
{
	Vec2 st = mSprites[SPR_TILES].GetTileUV(b.Faces[0]);
	Vec2 stS = mSprites[SPR_TILES].mUVsize;
	if(bBonusStage)
	{
		st = mSprites[SPR_BONUSTILES].GetTileUV(b.Faces[0]);
		stS = mSprites[SPR_BONUSTILES].mUVsize;
	}
	Vec3 p = pos;//- Vec3(0.5, 0.5, 0.5);
	 float s = scale*0.5f;
	//back surf
	 Block* b2=GetMapTile(pos+Vec3(0,0,-0.51));
	if(b2==0 || (int)b2->pos.x != (int)pos.x || (int)b2->pos.y != (int)pos.y || b.iSpecial == SURF_BREAK || b2->iSpecial == SURF_BREAK || b2->mbPlatform)//|| b2->iSpecial == SURF_ICE && b.iSpecial != SURF_ICE)
	{
		if(b.Faces[0] >= SURF_FUR && mbFUR)
		{
			mSurfaces.push_back(CreateSurface(Vec3(p.x+s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z-s)+surfTranslate,
				Vec3(0,0,-1), b.Faces[0]-SURF_FUR, DRAW_FUR, Distance(mCameraPos,p+surfTranslate+Vec3(0,0,-0.5f))));
		}
		else if(b.Faces[0] == SURF_JUMP || b.Faces[0] == SURF_JUMP+2 || b.iColortime>0)
		{
			Surface C = CreateSurface(Vec3(p.x+s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z-s)+surfTranslate,
				Vec3(0,0,-1), b.Faces[0]+1, DRAW_GLOW, Distance(mCameraPos,p+surfTranslate+Vec3(0,0,-0.5f)));
			C.RampCol = b.iColortime;
			mGlowSurfaces.push_back(C);
		}
	mShdrMgr.PassAttrib2fv(AtLocmins, st.v);
	mShdrMgr.PassAttrib2fv(AtLocmaxs, Vec2(st+stS).v);
	glNormal3f(-0.25f,-0.25f,-1);
	glTexCoord2f(st.x,st.y);glVertex3f(p.x+s,p.y-s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y);glVertex3f(p.x-s,p.y-s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y+stS.y);glVertex3f(p.x-s,p.y+s,p.z-s);
	glTexCoord2f(st.x,st.y+stS.y);glVertex3f(p.x+s,p.y+s,p.z-s);
	}
	//front surf
	b2=GetMapTile(pos+Vec3(0,0,0.51));
	if(b2==0 || (int)b2->pos.x != (int)pos.x || (int)b2->pos.y != (int)pos.y || b.iSpecial == SURF_BREAK || b2->iSpecial == SURF_BREAK  || b2->mbPlatform)//|| b2->iSpecial == SURF_ICE && b.iSpecial != SURF_ICE)
	{
		if(b.Faces[1] >= SURF_FUR && mbFUR)
		{
			mSurfaces.push_back(CreateSurface(Vec3(p.x-s,p.y-s,p.z+s)+surfTranslate, Vec3(p.x+s,p.y-s,p.z+s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z+s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z+s)+surfTranslate,
				Vec3(0,0,1), b.Faces[1]-SURF_FUR, DRAW_FUR, Distance(mCameraPos,p+surfTranslate+Vec3(0,0,0.5f))));
		}
		else if(b.Faces[1] == SURF_JUMP || b.Faces[1] == SURF_JUMP+2 || b.iColortime>0)
		{
			Surface C = CreateSurface(Vec3(p.x-s,p.y-s,p.z+s)+surfTranslate, Vec3(p.x+s,p.y-s,p.z+s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z+s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z+s)+surfTranslate,
				Vec3(0,0,1), b.Faces[1]+1, DRAW_GLOW, Distance(mCameraPos,p+surfTranslate+Vec3(0,0,0.5f)));
			C.RampCol = b.iColortime;
			mGlowSurfaces.push_back(C);
		}
		bBonusStage?
			st = mSprites[SPR_BONUSTILES].GetTileUV(b.Faces[1]):
		st = mSprites[SPR_TILES].GetTileUV(b.Faces[1]);
	mShdrMgr.PassAttrib2fv(AtLocmins, st.v);
	mShdrMgr.PassAttrib2fv(AtLocmaxs, Vec2(st+stS).v);
	glNormal3f(-0.25f,-0.25f,1);
	glTexCoord2f(st.x,st.y);glVertex3f(p.x-s,p.y-s,p.z+s);
	glTexCoord2f(st.x+stS.x,st.y);glVertex3f(p.x+s,p.y-s,p.z+s);
	glTexCoord2f(st.x+stS.x,st.y+stS.y);glVertex3f(p.x+s,p.y+s,p.z+s);
	glTexCoord2f(st.x,st.y+stS.y);glVertex3f(p.x-s,p.y+s,p.z+s);
	}
	//top surf
	b2=GetMapTile(pos+Vec3(0,-0.51,0));
	if(b2==0 || (int)b2->pos.x != (int)pos.x || (int)b2->pos.z != (int)pos.z || b.iSpecial == SURF_BREAK || b2->iSpecial == SURF_BREAK || b2->mbPlatform)// || b2->iSpecial == SURF_ICE && b.iSpecial != SURF_ICE)
	{
		if(b.Faces[2] >= SURF_FUR && mbFUR)
		{
			mSurfaces.push_back(CreateSurface(Vec3(p.x-s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y-s,p.z+s)+surfTranslate, Vec3(p.x-s,p.y-s,p.z+s)+surfTranslate,
				Vec3(0,-1,0), b.Faces[2]-SURF_FUR, DRAW_FUR, Distance(mCameraPos,p+surfTranslate+Vec3(0,-0.5f,0))));
		}
		else if(b.Faces[2] == SURF_JUMP || b.Faces[2] == SURF_JUMP+2 || b.iColortime>0)
		{
			Surface C = CreateSurface(Vec3(p.x-s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y-s,p.z+s)+surfTranslate, Vec3(p.x-s,p.y-s,p.z+s)+surfTranslate,
				Vec3(0,-1,0), b.Faces[2]+1, DRAW_GLOW, Distance(mCameraPos,p+surfTranslate+Vec3(0,-0.5f,0)));
			C.RampCol = b.iColortime;
			mGlowSurfaces.push_back(C);
		}
		bBonusStage?
			st = mSprites[SPR_BONUSTILES].GetTileUV(b.Faces[2]):
		st = mSprites[SPR_TILES].GetTileUV(b.Faces[2]);
	mShdrMgr.PassAttrib2fv(AtLocmins, st.v);
	mShdrMgr.PassAttrib2fv(AtLocmaxs, Vec2(st+stS).v);
	glNormal3f(-0.25f,-1,-0.25f);
	glTexCoord2f(st.x,st.y);glVertex3f(p.x-s,p.y-s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y);glVertex3f(p.x+s,p.y-s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y+stS.y);glVertex3f(p.x+s,p.y-s,p.z+s);
	glTexCoord2f(st.x,st.y+stS.y);glVertex3f(p.x-s,p.y-s,p.z+s);
	}
	//bottom surf
	b2=GetMapTile(pos+Vec3(0,0.51,0));
	if(b2==0 || (int)b2->pos.x != (int)pos.x || (int)b2->pos.z != (int)pos.z || b.iSpecial == SURF_BREAK || b2->iSpecial == SURF_BREAK || b2->mbPlatform)// || b2->iSpecial == SURF_ICE && b.iSpecial != SURF_ICE)
	{
		if(b.Faces[3] >= SURF_FUR && mbFUR)
		{
			mSurfaces.push_back(CreateSurface(Vec3(p.x+s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z+s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z+s)+surfTranslate,
				Vec3(0,1,0), b.Faces[3]-SURF_FUR, DRAW_FUR, Distance(mCameraPos,p+surfTranslate+Vec3(0,0.5f,0))));
		}
		else if(b.Faces[3] == SURF_JUMP || b.Faces[3] == SURF_JUMP+2 || b.iColortime>0)
		{
			Surface C = CreateSurface(Vec3(p.x+s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z+s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z+s)+surfTranslate,
				Vec3(0,1,0), b.Faces[3]+1, DRAW_GLOW, Distance(mCameraPos,p+surfTranslate+Vec3(0,0.5f,0)));
			C.RampCol = b.iColortime;
			mGlowSurfaces.push_back(C);
		}
		bBonusStage?
			st = mSprites[SPR_BONUSTILES].GetTileUV(b.Faces[3]):
		st = mSprites[SPR_TILES].GetTileUV(b.Faces[3]);
	mShdrMgr.PassAttrib2fv(AtLocmins, st.v);
	mShdrMgr.PassAttrib2fv(AtLocmaxs, Vec2(st+stS).v);
	glNormal3f(-0.25f,1,-0.25f);
	glTexCoord2f(st.x,st.y);glVertex3f(p.x+s,p.y+s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y);glVertex3f(p.x-s,p.y+s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y+stS.y);glVertex3f(p.x-s,p.y+s,p.z+s);
	glTexCoord2f(st.x,st.y+stS.y);glVertex3f(p.x+s,p.y+s,p.z+s);
	}
	//left surf
	b2=GetMapTile(pos+Vec3(-0.51,0,0));
	if(b2==0 || (int)b2->pos.y != (int)pos.y || (int)b2->pos.z != (int)pos.z || b.iSpecial == SURF_BREAK || b2->iSpecial == SURF_BREAK || b2->mbPlatform)// || b2->iSpecial == SURF_ICE && b.iSpecial != SURF_ICE)
	{
		if(b.Faces[4] >= SURF_FUR && mbFUR)
		{
			mSurfaces.push_back(CreateSurface(Vec3(p.x-s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y-s,p.z+s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z+s)+surfTranslate,
				Vec3(-1,0,0), b.Faces[4]-SURF_FUR, DRAW_FUR, Distance(mCameraPos,p+surfTranslate+Vec3(-0.5f,0,0))));
		}
		else if(b.Faces[4] == SURF_JUMP || b.Faces[4] == SURF_JUMP+2 || b.iColortime>0)
		{
			Surface C = CreateSurface(Vec3(p.x-s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x-s,p.y-s,p.z+s)+surfTranslate, Vec3(p.x-s,p.y+s,p.z+s)+surfTranslate,
				Vec3(-1,0,0), b.Faces[4]+1, DRAW_GLOW, Distance(mCameraPos,p+surfTranslate+Vec3(-0.5f,0,0)));
			C.RampCol = b.iColortime;
			mGlowSurfaces.push_back(C);
		}
		bBonusStage?
			st = mSprites[SPR_BONUSTILES].GetTileUV(b.Faces[4]):
		st = mSprites[SPR_TILES].GetTileUV(b.Faces[4]);
	mShdrMgr.PassAttrib2fv(AtLocmins, st.v);
	mShdrMgr.PassAttrib2fv(AtLocmaxs, Vec2(st+stS).v);
	glNormal3f(-1,-0.25f,-0.25f);
	glTexCoord2f(st.x,st.y);glVertex3f(p.x-s,p.y+s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y);glVertex3f(p.x-s,p.y-s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y+stS.y);glVertex3f(p.x-s,p.y-s,p.z+s);
	glTexCoord2f(st.x,st.y+stS.y);glVertex3f(p.x-s,p.y+s,p.z+s);
	}
	//right surf
	b2=GetMapTile(pos+Vec3(0.51,0,0));
	if(b2==0 || (int)b2->pos.y != (int)pos.y || (int)b2->pos.z != (int)pos.z || b.iSpecial == SURF_BREAK || b2->iSpecial == SURF_BREAK || b2->mbPlatform)// || b2->iSpecial == SURF_ICE && b.iSpecial != SURF_ICE)
	{
		if(b.Faces[5] >= SURF_FUR && mbFUR)
		{
			Surface C = CreateSurface(Vec3(p.x+s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z+s)+surfTranslate, Vec3(p.x+s,p.y-s,p.z+s)+surfTranslate,
				Vec3(1,0,0), b.Faces[5]-SURF_FUR, DRAW_FUR, Distance(mCameraPos,p+surfTranslate+Vec3(0.5f,0,0)));
			mSurfaces.push_back(C);
		}
		else if(b.Faces[5] == SURF_JUMP || b.Faces[5] == SURF_JUMP+2|| b.iColortime>0)
		{
			Surface C = CreateSurface(Vec3(p.x+s,p.y-s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z-s)+surfTranslate, Vec3(p.x+s,p.y+s,p.z+s)+surfTranslate, Vec3(p.x+s,p.y-s,p.z+s)+surfTranslate,
				Vec3(1,0,0), b.Faces[5]+1, DRAW_GLOW, Distance(mCameraPos,p+surfTranslate+Vec3(0.5f,0,0)));
			C.RampCol = b.iColortime;
			mGlowSurfaces.push_back(C);
		}
		bBonusStage?
			st = mSprites[SPR_BONUSTILES].GetTileUV(b.Faces[5]):
		st = mSprites[SPR_TILES].GetTileUV(b.Faces[5]);
	mShdrMgr.PassAttrib2fv(AtLocmins, st.v);
	mShdrMgr.PassAttrib2fv(AtLocmaxs, Vec2(st+stS).v);
	glNormal3f(1,-0.25f,-0.25f);
	glTexCoord2f(st.x,st.y);glVertex3f(p.x+s,p.y-s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y);glVertex3f(p.x+s,p.y+s,p.z-s);
	glTexCoord2f(st.x+stS.x,st.y+stS.y);glVertex3f(p.x+s,p.y+s,p.z+s);
	glTexCoord2f(st.x,st.y+stS.y);glVertex3f(p.x+s,p.y-s,p.z+s);
	}
}

void Engine::RenderWorld(int world, const float interp)
{
	if(bBonusStage)
	{
		glClearColor(0,0,0,0);
		glClear(GL_COLOR_BUFFER_BIT);
		UTIL_GL::GL3D::SetPerspective(70,1,1,20);
		gluLookAt(0,0,10,0,0,0,0,1,0);

		if(GCOLTIME>=6.0f)
			GCOLTIME-=6;
		float COL = GCOLTIME;//UTIL_Misc::Interpolate(GCOLTIME,GCOLTIME-0.03f,interp);
		Vec3 COLS[8];
		COLS[0]= Vec3(1,0.3f,0.3f);
		COLS[1]= Vec3(1,1,0.3f);
		COLS[2]= Vec3(0.3f,1,0.3f);
		COLS[3]= Vec3(0.3f,1,1);
		COLS[4]= Vec3(0.3f,0.3f,1);
		COLS[5]= Vec3(1,0.3f,1);
		COLS[6]= Vec3(1,0.3f,0.3f);
		COLS[7]= Vec3(1,1,0.3f);
		UTIL_GL::BindTexture(GL_TEXTURE_2D, mGfxImg[GFX_GRID]);
		float warp = UTIL_Misc::Interpolate((float)miBonusClock,(float)miBonusClock-1,interp)*0.01f;
		float warp2 = warp;
		float Deg = 360.0f/36.0f;
		float T = 1.0f/2.5f;
		float texV = 0;
		for(int n=0; n<9; n++)
		{
			float R = 0;
			float texU = 0;
			float S = 1.0f/9.0f;
			float C= (10-n)/5.f;
			float C2= (10-(n+1))/5.f;
			if(C>1)C=1;
			if(C2>1)C2=1;
			Vec3 col = UTIL_Misc::Interpolate(COLS[7-(int)(COL+1)],COLS[7-(int)(COL)],COL-(int)COL);
			Vec3 col2 = UTIL_Misc::Interpolate(COLS[7-(int)(COL+2)],COLS[7-(int)(COL+1)],COL-(int)COL);
			Vec3 V = Vec3(sin(DEG2RAD(warp2*60)) * (0.15f),cos(DEG2RAD(warp2*60)) * (0.15f),0);
			Vec3 V2 = Vec3(sin(DEG2RAD((warp2+1)*60)) * (0.15f),cos(DEG2RAD((warp2+1)*60)) * (0.15f),0);
			glBegin(GL_TRIANGLE_STRIP);
			for(int d =0; d<37; d++)
			{
				glColor3fv((col).v);
				Vec3 p = V+ Vec3(sin(DEG2RAD(R)) * (0.1f * (n+1)),cos(DEG2RAD(R)) * (0.1f * (n+1)),n);
				glTexCoord2f(texU+warp,texV+warp);
				glVertex3fv(p.v);
				p = V2+ Vec3(sin(DEG2RAD(R)) * (0.1f * (n+2)),cos(DEG2RAD(R)) * (0.1f * (n+2)),n+1);
				glColor3fv((col2).v);
				glTexCoord2f(texU+warp,(texV+T)+warp);
				glVertex3fv(p.v);
				R += Deg;
				texU+=S;
			}
			glEnd();
			texV+=T;
			COL++;
			warp2+=1;
			if(COL>=6)
				COL-=6;
	}
		glDisable(GL_TEXTURE_2D);
		glClear(GL_DEPTH_BUFFER_BIT);
		return;
	}
	if(world == 4)
	{
		glColor3f(1,1,1);
		RenderSky(world,3);
		glClear(GL_DEPTH_BUFFER_BIT);
	}
	if(world == 3)
	{
//	glColor3f(0.5,0.5,0.5);
	glColor3f(1,1,1);
	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	RenderSky(world,3);
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.9,0.9,0.9);
	RenderSky(world,2);
	glPopMatrix();
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.8,0.8,0.8);
	RenderSky(world,1);
	glPopMatrix();

	glClear(GL_DEPTH_BUFFER_BIT);
	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE);
	RenderClouds(interp);
	glDisable(GL_BLEND);
	}
	if(world == 2)
	{
//	glColor3f(0.5,0.5,0.5);
	glColor3f(1,1,1);
	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	RenderSky(world,3);
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.9,1,0.9);
	RenderSky(world,2);
	glPopMatrix();
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.8,1,0.8);
	RenderSky(world,1);
	glPopMatrix();
	glClear(GL_DEPTH_BUFFER_BIT);
/*	glPushMatrix();
	glScalef(0.7,1,0.7);
	RenderSky(0,0);
	glPopMatrix();*/
	}
	if(world == 1)
	{
//	glColor3f(0.5,0.5,0.5);
	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	RenderSky(world,3);
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.9,1,0.9);
	RenderSky(world,2);
	glPopMatrix();
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.8,1,0.8);
	RenderSky(world,1);
	glPopMatrix();
	glClear(GL_DEPTH_BUFFER_BIT);
/*	glPushMatrix();
	glScalef(0.7,1,0.7);
	RenderSky(0,0);
	glPopMatrix();*/
	}
	if(world == 0)
	{
//	glColor3f(0.5,0.5,0.5);
	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	RenderSky(world,3);
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.9,1,0.9);
	RenderSky(world,2);
	glPopMatrix();
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.8,1,0.8);
	RenderSky(world,1);
	glPopMatrix();
	glClear(GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glScalef(0.7,1,0.7);
	RenderSky(world,0);
	glPopMatrix();

	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE);
	glClear(GL_DEPTH_BUFFER_BIT);
	RenderClouds(interp);
	glDisable(GL_BLEND);

	}
}

/****************************************/
/* render the world skybox				*/
/* will probably need to be specific	*/
/* wit this at some point				*/
/****************************************/
void Engine::RenderSky(int world, int layer)
{
	float y = 76;
	if(world >= 1)
		y = 128;
//	if(world == 3)
//		y = 76;
	//glColor3f(1,1,1);
	mShdrMgr.BindProgram(mShdrPass);
//	mShdrMgr.PassSampler2D("tex",0,mGfxImg[GFX_W1_SKY1]);
	mShdrMgr.PassSampler2D("tex",0,imgSkyBox[layer][SKY_FRONT]);
//	UTIL_GL::BindTexture(GL_TEXTURE_2D, mGfxImg[GFX_W1_SKY1]);
	glBegin(GL_QUADS);
	//back surf
	glTexCoord2f(0,0);glVertex3f(-128,y,-128);
	glTexCoord2f(1,0);glVertex3f(128,y,-128);
	glTexCoord2f(1,1);glVertex3f(128,-y,-128);
	glTexCoord2f(0,1);glVertex3f(-128,-y,-128);
	glEnd();
	mShdrMgr.PassSampler2D("tex",0,imgSkyBox[layer][SKY_BACK]);
	glBegin(GL_QUADS);
	//front surf
	glTexCoord2f(0,0);glVertex3f(128,y,128);
	glTexCoord2f(1,0);glVertex3f(-128,y,128);
	glTexCoord2f(1,1);glVertex3f(-128,-y,128);
	glTexCoord2f(0,1);glVertex3f(128,-y,128);
	glEnd();
	if(layer==3)
	{
		mShdrMgr.PassSampler2D("tex",0,imgSkyBox[layer][SKY_UP]);
		glBegin(GL_QUADS);
		//top surf
		glTexCoord2f(0,0);glVertex3f(-128,y,-128);
		glTexCoord2f(1,0);glVertex3f(128,y,-128);
		glTexCoord2f(1,1);glVertex3f(128,y,128);
		glTexCoord2f(0,1);glVertex3f(-128,y,128);
		glEnd();
	}
	if(layer==3 || layer == 0 || layer == 1 && world==1 || layer == 2 && world==2 || world==3)
	{
		mShdrMgr.PassSampler2D("tex",0,imgSkyBox[layer][SKY_DOWN]);
		glBegin(GL_QUADS);
		//bottom surf
		glTexCoord2f(0,0);glVertex3f(-128,-y,-128);
		glTexCoord2f(1,0);glVertex3f(128,-y,-128);
		glTexCoord2f(1,1);glVertex3f(128,-y,128);
		glTexCoord2f(0,1);glVertex3f(-128,-y,128);
		glEnd();
	}
	if(world == 1 && layer !=2  || world != 1)
	{
		if(world == 2 && layer != 1 || world !=2)
		{
			mShdrMgr.PassSampler2D("tex",0,imgSkyBox[layer][SKY_LEFT]);
			glBegin(GL_QUADS);
			//left surf
			glTexCoord2f(0,0);glVertex3f(-128,y,128);
			glTexCoord2f(1,0);glVertex3f(-128,y,-128);
			glTexCoord2f(1,1);glVertex3f(-128,-y,-128);
			glTexCoord2f(0,1);glVertex3f(-128,-y,128);
			glEnd();
		}
		mShdrMgr.PassSampler2D("tex",0,imgSkyBox[layer][SKY_RIGHT]);
		glBegin(GL_QUADS);
		//right surf
		glTexCoord2f(0,0);glVertex3f(128,y,-128);
		glTexCoord2f(1,0);glVertex3f(128,y,128);
		glTexCoord2f(1,1);glVertex3f(128,-y,128);
		glTexCoord2f(0,1);glVertex3f(128,-y,-128);//*/
		glEnd();//*/
		mShdrMgr.BindProgram(0);
	}
}
/**********************************/
/* render cloud plane			  */
/**********************************/
void Engine::RenderClouds(const float interp)
{
	float s = UTIL_Misc::Interpolate(mfCloudScroll, mfOCloudScroll, interp);

	UTIL_GL::BindTexture(GL_TEXTURE_2D, imgSkyBox[0][5]);
	GLfloat fogColor[4]= {0, 0, 0, 1};
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogfv(GL_FOG_COLOR, fogColor);
	glFogf(GL_FOG_DENSITY, 1);
	glHint(GL_FOG_HINT, GL_DONT_CARE);
	glFogf(GL_FOG_START, 80);
	glFogf(GL_FOG_END, 128);
	glEnable(GL_FOG);

	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	//top surf
/*	glTexCoord2f(0,-s+4);glVertex3f(-256,8,-256);
	glTexCoord2f(4,-s+4);glVertex3f(256,8,-256);
	glTexCoord2f(4,-s);glVertex3f(256,8,256);
	glTexCoord2f(0,-s);glVertex3f(-256,8,256);
*/	//bottom surf
	glTexCoord2f(0,s);glVertex3f(-256,56,-256);
	glTexCoord2f(4,s);glVertex3f(256,56,-256);
	glTexCoord2f(4,s+4);glVertex3f(256,56,256);
	glTexCoord2f(0,s+4);glVertex3f(-256,56,256);
	glEnd();

/*	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
	//back surf
	glVertex3f(-128,8,-128);
	glVertex3f(128,8,-128);
	glVertex3f(128,-8,-128);
	glVertex3f(-128,-8,-128);
	//front surf
	glVertex3f(128,8,128);
	glVertex3f(-128,8,128);
	glVertex3f(-128,-8,128);
	glVertex3f(128,-8,128);
	//left surf
	glVertex3f(-128,8,128);
	glVertex3f(-128,8,-128);
	glVertex3f(-128,-8,-128);
	glVertex3f(-128,-8,128);
	//right surf
	glVertex3f(128,8,-128);
	glVertex3f(128,8,128);
	glVertex3f(128,-8,128);
	glVertex3f(128,-8,-128);
	glEnd();//*/

	glDisable(GL_FOG);
}

/*****************************************/
/* change engine state. timed			 */
/*****************************************/
void Engine::ChangeState(int state, int time)
{
	pTimedCallback = NULL;
	switch(state)
	{
	case STATE_GAME:
		pTimedUpdateFunc = UpdateGame;
		pTimedRenderFunc = RenderGame;
		break;
	case STATE_SPLASH:
		pTimedUpdateFunc = UpdateSplash;
		pTimedRenderFunc = RenderSplash;
		break;
	case STATE_CREDITS:
		pTimedUpdateFunc = UpdateCredits;
		pTimedRenderFunc = RenderCredits;
		break;
	case STATE_ENDDEMO:
		pTimedUpdateFunc = UpdateEndDemo;
		pTimedRenderFunc = RenderEndDemo;
		break;
	case STATE_DESIGN:
		pTimedUpdateFunc = UpdateDesigner;
		pTimedRenderFunc = RenderDesigner;
		break;
	case STATE_GAMESELECT:
		pTimedUpdateFunc = UpdateGameSelect;
		pTimedRenderFunc = RenderGameSelect;
		break;
	case STATE_STAGESELECT:
		pTimedUpdateFunc = UpdateStageSelect;
		pTimedRenderFunc = RenderStageSelect;
		break;
	}
	iStateTime = time;
	iEngineState = state;
	if(iStateTime<=0)
	{
		UpdateState();
	}
}
/*************************************/
/* change state, and make a callback */
/*************************************/
void Engine::ChangeState(int state, int time,void (*pCallTimed)())
{
	pTimedCallback = pCallTimed;
	switch(state)
	{
	case STATE_GAME:
		pTimedUpdateFunc = UpdateGame;
		pTimedRenderFunc = RenderGame;
		break;
	case STATE_SPLASH:
		pTimedUpdateFunc = UpdateSplash;
		pTimedRenderFunc = RenderSplash;
		break;
	case STATE_CREDITS:
		pTimedUpdateFunc = UpdateCredits;
		pTimedRenderFunc = RenderCredits;
		break;
	case STATE_ENDDEMO:
		pTimedUpdateFunc = UpdateEndDemo;
		pTimedRenderFunc = RenderEndDemo;
		break;
	case STATE_DESIGN:
		pTimedUpdateFunc = UpdateDesigner;
		pTimedRenderFunc = RenderDesigner;
		break;
	case STATE_GAMESELECT:
		pTimedUpdateFunc = UpdateGameSelect;
		pTimedRenderFunc = RenderGameSelect;
		break;
	case STATE_STAGESELECT:
		pTimedUpdateFunc = UpdateStageSelect;
		pTimedRenderFunc = RenderStageSelect;
		break;
	}
	iStateTime = time;
	iEngineState = state;
	if(iStateTime<=0)
	{
		UpdateState();
	}
}
/*************************************/
/* update the timed state			 */
/*************************************/
void Engine::UpdateState()
{
	iStateTime--;
	if(iStateTime<=0)
	{
		pUpdateFunc = pTimedUpdateFunc;
		pRenderFunc = pTimedRenderFunc;
		if(pTimedCallback)
		{
			(*pTimedCallback)();
			pTimedCallback = NULL;
		}
	}
}
/**************************************/
/* render all menus					  */
/**************************************/
void Engine::RenderMenus(const float interp)
{
	UTIL_GL::GL2D::SetOrtho(640,480);
	std::list<Menu*>::iterator i;
	for(i=mMenus.begin(); i!=mMenus.end(); i++)
	{
		(*i)->Render(mInterp);
	}
}
/****************************************/
/* render scene to fade buffer for      */
/* smooth transition					*/
/****************************************/
void Engine::RenderSceneToFadeBuffer()
{
	//copy the scene into the fade buffer
	BindFBO(gpEng->mFadeFBO,1);
	//render the scene
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDisable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	UTIL_GL::GL2D::SetOrtho(gpEng->winWidth,gpEng->winHeight);
	if(!mbFallBackMode)
	{
	UTIL_GL::BindTexture(GL_TEXTURE_RECTANGLE_ARB, gpEng->mMainFBO.colortexID[miBoundFBO]);
	glBegin(GL_QUADS);
	glTexCoord2f( 0,mMainFBO.h);					glVertex2f(0,0);
	glTexCoord2f( mMainFBO.w,mMainFBO.h);	glVertex2f(gpEng->winWidth,0);
	glTexCoord2f( mMainFBO.w,0);					glVertex2f(gpEng->winWidth,gpEng->winHeight);
	glTexCoord2f( 0,0);									glVertex2f(0,gpEng->winHeight);
	glEnd();
	glDisable(GL_TEXTURE_RECTANGLE_ARB);
	}
	else
	{
	UTIL_GL::BindTexture(GL_TEXTURE_2D, gpEng->mMainFBO.colortexID[miBoundFBO]);
	glBegin(GL_QUADS);
	glTexCoord2f( 0,1);	glVertex2f(0,0);
	glTexCoord2f( 1,1);	glVertex2f(gpEng->winWidth,0);
	glTexCoord2f( 1,0);	glVertex2f(gpEng->winWidth,gpEng->winHeight);
	glTexCoord2f( 0,0);	glVertex2f(0,gpEng->winHeight);
	glEnd();
	}
	glDisable(GL_TEXTURE_2D);
	UnBindFBO();

	if(UTIL_GL::GetGLError())
	{
		gLog.OutPut("Error fade2buffer!\n");
	}

}
/************************************/
/* save the players game slot		*/
/************************************/
bool Engine::SaveGameSlot()
{
	CFileIO fIO;
	char szFile[256];
	sprintf(szFile,"Saving Game Slot: %i\n",miSaveGameSlot);
	gLog.OutPut(szFile);

	sprintf(szFile,"Data/Slots/Slot%i.slt",miSaveGameSlot);
	
	if(!fIO.GetFile(szFile,"wb"))
	{
		return false;//doesnt exit - no slot created
	}	

	if(!fIO.WriteBinary(&szPlayerName))
		return false;
	if(!fIO.WriteBinary(&miWorld))
		return false;
	if(!fIO.WriteBinary(&miStarCount))
		return false;
	if(!fIO.WriteBinary(&miGemCount))
		return false;
	if(!fIO.WriteBinary(&miLives))
		return false;
	if(!fIO.WriteBinary(&iTimePlayed))
		return false;
	if(!fIO.WriteBinary(&miPlayerBallIdx))
		return false;
	int numLayers = (int)mPlayerType.iColLayers.size();
	if(!fIO.WriteBinary(&numLayers))
		return false;
	for(int n=0; n<numLayers; n++)
	{
		int color = mPlayerType.iColLayers[n];
		if(!fIO.WriteBinary(&color))
			return false;
	}
	if(!fIO.WriteBinary(&iBonusStage))
		return false;
	if(!fIO.WriteBinary(&iPreservedBonusStage))
		return false;
	if(!fIO.WriteBinary(&iPlayerStickerType))
		return false;
	if(!fIO.WriteBinary(&miScore))
		return false;
	if(!fIO.WriteBinary(&miPreservedScore))
		return false;
	return true;
}
/*****************************************/
/* save the players persistent data		 */
/*****************************************/
bool Engine::SavePerstistanceData()
{
	CFileIO fIO;
	char szFile[256];
	sprintf(szFile,"Data/Slots/World%iSlot%i.slt",miWorld,miSaveGameSlot);

	if(!fIO.GetFile(szFile,"wb"))
	{
		return false;//doesnt exit - no slot created
	}
	for(int n=0; n<15; n++)
	{
		if(!fIO.WriteBinary(&mbStageSelect[n]))
			return false;
		unsigned int numData = (int)miStagePersistance[n].size();
		if(!fIO.WriteBinary(&numData))
			return false;

		std::list<unsigned int>::iterator it;
		for(it=miStagePersistance[n].begin(); it!=miStagePersistance[n].end(); it++)
		{
			if(!fIO.WriteBinary(&(*it)))
				return false;
		}
	}
	return true;
}
/*******************************************/
/* load hte players persistent data		   */
/*******************************************/
bool Engine::LoadPerstistanceData()
{
	gLog.OutPut("Loading Persistence data.\n");
	CFileIO fIO;
	char szFile[256];
	sprintf(szFile,"Data/Slots/World%iSlot%i.slt",miWorld,miSaveGameSlot);

	if(!fIO.GetFile(szFile,"rb"))
	{
		return false;//doesnt exit - no slot created
	}
	for(int n=0; n<15; n++)
	{
		if(!fIO.ReadBinary(&mbStageSelect[n]))
			return false;

		miStagePersistance[n].clear();
		unsigned int numData = 0;
		if(!fIO.ReadBinary(&numData))
			return false;

		for(int i=0; i<numData; i++)
		{
			unsigned int idx;
			if(!fIO.ReadBinary(&idx))
				return false;
			miStagePersistance[n].push_back(idx);
		}
	}
	return true;
}
/******************************************************************************************/
/* Callbacks from this point onwards													  */
/******************************************************************************************/
void CALLBACK_StartStageSelect()
{
	BindFBO(gpEng->mFadeFBO,1);
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT);
	UnBindFBO();
	gpEng->mFadeEng.Cleanup();
	gpEng->StartStageSelect();
}
void CALLBACK_StartStageSelect2()
{
	gpEng->RenderSceneToFadeBuffer();
	gpEng->LoadStageSelectData();
	gpEng->StartStageSelect();
}
void CALLBACK_StartGameSelect()
{
	BindFBO(gpEng->mFadeFBO,1);
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT);
	UnBindFBO();
	gpEng->mFadeEng.Cleanup();
//	gpEng->mMenus.remove_if(CleanupMenus);
	gpEng->StartGameSelect();
	FADE_Star* FS2 = new FADE_Star;
	FS2->Spawn(1,Vec3(0,1,1));
	gpEng->mFadeEng.InsertSystem(FS2);//*/
}
void CALLBACK_StartGameSelect2()
{
	gpEng->StartGameSelect();
}
void CALLBACK_StartSplash()
{
	BindFBO(gpEng->mFadeFBO,1);
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT);
	UnBindFBO();
	gpEng->mFadeEng.Cleanup();
//	gpEng->mMenus.remove_if(CleanupMenus);
	gpEng->StartSplash();
}
void CALLBACK_Quit()
{
	gpEng->mbQuit = true;
}
void CALLBACK_VideoQuit()
{
	gpEng->mbQuit = true;
//	system("Screwball.exe");
}
