#pragma once
#include "UTIL_SDL.h"
#include "UTIL_OpenGL.h"
#include "UTIL_Misc.h"
#include "Timer.h"
#include "FileIO.h"
#include "Button.h"
#include "Log.h"
#include "Vector.h"
#include "Font.h"
#include <vector>
#include <list>
#include <string>
#include <algorithm>
#include "Player.h"
#include "MD2.h"
#include "OBJ.h"
#include "SpriteSheet.h"
#include "ShaderMgr.h"
#include "FrameBufferObject.h"
#include "TextureManager.h"
#include "Shaders.h"
#include "Material.h"
#include "Entity.h"
#include "Frustum.h"
#include "Block.h"
#include "Platform.h"
#include "ParticleSystem.h"
#include "FadeSystem.h"
#include "Menu.h"
#include "BallType.h"
#include "TextParticle.h"
#include "Recording.h"
#include "Laser.h"
#include "Sound.h"
#include "SND_CHANNELS.h"

//const float TIME_STEP = 0.0111111f;
const float TIME_STEP = 0.012f;

#define MAP_DIMENSIONS 64
enum 
{
	STATE_GAME,			//the game
	STATE_SPLASH,		//splash page
	STATE_CREDITS,		//Credits page
	STATE_DESIGN,		//design ball
	STATE_GAMESELECT,	//pick a slot to play
	STATE_STAGESELECT,	//pick world and stage
	STATE_ENDDEMO	//End of the demo game
};

enum
{
	GFX_TILES = 0,
	GFX_TILES_HI,
	GFX_TILES_NM,
	GFX_FUR_TILES,
	GFX_FUR_VECTOR_TILES,
	GFX_GEM,
	GFX_HEART,
	GFX_BONUSTILES,
	GFX_BONUSTILES_NM,
	GFX_GRID,
	GFX_BUSH1,
	GFX_BUSH2,
	GFX_BUSH3,
	GFX_STAR_GLOW,
	GFX_LOADING1
};
enum
{
	GFXMENU_CUSTOMIZE=0,
	GFXMENU_GETREADY,
	GFXMENU_FALLOUT,
	GFXMENU_FAIL,
	GFXMENU_TIMEOUT,
	GFXMENU_CLEAR,
	GFXMENU_GAMEOVER,
	GFXMENU_PERFECT,
	GFXMENU_WORLD1,
	GFXMENU_WORLD2,
	GFXMENU_WORLD3,
	GFXMENU_WORLD4,
	GFXMENU_WORLD5,
	GFXMENU_BONUS_STAGE,
	GFXMENU_SPLASH
};
enum 
{
	MDL_BALL = 0,
	MDL_HALFBALL,
	MDL_SPHERE,
	MDL_SPHERE2,
	MDL_STICKER,
	MDL_GEM,
	MDL_SPIKES,
	MDL_STAR,
	MDL_CLOCK,
	MDL_SLINKY01,
	MDL_SLINKY02,
	MDL_SLINKY03,
	MDL_SLINKY04,
	MDL_SLINKY05,
	MDL_SLINKY06,
	MDL_SLINKY07,
	MDL_SLINKY08,
	MDL_SLINKY09,
	MDL_SLINKY10,
	MDL_SLINKY11,
	MDL_SLINKY12,
	MDL_SLINKY13,
	MDL_SLINKY14,
	MDL_SLINKY15,
	MDL_SLINKY16,
	MDL_ARROW,
	MDL_ROTORBLADE,
	MDL_1UP,
	MDL_BUTTON,
	MDL_BUTTON_RIM,
	MDL_LASER,
	MDL_LASER_GLOW,
	MDL_TELEPORT,
	MDL_TELEPORT_PAD
};
enum
{
	SPR_TILES = 0,
	SPR_FUR_TILES,
	SPR_BONUSTILES,
	SPR_UI,
	SPR_PARTICLES,
	SPR_PARTICLESALPHA,
	SPR_MENU,
	SPR_CLOCK,
	SPR_CLOCK_DIGITS
};
enum
{
	SKY_DOWN = 0,
	SKY_FRONT,
	SKY_RIGHT,
	SKY_BACK,
	SKY_LEFT,
	SKY_UP
};
enum
{
	MAT_FLAT = 0,
	MAT_WORLD1,
	MAT_WORLD2,
	MAT_WORLD3,
	MAT_WORLD4,
	MAT_WORLD5,
	MAT_BONUS,
	MAT_TELEPORT
};
enum
{
	SND_BALL_CLIP = 0,
	SND_CHEER,
	SND_BALL_JUMP,
	SND_BALL_JUMP_PAD,
	SND_BALL_POP,
	SND_BALL_EXPLODE,
	SND_UI_BOUNCE,
	SND_UI_MOVE,
	SND_UI_SELECT,
	SND_UI_SELECT2,
	SND_UI_BACKUP,
	SND_UI_WHOOSH,
	SND_UI_WHOOSH2,
	SND_UI_WHOOSH3,
	SND_FADE1,
	SND_FADE2,
	SND_VOICE_GETREADY,
	SND_VOICE_GO,
	SND_VOICE_TEN,
	SND_VOICE_NINE,
	SND_VOICE_EIGHT,
	SND_VOICE_SEVEN,
	SND_VOICE_SIX,
	SND_VOICE_FIVE,
	SND_VOICE_FOUR,
	SND_VOICE_THREE,
	SND_VOICE_TWO,
	SND_VOICE_ONE,
	SND_VOICE_LOSTBALL,
	SND_VOICE_DROPOUT,
	SND_VOICE_BONUS_STAGE,
	SND_TELEPORT1,
	SND_TELEPORT2,
	SND_BUTTON,
	SND_PICKUP1,
	SND_PICKUP2,
	SND_SPIKES,
	SND_CRUMBLE,
	SND_ARROW,
	SND_HIT
};
//player ball selection
#define MAX_COLORS 51
extern const Vec3 BallColors[MAX_COLORS];


class Engine
{
public:
	double accumulator;

	Engine(int w, int h, bool fscreen);
	~Engine();
	int winWidth, winHeight;

	void InitDevil();
	void Pump();


	void KeyStateAlltered(int key, bool state);
	void TextInput(SDL_keysym key);
	void KeysPostUpdate();
	void CheckJoystickMotion(SDL_Event event);
	void SaveInput();
	void LoadInput();
	Button KEY_MOUSE1, KEY_MOUSE2, KEY_MOUSE3, KEY_QUIT, KEY_PAUSE;
	Button KEY_TURN_LEFT, KEY_TURN_RIGHT, KEY_FORWARD, KEY_BACK, KEY_JUMP, KEY_ACTION2, KEY_CAMERA;
	bool mbCamPreviewMode;
	Vec3 mPreviewCamRight, mPreviewCamUp, mPreviewCamForward;
	int mPrevCamZoomDistance;
	float fJoystickSensi;
	int oldPadAxisH, oldPadAxisV;
	bool bCfgJoystick;
	Button* pCfgButton;
	int iControlMethod;
	int MouseX, MouseY;
	bool bTextInput;
	char szInputText[6];
	int iTextIdx, iTextMaxLength;
	Vec2 mMouseMotion;
	int iOption;	
	bool bJoystick;

	bool mbQuit,mbPaused;
	Timer mTimer;
	Font mFont1;
	float mInterp;

	unsigned int miInternalClock;

	void RenderCube(Block b, Vec3 pos, Vec3 surfTranslate, float scale, GLint AtLocmins, GLint AtLocmaxs);
	void RenderSky(int world, int layer);
	void RenderClouds(const float interp);
	void RenderWorld(int world, const float interp);

	unsigned int imgSkyBox[4][6];
	float mfCloudScroll, mfOCloudScroll;

	Player mPlayer;

	Vec3 mCameraPos, mOCameraPos, mCamTarget, mCameraShake;
	float mfCamZoom,mfCamVertical;
	int iCameraClipCounter;

	bool LoadConstantData();
	bool LoadWorldData();
	void CleanupWorldData();

	unsigned int mGfxDistort;
	unsigned int mGfxImg[20];
	OBJ mObjects[64];
	SpriteSheet mSprites[10];

	bool InitializeExtensions();
	ShaderMgr mShdrMgr;
	TextureManager mTexMgr;

	FBO mMainFBO, mMainFBO2;
	FBO mFadeFBO;
	int miBoundFBO;

	//shaders
	bool LoadShader(char* name, char* szVp, char* szFp, int& outShdr);
	int mShdrPass;
	int mShdrPassRect;
	int mShdrPassRectScale;
	int mShdrPPDirLight;
	int mShdrPPDirLightLerp;
	int mShdrPBallLight;
	int mShdrPassLerp;
	int mShdrDOFH,mShdrDOFV;
	int mShdrRadial;
	int mShdrFur;
	int mShdrFadeFx1;
	int mShdrFadeFx2;
	int mShdrPassGrey;
	int mShdrPassRectScaleGrey;
	int mShdrDistort;
	int mShdrDistortRect;
	int mShdrSuperSample2x;
	int mShdrSuperSample2xRect;
	int iSuperSamples;

	//materials
	MaterialManager mMatMgr;
	Material* mMaterials[10];

	bool mbFallBackMode;
	bool mbDOF;
	bool mbFUR;
	bool mbShadows;
	bool mbBlur;
	bool mbShowFPS;

	//entities
	std::list<Entity*> mGameEnts;
	std::list<Beam> mBeams;


	//frustum
	Frustum mFrustum;

	//map
	bool LoadMap(char* name, bool persist);

	Block* cMap[(MAP_DIMENSIONS*MAP_DIMENSIONS*MAP_DIMENSIONS)+1];
	std::list<Block> mMapBlocks;
	std::list<Block*> mSpecialBlocks;//special blocks, might be shrinking etc

	void ClearMap();
	void CleanupMap();
	Block* GetMapTile(Vec3 p);
	std::list<Platform*> mPlatforms;

	std::list<Surface> mSurfaces;//collect special surfaces with this (fur)
	std::list<Surface> mGlowSurfaces;//collect special surfaces with this (glow)
	Vec3 mShadowCol;


	//engine states
	void ChangeState(int state, int time);
	void ChangeState(int state, int time,void (*pCallTimed)(void));
	void UpdateState();
	int iStateTime;
	int iEngineState;
	void (Engine::*pUpdateFunc)();
	void (Engine::*pRenderFunc)(const float interp);
	void (Engine::*pTimedUpdateFunc)();
	void (Engine::*pTimedRenderFunc)(const float interp);
	void (*pTimedCallback)();

	void UpdateGame();
	void RenderGame(const float interp);
	void RenderHUD(const float interp);
	std::list<Vec3> mHudColors;
	float mfHudColRamp;

	void StartDesigner();
	void UpdateDesigner();
	void RenderDesigner(const float interp);

	void StartGameSelect();
	void UpdateGameSelect();
	void RenderGameSelect(const float interp);

	void LoadStageSelectData();
	void StartStageSelect();
	void LoadSelectedStage();
	void UpdateStageSelect();
	void RenderStageSelect(const float interp);

	void StartSplash();
	void UpdateSplash();
	void RenderSplash(const float interp);
	void StartTutorial();

	void StartCredits();
	void UpdateCredits();
	void RenderCredits(const float interp);

	void StartEndDemo();
	void UpdateEndDemo();
	void RenderEndDemo(const float interp);

	BallType mBallTypes[10];
	BallType mPlayerType;
	int iPlayerStickerType;
	int miPlayerBallIdx;//used for saving
	char szPlayerName[8];
	int miWorld,miOWorld;//current world: TODO must save state of all the worlds
	int miGemCount;// how many gems the player has collect so far
	int miLocalGemCount;//how many he has collected for the current level. Add to globabl count at end of stage
	int miStageGemCount;//how many gems are available on this stage
	bool mbCollectedGem;
	bool mbPerfect;// collected al lgems on the level
	int miPerfectTimer;
	int miStarCount,miLocalStarCount, miStageStarCount;
	int miLives;//lives remaining
	unsigned int iTimePlayed;
	float mfStageTime,mfStageTimeEnd,mfOldStageTime;//the time on this level
	unsigned int mgPersistanceCounter;
	float fStageWonY,fOStageWonY,fStageWonGrav;
	int iHUDGemFlashTime;
	int iHUDStarFlashTime;
	int iBonusStage;
	bool bBonusStage;
	int iPreservedBonusStage;//store this once the player changes world. This can be used for continuing a game.
	float GCOLTIME;
	int iColoredBlocks;
	Block *pLastBlock;
	int miLocal1UpCount;
	int mi1UpGiveTimer;
	int mi1UpSlotTime[10];
	int mi1UpSlotRotateTime[10];
	int mi1UpSlotTwistTime;
	bool mbPauseClock;
	float fPauseClockTime;
	float fGreyScaleClock, fGreyScaleClockStart;
	unsigned int miScore, miPreservedScore;

	int miSaveGameSlot;
	bool SaveGameSlot();
	bool mbStageSelect[15];//lets us know if the stages are open for play
	std::list<unsigned int>	miStagePersistance[15];//store which entity indexes need to be removed
	std::list<unsigned int>	miCurrentPersistance;//store the perstistance for the current stage
	bool SavePerstistanceData();
	bool LoadPerstistanceData();
	unsigned int miSelectedStageBall;//which ball the player is selecting

	//menu stuffs
	unsigned int MenuImgs[20];
	std::list<Menu*> mMenus;
	void RenderMenus(const float interp);


	//particles
	ParticleEngine mPartEng;
	FadeEngine	mFadeEng;
	std::list<TextParticle> mTextParty;

	void RenderSceneToFadeBuffer();

	//stickers
	std::vector<unsigned int> mStickers;
	int iSelectedSticker;

	//demorecording related
	std::list<BetaRecording> mBetaRecording;
	int miBonusClock;

	int iDebugFrame;

	//music related
	Music mMainMusic, mAltMainMusic, mClockMusic, mClearedMusic, mBonusMusic, mSplashMusic;
	float fMusicTimer;//so we can resume from a position

	Sample mSnd[64];

	//Splashballs
	std::list<SplashBall> mSplashBalls;
	bool bTutorialGame;

	void RenderLoadingBar(float percent);
};
extern Engine* gpEng;
void CALLBACK_StartStageSelect();
void CALLBACK_StartStageSelect2();
void CALLBACK_StartGameSelect();
void CALLBACK_StartGameSelect2();
void CALLBACK_StartSplash();
void CALLBACK_Quit();
void CALLBACK_VideoQuit();