#include "Entity.h"
#include "Engine.h"
#include "Gemstone.h"
#include "Platform.h"
#include "Spikes.h"
#include "Star.h"
#include "Clock.h"
#include "Slinky.h"
#include "Arrow.h"
#include "Rotor.h"
#include "1Up.h"
#include "Switch.h"
#include "Laser.h"
#include "Teleporter.h"

Entity* EntFactory(int type)
{
	Entity* e = NULL;
	switch(type)
	{
	case ENT_GEM:
//		gLog.OutPut("Creating Gemstone entity\n");
		e = new Gemstone;
		break;
	case ENT_SPIKES:
//		gLog.OutPut("Creating Spikes entity\n");
		e = new Spikes;
		break;
		break;
	case ENT_STAR:
//		gLog.OutPut("Creating Star entity\n");
		gpEng->miStageStarCount++;
		e = new Star;
		break;
	case ENT_CLOCK:
//		gLog.OutPut("Creating Clock entity\n");
		e = new Clock;
		break;
	case ENT_SLINKY:
//		gLog.OutPut("Creating Slinky entity\n");
		e = new Slinky;
		break;
	case ENT_ARROW:
//		gLog.OutPut("Creating ArrowGenerator entity\n");
		e = new ArrowGenerator;
		break;
	case ENT_ROTORBLADE:
//		gLog.OutPut("Creating RotorBlade entity\n");
		e = new Rotor;
		break;
	case ENT_1UP:
//		gLog.OutPut("Creating 1Up entity\n");
		e = new Life1Up;
		break;
	case ENT_BUTTON:
//		gLog.OutPut("Creating Button entity\n");
		e = new Switch;
		break;
	case ENT_LASER:
//		gLog.OutPut("Creating Laser entity\n");
		e = new Laser;
		break;
	case ENT_TELEPORT:
//		gLog.OutPut("Creating Laser entity\n");
		e = new Teleporter;
		break;

	default:
		gLog.OutPut("Failed to create an entity for unknown type!\n");
		break;
	}
	e->iBaseType = type;
	return e;
}
bool PruneEnts(Entity* e)
{
	if(e->bRemove)
	{
		//TODO: Must remove child from a platform is riding
		delete e;
		e = NULL;
		return true;
	}
	e->Update();
	e->bVisible = gpEng->mFrustum.BoundsInFrustum(e->mMins, e->mMaxs);
	return false;
}
bool CleanEnts(Entity* e)
{
	if(e != NULL)
	{
	e->bRemove = true;
	delete e;
	e = NULL;
	}
	return true;
}
bool SortEntsByDistance(Entity* e1, Entity* e2)
{
	if(e1 != NULL && e2 != NULL)
	if(e1->fDistToCamera > e2->fDistToCamera)
		return true;
	return false;
}
bool FreezeEnts(Entity* e)
{
	e->Freeze();
	return false;
}
void Entity::SetDefaultAttribs(Vec3 p, Vec3 f, Vec3 r, Vec3 u)
{
//	gLog.OutPut("Setting default attribs\n");


	mPos = mOPos = p;
	mForward = Vec3(0,0,-1);
	mRight = Vec3(1,0,0);
	mUp = Vec3(0,1,0);
	mForward = f;
	mRight = r;
	mUp = u;
/*
	mForward = ArbitraryRotate(mForward,DEG2RAD(rot.y),mRight);
	mForward = ArbitraryRotate(mForward,DEG2RAD(rot.x),mUp);
	mRight = ArbitraryRotate(mRight,DEG2RAD(rot.y),mRight);
	mRight = ArbitraryRotate(mRight,DEG2RAD(rot.x),mUp);
	mUp = ArbitraryRotate(mUp,DEG2RAD(rot.y),mRight);
	mUp = ArbitraryRotate(mUp,DEG2RAD(rot.x),mUp);
*/
	mOForward = mForward;
	mORight = mRight;
	mOUp = mUp;

	bRemove = false;
	bVisible = false;

	mpPlatform = NULL;
	fDistToCamera = 0;
//	gLog.OutPut("Done\n");
}

bool Entity::BoxOverlap(Vec3 mins, Vec3 maxs)
{
	if(mMaxs.x < mins.x || mMins.x > maxs.x)
		return false;
	if(mMaxs.y < mins.y || mMins.y > maxs.y)
		return false;
	if(mMaxs.z < mins.z || mMins.z > maxs.z)
		return false;
	return true;
}
void Entity::RenderFur(const float interp)
{
}
void Entity::RenderGlow(const float interp)
{
}
void Entity::TakeDamage()
{
}
void Entity::Trigger()
{
}
char gEntName[16];
bool TriggerEntityByGName(Entity* e)
{
	if(e->iBaseType == ENT_LASER)
	{
			if(!strcmp(((Laser*)e)->Name, gEntName))
				e->Trigger();
	}
	if(e->iBaseType == ENT_TELEPORT)
	{
		if(!strcmp(((Teleporter*)e)->Name, gEntName))
			e->Trigger();
	}
	if(e->iBaseType == ENT_BUTTON)
	{
		if(!strcmp(((Switch*)e)->Name, gEntName))
			((Switch*)e)->IsToggled = false;
	}
	return false;
}
void Entity::Freeze()
{
	mOPos = mPos;
	mOForward = mForward;
	mORight = mRight;
	mOUp = mUp;

}