#include "Engine.h"

bool PruneFadeSystems(FadeSystem* f)
{
	if(f->bRemove)
	{
		delete f;
		f = NULL;
	//	gLog.OutPut("Removed Fade system!\n");
		return true;
	}
	f->Update();
	return false;
}

void FadeEngine::Update()
{
	mSystems.remove_if(PruneFadeSystems);
}
void FadeEngine::Render(const float interp)
{
	std::list<FadeSystem*>::iterator it;
	for(it=mSystems.begin(); it!=mSystems.end(); it++)
	{
		(*it)->Render(interp);
	}
}
void FadeEngine::RenderSecondary(const float interp)
{
	std::list<FadeSystem*>::iterator it;
//	for(it=mSystems.begin(); it!=mSystems.end(); it++)
	{
		mSystems.back()->RenderSecondary(interp);
//		(*it)->RenderSecondary(interp);
	}
}
void FadeEngine::Cleanup()
{
	std::list<FadeSystem*>::iterator it;
	for(it=mSystems.begin(); it!=mSystems.end(); it++)
	{
		delete (*it);
		(*it) = NULL;
	}
	mSystems.clear();
}
void FadeEngine::InsertSystem(FadeSystem* FS)
{
	mSystems.push_back(FS);
}

/************************************************/
/* FADE1: Works alongside PS_Fade1				*/
/************************************************/
void FADE_Fade1::Spawn(float life, Vec3 col)
{
	PlaySample(CHNL_FADE1, gpEng->mSnd[SND_FADE1],false);
	fVerticalBar = -200;
	Color = col;
	bRemove = false;
	iType = 0;
}
void FADE_Fade1::Update()
{
	fOVerticalBar = fVerticalBar;
	fVerticalBar += 3.0f;
	if(fVerticalBar >= 640)
		bRemove = true;
}
void FADE_Fade1::Render(const float interp)
{
	float f = UTIL_Misc::Interpolate(fVerticalBar,fOVerticalBar,interp);
	glColor3fv(Color.v);
	glBegin(GL_QUADS);
	glVertex2f(0,0); glVertex2f(640,0);
	glVertex2f(640,f-64); glVertex2f(0,f-64);

	glVertex2f(0,f-64); glVertex2f(640,f-64);
	glColor3f(0,0,0);
	glVertex2f(640,f+64); glVertex2f(0,f+64);
	glEnd();
}
/************************************************/
/* starshaped fade = growing start				*/
/************************************************/
void FADE_Star::Spawn(float life, Vec3 col)
{
	PlaySample(CHNL_FADE1, gpEng->mSnd[SND_FADE1],false);
	fSize = fOSize = 1;
	fRotation = fORotation = 0;
	Color = Vec3(1,1,1);
	bRemove = false;
	iType = 0;
}
void FADE_Star::Update()
{
	fORotation = fRotation;
	fRotation += 3;
	fOSize = fSize;
	fSize *= 1.05f;
	if(fSize >= 1000)
		bRemove = true;

}

bool PruneMotion(Motion& m)
{
	m.fAlpha -= (TIME_STEP *0.5);
	if(m.fAlpha<=0)
		return true;
	return false;
}

void FADE_Star::Render(const float interp)
{
	float s = UTIL_Misc::Interpolate(fSize,fOSize,interp);
	glColor3fv(Color.v);
	UTIL_GL::BindTexture(GL_TEXTURE_2D, gpEng->mSprites[SPR_PARTICLES].mImg);
	Vec2 UV,UVS;
	UV = gpEng->mSprites[SPR_PARTICLES].GetTileUV(1);
	UVS = gpEng->mSprites[SPR_PARTICLES].mUVsize;
	glBegin(GL_QUADS);
/*	glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex2f(320-s,240-s);
	glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex2f(320+s,240-s);
	glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex2f(320+s,240+s);
	glTexCoord2f(UV.x, UV.y);					glVertex2f(320-s,240+s);
*/
	Vec2 p(320,240);
	Vec2 locUp,locRgt;

	float r = UTIL_Misc::Interpolate(fRotation, fORotation, interp);
	UTIL_Misc::MakeVectors(r-90,locUp,locRgt);

	glTexCoord2f(UV.x, UV.y);					glVertex2fv((Vec2(p.x,p.y)-locRgt*s - locUp*s).v);
	glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex2fv((Vec2(p.x,p.y)+locRgt*s - locUp*s).v);
	glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex2fv((Vec2(p.x,p.y)+locRgt*s + locUp*s).v);
	glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex2fv((Vec2(p.x,p.y)-locRgt*s + locUp*s).v);
	glEnd();

	glDisable(GL_TEXTURE_2D);
}
/*void FADE_Star::RenderSecondary(const float interp)
{
	float s = UTIL_Misc::Interpolate(fSize,fOSize,interp);
	glColor3f(Color.x, Color.y, Color.z);
	UTIL_GL::BindTexture(GL_TEXTURE_2D, gpEng->mSprites[SPR_PARTICLESALPHA].mImg);
	Vec2 UV,UVS;
	UV = gpEng->mSprites[SPR_PARTICLESALPHA].GetTileUV(3);
	UVS = gpEng->mSprites[SPR_PARTICLESALPHA].mUVsize;
	glBegin(GL_QUADS);
	Vec2 p(320,240);
	Vec2 locUp,locRgt;

	float r = UTIL_Misc::Interpolate(fRotation, fORotation, interp);
	UTIL_Misc::MakeVectors(r-90,locUp,locRgt);

	glTexCoord2f(UV.x, UV.y);					glVertex2fv((Vec2(p.x,p.y)-locRgt*s - locUp*s).v);
	glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex2fv((Vec2(p.x,p.y)+locRgt*s - locUp*s).v);
	glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex2fv((Vec2(p.x,p.y)+locRgt*s + locUp*s).v);
	glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex2fv((Vec2(p.x,p.y)-locRgt*s + locUp*s).v);
	glEnd();

	glDisable(GL_TEXTURE_2D);
}*/
/************************************************/
/* horizontal box fade				*/
/************************************************/
void FADE_Box::Spawn(float life, Vec3 col)
{
	PlaySample(CHNL_FADE1, gpEng->mSnd[SND_FADE2],false);
	for(int x=0;x<10;x++)
		for(int y=0; y<10; y++)
			fSize[x+y*10] = 0.5f;

	fSize[0] = 0.4f;
	Color = col;
	bRemove = false;
	tileIDX = 0;
	iType = 0;
}
void FADE_Box::Update()
{
	for(int x=0;x<10;x++)
	for(int y=0;y<10;y++)
	{

		if(fSize[x+y*10]<0.5f || x>0 && fSize[(x-1)+y*10]<=0.4f ||  y>0 && fSize[x+(y-1)*10]<=0.4f)
		{
			fSize[x+y*10]-=0.02f;
			if(fSize[x+y*10]<0)
			{
				fSize[x+y*10] = 0;
			}
		}
	}
	if(fSize[(10*10)-1]<=0)
		bRemove = true;
}
void FADE_Box::Render(const float interp)
{
	glColor3fv(Color.v);
	Vec2 UV,UVS;
	UV = gpEng->mSprites[SPR_PARTICLES].GetTileUV(1);
	UVS = gpEng->mSprites[SPR_PARTICLES].mUVsize;
	float fx= 640/10.0f;
	float fy= 480/10.0f;
	glBegin(GL_QUADS);
	int idx;
	for(int x=0;x<10;x++)
	for(int y=0;y<10;y++)
	{
		idx = x+y*10;
		glVertex2f((fx*x)+(fx*fSize[idx]),			(fy*y)+(fy*fSize[idx]));
		glVertex2f((fx*(x+1))-(fx*fSize[idx]),		(fy*y)+(fy*fSize[idx]));
		glVertex2f((fx*(x+1))-(fx*fSize[idx]),		(fy*(y+1))-(fy*fSize[idx]));
		glVertex2f((fx*x)+(fx*fSize[idx]),			(fy*(y+1))-(fy*fSize[idx]));
	}

	glEnd();
}
/************************************************/
/* starshaped fadeout - shrinking start			*/
/************************************************/
void FADE_StarOut::Spawn(float life, Vec3 col)
{
	PlaySample(CHNL_FADE1, gpEng->mSnd[SND_FADE1],false);
	fSize = fOSize = 1000;
	fRotation = fORotation = 0;
	Color = col;
	bRemove = false;
	iType = 1;
}
void FADE_StarOut::Update()
{
	fORotation = fRotation;
	fRotation += 3;
	fOSize = fSize;
	fSize *= 0.95f;
	if(fSize <= 0)
		bRemove = true;


}
void FADE_StarOut::Render(const float interp)
{
	float s = UTIL_Misc::Interpolate(fSize,fOSize,interp);
	glColor3fv(Color.v);
	UTIL_GL::BindTexture(GL_TEXTURE_2D, gpEng->mSprites[SPR_PARTICLES].mImg);
	Vec2 UV,UVS;
	UV = gpEng->mSprites[SPR_PARTICLES].GetTileUV(1);
	UVS = gpEng->mSprites[SPR_PARTICLES].mUVsize;
	glBegin(GL_QUADS);
/*	glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex2f(320-s,240-s);
	glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex2f(320+s,240-s);
	glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex2f(320+s,240+s);
	glTexCoord2f(UV.x, UV.y);					glVertex2f(320-s,240+s);
*/
	Vec2 p(320,240);
	Vec2 locUp,locRgt;
	float r = UTIL_Misc::Interpolate(fRotation, fORotation, interp);
	UTIL_Misc::MakeVectors(r-90,locUp,locRgt);

	glTexCoord2f(UV.x, UV.y);					glVertex2fv((Vec2(p.x,p.y)-locRgt*s - locUp*s).v);
	glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex2fv((Vec2(p.x,p.y)+locRgt*s - locUp*s).v);
	glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex2fv((Vec2(p.x,p.y)+locRgt*s + locUp*s).v);
	glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex2fv((Vec2(p.x,p.y)-locRgt*s + locUp*s).v);
	std::list<Motion>::iterator it;
	glEnd();
	glDisable(GL_TEXTURE_2D);
}
/*void FADE_StarOut::RenderSecondary(const float interp)
{
//	gLog.OutPut("Fade out 2nd\n");
	float s = UTIL_Misc::Interpolate(fSize,fOSize,interp);
	glColor3f(Color.x, Color.y, Color.z);
	UTIL_GL::BindTexture(GL_TEXTURE_2D, gpEng->mSprites[SPR_PARTICLESALPHA].mImg);
	Vec2 UV,UVS;
	UV = gpEng->mSprites[SPR_PARTICLESALPHA].GetTileUV(3);
	UVS = gpEng->mSprites[SPR_PARTICLESALPHA].mUVsize;
	glBegin(GL_QUADS);
	Vec2 p(320,240);
	Vec2 locUp,locRgt;
	float r = UTIL_Misc::Interpolate(fRotation, fORotation, interp);
	UTIL_Misc::MakeVectors(r-90,locUp,locRgt);

	glTexCoord2f(UV.x, UV.y);					glVertex2fv((Vec2(p.x,p.y)-locRgt*s - locUp*s).v);
	glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex2fv((Vec2(p.x,p.y)+locRgt*s - locUp*s).v);
	glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex2fv((Vec2(p.x,p.y)+locRgt*s + locUp*s).v);
	glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex2fv((Vec2(p.x,p.y)-locRgt*s + locUp*s).v);
	glEnd();
	glDisable(GL_TEXTURE_2D);
}*/