#include "Engine.h"
#include "SDL_KeyMap.h"
#include "Serializer.h"
/*==========================================*/
/*=  a key state has changed               =*/
/*==========================================*/
void Engine::KeyStateAlltered(int key, bool state)
{
	if(bCfgJoystick)
	{
		if(key == SDLK_ESCAPE)
		{
			bCfgJoystick = false;
			return;
		}
		if(key<JOYSTICK_BUTTON || !state)
			return;
		pCfgButton->secondKey = key;
		iOption++;
		return;
	}

	if(key == KEY_MOUSE3.primaryKey || key == KEY_MOUSE3.secondKey)
	{	KEY_MOUSE3.Update(state);	return; }
	if(key == KEY_MOUSE1.secondKey)
	{	KEY_MOUSE1.Update(state);	return; }
	if(key == KEY_TURN_LEFT.primaryKey || key == KEY_TURN_LEFT.secondKey)
	{	KEY_TURN_LEFT.Update(state);	return; }
	if(key == KEY_TURN_RIGHT.primaryKey || key == KEY_TURN_RIGHT.secondKey)
	{	KEY_TURN_RIGHT.Update(state);	return; }
	if(key == KEY_FORWARD.primaryKey || key == KEY_FORWARD.secondKey)
	{	KEY_FORWARD.Update(state);	return; }
	if(key == KEY_BACK.primaryKey || key == KEY_BACK.secondKey)
	{	KEY_BACK.Update(state);	return; }
	if(key == KEY_CAMERA.primaryKey || key == KEY_CAMERA.secondKey)
	{	KEY_CAMERA.Update(state);	return; }
	if(key == KEY_JUMP.primaryKey || key == KEY_JUMP.secondKey)
	{	KEY_JUMP.Update(state);	}
	if(key == KEY_ACTION2.primaryKey || key == KEY_ACTION2.secondKey)
	{	KEY_ACTION2.Update(state);	; }
	if(key == KEY_QUIT.secondKey)
	{	KEY_QUIT.Update(state); KEY_ACTION2.Update(state);	return; }

	switch(key)
	{
	case SDLK_RETURN: KEY_JUMP.Update(state); break;
	case SDLK_ESCAPE: KEY_QUIT.Update(state); break;
	case SDLK_PAUSE: KEY_PAUSE.Update(state); break;
	case 1: KEY_MOUSE1.Update(state); break;
	case 2: KEY_MOUSE2.Update(state); break;
	case 3: KEY_MOUSE3.Update(state); break;

	default:
		break;
	}
}
/*==========================================*/
/*=post update on key (allows more control)=*/
/*==========================================*/
void Engine::KeysPostUpdate()
{
	KEY_QUIT.PostUpdate();
	KEY_PAUSE.PostUpdate();
	KEY_MOUSE1.PostUpdate();
	KEY_MOUSE2.PostUpdate();
	KEY_MOUSE3.PostUpdate();
	KEY_TURN_LEFT.PostUpdate();
	KEY_TURN_RIGHT.PostUpdate();
	KEY_FORWARD.PostUpdate();
	KEY_BACK.PostUpdate();
	KEY_JUMP.PostUpdate();
	KEY_ACTION2.PostUpdate();
	KEY_CAMERA.PostUpdate();
}

void Engine::SaveInput()
{
	DeleteFile("Input.dat");
	gSerializer.PutComment("Input.dat","Input Configuration");
/*	gSerializer.ReadVariable("Input.dat","PAD_SHOOT",KEY_MOUSE1.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_BOOST",KEY_MOUSE3.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_ROLLL",KEY_ROLL_LEFT.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_ROLLR",KEY_ROLL_RIGHT.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_HOVER",KEY_HOVER.secondKey);
	gSerializer.ReadVariable("Input.dat","JoySensi",fJoystickSensi);
*/
}
void Engine::LoadInput()
{
	fJoystickSensi = 80;
//	KEY_MOUSE3.primaryKey = SDLK_w;
//	KEY_MOUSE3.secondKey = JOYSTICK_BUTTON;
//	KEY_MOUSE1.secondKey = JOYSTICK_BUTTON;
	KEY_TURN_LEFT.primaryKey = SDLK_LEFT;
	KEY_TURN_LEFT.secondKey = JOYSTICK_LEFT;
	KEY_TURN_RIGHT.primaryKey = SDLK_RIGHT;
	KEY_TURN_RIGHT.secondKey = JOYSTICK_RIGHT;
	KEY_FORWARD.primaryKey = SDLK_UP;
	KEY_FORWARD.secondKey = JOYSTICK_UP;
	KEY_BACK.primaryKey = SDLK_DOWN;
	KEY_BACK.secondKey = JOYSTICK_DOWN;
	KEY_JUMP.primaryKey = SDLK_SPACE;
	KEY_JUMP.secondKey = JOYSTICK_BUTTON;
	KEY_ACTION2.primaryKey = SDLK_ESCAPE;
	KEY_ACTION2.secondKey = JOYSTICK_BUTTON+1;
	KEY_QUIT.secondKey = JOYSTICK_BUTTON+7;
	KEY_CAMERA.primaryKey = SDLK_LCTRL;
	KEY_CAMERA.secondKey = JOYSTICK_BUTTON+2;
	bCfgJoystick = false;
/*	CFileIO fIO;
	if(fIO.GetFile("Input.dat","rt"))
	{
		gSerializer.ReadVariable("Input.dat","PAD_SHOOT",KEY_MOUSE1.secondKey);
		gSerializer.ReadVariable("Input.dat","PAD_BOOST",KEY_MOUSE3.secondKey);
		gSerializer.ReadVariable("Input.dat","PAD_ROLLL",KEY_ROLL_LEFT.secondKey);
		gSerializer.ReadVariable("Input.dat","PAD_ROLLR",KEY_ROLL_RIGHT.secondKey);
		gSerializer.ReadVariable("Input.dat","PAD_HOVER",KEY_HOVER.secondKey);
		gSerializer.ReadVariable("Input.dat","JoySensi",fJoystickSensi);
	}
*/
}

//joystick
void Engine::CheckJoystickMotion(SDL_Event event)
{
//	gLog.OutPut("CheckJoystick START\n");
	if(event.jaxis.axis == 0)//horizontal?
	{
		int axis;
		if(event.jaxis.value<0)
			axis=event.jaxis.value/(32768/(fJoystickSensi/50));
		else
			axis=event.jaxis.value/(32767/(fJoystickSensi/50));

		if(axis!=oldPadAxisH)
		{
			if(oldPadAxisH<0)
				KeyStateAlltered(JOYSTICK_LEFT, false);
			if(oldPadAxisH>0)
				KeyStateAlltered(JOYSTICK_RIGHT, false);
		}
		if(axis<0)
			KeyStateAlltered(JOYSTICK_LEFT, true);
		if(axis>0)
			KeyStateAlltered(JOYSTICK_RIGHT, true);
		oldPadAxisH = axis;
	}
	if(event.jaxis.axis == 1)//Vertical?
	{
		int axis;
		if(event.jaxis.value<0)
			axis=event.jaxis.value/(32768/(fJoystickSensi/50));
		else
			axis=event.jaxis.value/(32767/(fJoystickSensi/50));
		
		if(axis!=oldPadAxisV)
		{
			if(oldPadAxisV<0)
				KeyStateAlltered(JOYSTICK_UP, false);
			if(oldPadAxisV>0)
				KeyStateAlltered(JOYSTICK_DOWN, false);
		}
		if(axis<0)
			KeyStateAlltered(JOYSTICK_UP, true);
		if(axis>0)
			KeyStateAlltered(JOYSTICK_DOWN, true);
		oldPadAxisV = axis;
	}
//	gLog.OutPut("CheckJoystick END\n");
}
void Engine::TextInput(SDL_keysym key)
{
	//text input
	if(bTextInput)
	{
//		Mix_PlayChannel(-1, sndUIMove, 0);
		if(key.sym == SDLK_RETURN || key.sym == SDLK_ESCAPE)
		{
			bTextInput = false;
			SDL_EnableUNICODE(0);
			szInputText[iTextIdx] = '\0';
			return;
		}
		else if(key.sym == SDLK_BACKSPACE)
		{
			iTextIdx>0?iTextIdx--:iTextIdx=0;
			szInputText[iTextIdx] = '_';
			return;
		}
		if(key.unicode == 0)
			return;
		if((key.unicode & 0xFF80) == 0 )
			szInputText[iTextIdx] = toupper(key.unicode & 0x7F);
		iTextIdx++;

		//szInputText[iTextIdx] = ;
		if(iTextIdx>=iTextMaxLength)
		{
			bTextInput = false;
			SDL_EnableUNICODE(0);
		}
	}
}