#include "MD2.h"
#include "FileIO.h"
#include "UTIL_Misc.h"
#include "UTIL_openGL.h"
#include "Log.h"

MD2Model::MD2Model()
{
	numVerts = 0;
	numTri = 0;
	numFrames = 0;
	numTex = 0;
	frameSize = 0;
	currentFrame = 0;
	nextFrame = 1;
	interpol = 0;
	triIndex = NULL;
	texCoords = NULL;
	verts = NULL;

}

MD2Model::~MD2Model()
{
}

bool MD2Model::Load(char* modelFile, int W, int H, unsigned int ID)
{
	imgID = ID;
	CFileIO fIO;
	if(!fIO.GetFile(modelFile,"rb"))
	{
		gLog.OutPut("Failed To Loaded Model: ");
		char* name = strrchr(modelFile,'/')+1;
		gLog.OutPut(name);
		gLog.OutPut("\n");
		return false;
	}
	int fileLen = fIO.GetFileLength();
	char* buffer = new char[fileLen+1];
	if(!fIO.ReadBinary(&buffer[0],fileLen))
	{
		gLog.OutPut("Failed To Loaded Model: ");
		char* name = strrchr(modelFile,'/')+1;
		gLog.OutPut(name);
		gLog.OutPut("\n");
		return false;
	}

	ModelHeader_t* mdlhdr = (ModelHeader_t*)buffer;
/*	char output[512];
	sprintf(output,"framesize %i\nident %i\nnumframes %i\nnumGLcmds %i\nnumSkins %i\nnumST %i\nnumTri %i\nnumVerts %i\noffsetEnd %i\noffsetFrames %i\noffsetGLcmds %i\noffsetSkins %i\noffsetST %i\noffsetTris %i\nskinHeight %i\nskinWidth %i\nversion %i\n",
		mdlhdr->framesize,
		mdlhdr->ident,
		mdlhdr->numFrames,
		mdlhdr->numGLcmds,
		mdlhdr->numSkins,
		mdlhdr->numST,
		mdlhdr->numTri,
		mdlhdr->numVerts,
		mdlhdr->offsetEnd,
		mdlhdr->offsetFrames,
		mdlhdr->offsetGLcmds,
		mdlhdr->offsetSkins,
		mdlhdr->offsetST,
		mdlhdr->offsetTris,
		mdlhdr->skinHeight,
		mdlhdr->skinWidth,
		mdlhdr->version);
		gLog.OutPut(output);//*/
	verts = new Vec3[mdlhdr->numVerts * mdlhdr->numFrames];
	numVerts = mdlhdr->numVerts;
	numFrames = mdlhdr->numFrames;
	frameSize = mdlhdr->framesize;

	int j,i;
	Frame_t* frame;
	Vec3* vertexListPointer;
	for(j=0;j<numFrames;j++)
	{
		frame = (Frame_t*)&buffer[mdlhdr->offsetFrames + frameSize*j];
		vertexListPointer = (Vec3*)&verts[numVerts*j];
		for(i=0;i<numVerts;i++)
		{
			vertexListPointer[i].x = frame->scale[0] * frame->fp[i].v[0] + frame->translate[0];
			vertexListPointer[i].y = frame->scale[1] * frame->fp[i].v[1] + frame->translate[1];
			vertexListPointer[i].z = frame->scale[2] * frame->fp[i].v[2] + frame->translate[2];
		}
	}

	numTex = mdlhdr->numST;
	texCoords = new Vec2[numTex];
	stIndex_t* stPtr = (stIndex_t*)&buffer[mdlhdr->offsetST];
	for(i=0;i<numTex;i++)
	{
		texCoords[i].x = (float)stPtr[i].s / (float)W;
		texCoords[i].y = (float)stPtr[i].t / (float)H;
	}

	numTri = mdlhdr->numTri;
	triIndex = new Mesh_t[numTri];
	Mesh_t* bufIdxPtr = (Mesh_t*)&buffer[mdlhdr->offsetTris];
	for(j=0;j<numFrames;j++)
	{
		for(i=0;i<numTri;i++)
		{
			triIndex[i].meshIndex[0] = bufIdxPtr[i].meshIndex[0];
			triIndex[i].meshIndex[1] = bufIdxPtr[i].meshIndex[1];
			triIndex[i].meshIndex[2] = bufIdxPtr[i].meshIndex[2];
			triIndex[i].stIndex[0] = bufIdxPtr[i].stIndex[0];
			triIndex[i].stIndex[1] = bufIdxPtr[i].stIndex[1];
			triIndex[i].stIndex[2] = bufIdxPtr[i].stIndex[2];
		}
	}

	currentFrame = 0;
	nextFrame = 0;
	interpol = 0;

	Vec3 vertex[6], CB, CA;
	//gLog.OutPut("Creating vertex normals\n");
	surfNormals = new Vec3[numTri * mdlhdr->numFrames];
	normals = new Vec3[mdlhdr->numVerts * mdlhdr->numFrames];
	for(j=0;j<numFrames;j++)
	{
		Vec3* vList = &verts[numVerts*j];
		for(i=0;i<numTri;i++)
		{
			vertex[0] = vList[triIndex[i].meshIndex[0]];
			vertex[1] = vList[triIndex[i].meshIndex[1]];
			vertex[2] = vList[triIndex[i].meshIndex[2]];
		
			CB = vertex[2] - vertex[1];
			CA = vertex[2] - vertex[0];
			surfNormals[i+(j*numTri)] = Normalize(Cross(CB,CA));
		}
	}
	for(j=0;j<numFrames;j++)
	{
		Vec3* vList = &verts[numVerts*j];
		Vec3* vNorms = &normals[numVerts*j];
		for(i=0;i<numTri;i++)
		{
			vertex[0] = vList[triIndex[i].meshIndex[0]];
			vertex[1] = vList[triIndex[i].meshIndex[1]];
			vertex[2] = vList[triIndex[i].meshIndex[2]];
			vNorms[triIndex[i].meshIndex[0]] += surfNormals[i+(j*numTri)];
			vNorms[triIndex[i].meshIndex[1]] += surfNormals[i+(j*numTri)];
			vNorms[triIndex[i].meshIndex[2]] += surfNormals[i+(j*numTri)];
			for(int k=0;k<numTri;k++)
			{
				if(k==i)
					continue;
				vertex[3] = vList[triIndex[k].meshIndex[0]];
				vertex[4] = vList[triIndex[k].meshIndex[1]];
				vertex[5] = vList[triIndex[k].meshIndex[2]];
				if(vertex[0]==vertex[3] || vertex[0]==vertex[4] || vertex[0]==vertex[5])
					vNorms[triIndex[k].meshIndex[0]] += surfNormals[k+(j*numTri)];
				if(vertex[1]==vertex[3] || vertex[1]==vertex[4] || vertex[1]==vertex[5])
					vNorms[triIndex[k].meshIndex[1]] += surfNormals[k+(j*numTri)];
				if(vertex[2]==vertex[3] || vertex[2]==vertex[4] || vertex[2]==vertex[5])
					vNorms[triIndex[k].meshIndex[2]] += surfNormals[k+(j*numTri)];
			}
		}
		for(i=0;i<numTri;i++)
		{
			vNorms[triIndex[i].meshIndex[0]]=vNorms[triIndex[i].meshIndex[0]].Normalize();
			vNorms[triIndex[i].meshIndex[1]]=vNorms[triIndex[i].meshIndex[1]].Normalize();
			vNorms[triIndex[i].meshIndex[2]]=vNorms[triIndex[i].meshIndex[2]].Normalize();
		}//*/
	}//*/
	gLog.OutPut("Loaded Model: ");
	char* name = strrchr(modelFile,'/')+1;
	gLog.OutPut(name);
	gLog.OutPut("\n");
	return true;
}

void MD2Model::Animate(int startFrame, int endFrame, float percent)
{
	Vec3* vList=NULL;
	Vec3* nextList=NULL;
	Vec3* vertN=NULL;

	int i;
	Vec3 p1, p2;
	Vec3 vertex[3];
	interpol = 0;

	if(startFrame>currentFrame)
		currentFrame = startFrame;
	if(startFrame<0 || endFrame<0) 
		return;
	if(startFrame>=numFrames || endFrame>=numFrames)
		return;
	if(interpol >= 1.0f)
	{
		interpol = 0.0f;
		currentFrame++;
		if(currentFrame>=endFrame)
			currentFrame = startFrame;
		nextFrame = currentFrame+1;
		if(nextFrame>=endFrame)
			nextFrame=startFrame;
	}
	vList = &verts[numVerts*currentFrame];
	nextList = &verts[numVerts*nextFrame];
	vertN = &normals[numVerts*currentFrame];

	glBegin(GL_TRIANGLES);
	for(i=0;i<numTri;i++)
	{
		p1 = vList[triIndex[i].meshIndex[0]];
		p2 = nextList[triIndex[i].meshIndex[0]];
		vertex[0] = UTIL_Misc::Interpolate(p2,p1,interpol);
		p1 = vList[triIndex[i].meshIndex[1]];
		p2 = nextList[triIndex[i].meshIndex[1]];
		vertex[1] = UTIL_Misc::Interpolate(p2,p1,interpol);
		p1 = vList[triIndex[i].meshIndex[2]];
		p2 = nextList[triIndex[i].meshIndex[2]];
		vertex[2] = UTIL_Misc::Interpolate(p2,p1,interpol);

/*		Vec3 CB = vertex[2] - vertex[1];
		Vec3 CA = vertex[2] - vertex[0];
		glNormal3fv(Normalize(Cross(CB,CA)).v);
*/
		glNormal3fv(vertN[triIndex[i].meshIndex[0]].v);
		glTexCoord2fv(texCoords[triIndex[i].stIndex[0]].v);
		glVertex3fv(vertex[0].v);
		glNormal3fv(vertN[triIndex[i].meshIndex[1]].v);
		glTexCoord2fv(texCoords[triIndex[i].stIndex[1]].v);
		glVertex3fv(vertex[1].v);
		glNormal3fv(vertN[triIndex[i].meshIndex[2]].v);
		glTexCoord2fv(texCoords[triIndex[i].stIndex[2]].v);
		glVertex3fv(vertex[2].v);
	}
	glEnd();
	interpol += percent;

}

void MD2Model::MakeDisplayList()
{
	iDisplayList = glGenLists(1);
//	char sz[32];
//	sprintf(sz,"Made Md2 ID: %i\n",iDisplayList);
//	gLog.OutPut(sz);
	glNewList(iDisplayList,GL_COMPILE_AND_EXECUTE);
	Animate(0,0,0);
	glEndList();
}
void MD2Model::RenderStatic()//uses display list
{
	glCallList(iDisplayList);
}