#include "Material.h"
#include "Engine.h"
#include "Log.h"
#include "FileIO.h"

MaterialManager::MaterialManager()
{
	iMatCounter = 0;
}
void MaterialManager::CleanUp()
{
	std::map<std::string, Material*>::iterator m;
	for(m=mMaterials.begin(); m!=mMaterials.end(); m++)
	{
		std::map<unsigned int, std::list<unsigned int> >::iterator i;
		for(i=(*m).second->mPass.begin(); i!=(*m).second->mPass.end(); i++)
		{
			std::list<unsigned int>::iterator k;
			for(k=(*i).second.begin(); k!=(*i).second.end(); k++)
			{
				char name[64];
				if(gpEng->mTexMgr.GetTextureName((*k),name))
					gpEng->mTexMgr.FreeTexture(name);
			}
		}
		delete (*m).second;
		(*m).second = NULL;
	}
	mMaterials.clear();
	mIDMaterials.clear();
	iMatCounter = 0;
}
void MaterialManager::FreeMaterial(Material* m)
{
	std::map<unsigned int, std::list<unsigned int> >::iterator i;
	for(i=m->mPass.begin(); i!=m->mPass.end(); i++)
	{
		std::list<unsigned int>::iterator k;
		for(k=(*i).second.begin(); k!=(*i).second.end(); k++)
		{
			char name[64];
			if(gpEng->mTexMgr.GetTextureName((*k),name))
				gpEng->mTexMgr.FreeTexture(name);
		}
	}
	mMaterials.erase(m->name);
	delete m;
	m = NULL;
}
Material* MaterialManager::LoadMaterial(char* dir, char* name)
{
	std::map<std::string, Material*>::iterator it;
	it=mMaterials.find(name);
	if(it != mMaterials.end())
		return (*it).second;

	gLog.OutPut("Loading Material: ");
	gLog.OutPut(name);
	gLog.OutPut("\n");
	Material* m = new Material;
	m->ambient[0] = 0.5; 
	m->ambient[1] = 0.5; 
	m->ambient[2] = 0.5; 
	m->ambient[3] = 1.0; 
	m->diffuse[0] = 0.5; 
	m->diffuse[1] = 0.5; 
	m->diffuse[2] = 0.5; 
	m->diffuse[3] = 1.0; 
	m->specular[0] = 0.5; 
	m->specular[1] = 0.5; 
	m->specular[2] = 0.5; 
	m->specular[3] = 1.0;
	m->Shininess = 100;
	m->fPulseTime = 0.0;
	m->iPulseType = 0;
	m->rotation = 0.0;
	m->name=name;
	m->bSpheremap = false;
	if(!ParseMaterial(dir, name,m))
	{
		gLog.OutPut("Failed to Parse Material: ");
		gLog.OutPut(name);
		gLog.OutPut("!\n");
		delete m;
		m = NULL;
		return NULL;
	}
	
	m->ID = iMatCounter;
	mIDMaterials[iMatCounter] = m;
	mMaterials[name] = m;
	iMatCounter++;
	return m;
}

bool MaterialManager::ParseMaterial(char* dir, char* name, Material* m)
{
	CFileIO fIO;
	char file[256];
	sprintf(file, "%s/%s.mat", dir, name);
	if(!fIO.GetFile(file,"rt"))
		return false;
	char* line = NULL;
	while((line=fIO.ReadLine()))
	{
		if((int)strlen(line)<=1)
			continue;
		char* tok = NULL;
		tok = strtok(line,"		\n");
		if(!strncmp(tok, "//", 2))//its a comment so ignore it
			continue;

		if(!strncmp(tok, "#AMB", 4))//parse the ambient color
		{
			tok = strtok(NULL,",");
			m->ambient[0] = atof(tok);
			tok = strtok(NULL,",");
			m->ambient[1] = atof(tok);
			tok = strtok(NULL,",");
			m->ambient[2] = atof(tok);
			tok = strtok(NULL,"		\n");
			m->ambient[3] = atof(tok);
			continue;
		}
		if(!strncmp(tok, "#DIF", 4))//parse the diffuse color
		{
			tok = strtok(NULL,",\n");
			m->diffuse[0] = atof(tok);
			tok = strtok(NULL,",\n");
			m->diffuse[1] = atof(tok);
			tok = strtok(NULL,",\n");
			m->diffuse[2] = atof(tok);
			tok = strtok(NULL,"		\n");
			m->diffuse[3] = atof(tok);
			continue;
		}
		if(!strncmp(tok, "#SPEC", 5))//parse the specular color
		{
			tok = strtok(NULL,",");
			m->specular[0] = atof(tok);
			tok = strtok(NULL,",");
			m->specular[1] = atof(tok);
			tok = strtok(NULL,",");
			m->specular[2] = atof(tok);
			tok = strtok(NULL,"		\n");
			m->specular[3] = atof(tok);
			continue;
		}
		if(!strncmp(tok, "#SHINE", 6))//parse the shininess factor
		{
			tok = strtok(NULL,"		\n");
			m->Shininess = atoi(tok);
			continue;
		}
		if(!strncmp(tok, "#SCROLL", 7))//parse the scroll vector
		{
			tok = strtok(NULL,",");
			m->scroll.x = atof(tok);
			tok = strtok(NULL,"		\n");
			m->scroll.y = atof(tok);
			continue;
		}
		if(!strncmp(tok, "#PULSE", 6))//parse the pulse command
		{
			tok = strtok(NULL,",");
			m->pulseTarget[0] = atof(tok);
			tok = strtok(NULL,",");
			m->pulseTarget[1] = atof(tok);
			tok = strtok(NULL,",");
			m->pulseTarget[2] = atof(tok);
			tok = strtok(NULL,"		\n");
			m->fPulseTime = atof(tok);
			if(m->fPulseTime<0.01)
				m->fPulseTime=0.01;
			tok = strtok(NULL,"		\n");
			m->iPulseType = atoi(tok);
			continue;
		}
		if(!strncmp(tok, "#ROT", 7))//parse the rotation value
		{
			tok = strtok(NULL,"		\n");
			m->rotation = atof(tok);
			continue;
		}
		if(!strncmp(tok, "#BASE", 5))//parse a base material
		{
			char* directory = strtok(NULL,"		\n");
			char* filename = strtok(NULL,"		\n");
			unsigned int ID=0;
			if(!(ID=gpEng->mTexMgr.LoadTexture(true,directory,filename,GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
				return false;
			m->mPass[BASE_PASS].push_back(ID);
			continue;
		}
		if(!strncmp(tok, "#BUMP", 5))//parse a base material
		{
			char* directory = strtok(NULL,"		\n");
			char* filename = strtok(NULL,"		\n");
			unsigned int ID=0;
			if(!(ID=gpEng->mTexMgr.LoadTexture(true,directory,filename,GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
				return false;
			m->mPass[BUMP_PASS].push_back(ID);
			continue;
		}
		if(!strncmp(tok, "#GLOW", 5))//parse a glow material
		{
			char* directory = strtok(NULL,"		\n");
			char* filename = strtok(NULL,"		\n");
			m->pulseTarget[0] = atof(tok);
			unsigned int ID=0;
			if(!(ID=gpEng->mTexMgr.LoadTexture(true,directory,filename,GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
				return false;
			m->mPass[GLOW_PASS].push_back(ID);
			continue;
		}
		if(!strncmp(tok, "#SPHERE", 6))
		{
			m->bSpheremap = true;
		}

	}

	return true;
}

void MaterialManager::SetMaterialAttributes(Material* m, int pass, const float interp)
{

	if(pass != DEPTH_PASS)
	{
		UTIL_GL::BindMultiTexture(GL_TEXTURE0,m->mPass[pass].front());
/*		if(m->bSpheremap)
		{
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			glTexGeni(GL_TEXTURE_GEN_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
			glTexGeni(GL_TEXTURE_GEN_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		}//*/

		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		glTranslatef(m->scroll.x*(gpEng->miInternalClock+interp), m->scroll.y*(gpEng->miInternalClock+interp), 0);
		glRotatef(m->rotation*(gpEng->miInternalClock+interp),0,0,1);
		glMatrixMode(GL_MODELVIEW);
		if(pass == BASE_PASS)
			gpEng->mShdrMgr.PassSampler2D("bumpMap",1,m->mPass[BUMP_PASS].front());

	}
	else
	{
		glDisable(GL_TEXTURE_2D);
	}
	if(pass == BASE_PASS)
	{
		float amb[4] = {m->ambient[0],m->ambient[1],m->ambient[2],m->ambient[3]};
		float dif[4] = {m->diffuse[0],m->diffuse[1],m->diffuse[2],m->diffuse[3]};
		float spec[4] = {m->specular[0],m->specular[1],m->specular[2],m->specular[3]};
		if(m->fPulseTime>0 && m->iPulseType<2)
		{
			int clock = gpEng->miInternalClock% (int)(m->fPulseTime*(1.0/TIME_STEP));
			bool pulse = gpEng->miInternalClock/(int)(m->fPulseTime*(1.0/TIME_STEP))%2?true:false;
			float time = ((float)clock / (m->fPulseTime*(1.0/TIME_STEP)));
			if(pulse)time = 1.0-time;
			for(int n=0; n<3; n++)
			{
				amb[n] = amb[n] * UTIL_Misc::Interpolate(amb[n]*m->pulseTarget[n],amb[n],time);
				dif[n] = dif[n] * UTIL_Misc::Interpolate(dif[n]*m->pulseTarget[n],dif[n],time);
				spec[n] = spec[n] * UTIL_Misc::Interpolate(spec[n]*m->pulseTarget[n],spec[n],time);
			}
		}
		glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, dif);
		glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
		glMateriali(GL_FRONT, GL_SHININESS, m->Shininess);

	}
	else if(pass == GLOW_PASS)
	{
		if(m->fPulseTime>0 && m->iPulseType!=1)
		{
			int clock = gpEng->miInternalClock% (int)(m->fPulseTime*(1.0/TIME_STEP));
			bool pulse = gpEng->miInternalClock/(int)(m->fPulseTime*(1.0/TIME_STEP))%2?true:false;
			float time = ((float)clock / (m->fPulseTime*(1.0/TIME_STEP)));
			if(pulse)time = 1.0-time;
			float col[3] = {1,1,1};
			for(int n=0; n<3; n++)
				col[n] = col[n] * UTIL_Misc::Interpolate(col[n]*m->pulseTarget[n],col[n],time);
			glColor3fv(col);

		}
	}
}

void MaterialManager::SetMaterialLightOnly(Material* m)
{
	float amb[4] = {m->ambient[0],m->ambient[1],m->ambient[2],m->ambient[3]};
	float dif[4] = {m->diffuse[0],m->diffuse[1],m->diffuse[2],m->diffuse[3]};
	float spec[4] = {m->specular[0],m->specular[1],m->specular[2],m->specular[3]};

	glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, dif);
	glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
	glMateriali(GL_FRONT, GL_SHININESS, m->Shininess);

}