#include "Engine.h"

bool SortFacesByMaterial(Face_t f1, Face_t f2)
{
	if(f1.mattID < f2.mattID)
		return true;
	return false;
}
bool SortFacesMaterial(Face_t* f1, Face_t* f2)
{
	if(f1->mattID < f2->mattID)
		return true;
	return false;
}
bool OBJ::Load(char* dir, char* name, float scale)
{
	CFileIO fIO;
	char filename[256];
	gLog.OutPut("Loading Model Data...");
	gLog.OutPut(name);
	gLog.OutPut("\n");
	sprintf(filename, "%s/%s", dir,name);
	if(!fIO.GetFile(filename,"rt"))
	{
		gLog.OutPut("Failed to load Model: ");
		gLog.OutPut(name);
		gLog.OutPut("!\n");
		return false;
	}

	mCurrentMaterialCounter = 0;
	ParseFile(fIO,dir);

	mMins = Vec3(99999,99999,99999);
	mMaxs = Vec3(-99999,-99999,-99999);
	if(!mVerts.empty())
	{
		std::vector<Vec3>::iterator v;
		for(v=mVerts.begin(); v<mVerts.end(); v++)
		{
			(*v) *= scale;

			if((*v).x < mMins.x)
				mMins.x = (*v).x;
			if((*v).y < mMins.y)
				mMins.y = (*v).y;
			if((*v).z < mMins.z)
				mMins.z = (*v).z;

			if((*v).x > mMaxs.x)
				mMaxs.x = (*v).x;
			if((*v).y > mMaxs.y)
				mMaxs.y = (*v).y;
			if((*v).z > mMaxs.z)
				mMaxs.z = (*v).z;
		}
	}
//	mFaces.sort(SortFacesByMaterial);
//	BuildVertexArray();
	gLog.OutPut("Done...\n");
	return true;
}

void OBJ::BuildVertexArray()
{
	gLog.OutPut("Building Model Buffer...\n");
	//pack the vertex data into a 32byte aligned struct
	mNumVerts =(int)mFaces.size()*3;
	char szVerts[64];
	sprintf(szVerts,"Vertex Count %i\n",mNumVerts);
	gLog.OutPut(szVerts);
	vArray = new VertexData[mNumVerts];
	std::list<Face_t>::iterator f;
	int n=0;
	for(f=mFaces.begin(); f!=mFaces.end(); f++)
	{
		for(int i=0;i<3;i++,n++)
		{
			vArray[n].x = mVerts[(*f).vertex[i]].x;
			vArray[n].y = mVerts[(*f).vertex[i]].y;
			vArray[n].z = mVerts[(*f).vertex[i]].z;
			vArray[n].nx = mNormals[(*f).normal[i]].x;
			vArray[n].ny = mNormals[(*f).normal[i]].y;
			vArray[n].nz = mNormals[(*f).normal[i]].z;
			vArray[n].s = mUVs[(*f).uv[i]].x;
			vArray[n].t = mUVs[(*f).uv[i]].y;
			(*f).indices[i] = (unsigned short)n;
		}
	}
	//create vertex buffer
	glGenBuffers(1, &miVBOID);
	glBindBuffer(GL_ARRAY_BUFFER, miVBOID);
	glBufferData(GL_ARRAY_BUFFER, mNumVerts*sizeof(VertexData), NULL, GL_STATIC_DRAW);
	glBufferSubData(GL_ARRAY_BUFFER, 0, mNumVerts*sizeof(VertexData), &vArray[0]);
	glBindBuffer(GL_ARRAY_BUFFER, 0);


}
//void OBJ::RenderBuffer(std::vector<unsigned short> indices)
void OBJ::RenderBuffer(bool bmat)
{
	if(bmat)
	gpEng->mMatMgr.SetMaterialAttributes(mMaterials[mFaces.front().mattID].pMat,BASE_PASS,0);
	glBindBuffer(GL_ARRAY_BUFFER, miVBOID);
	glVertexPointer(3, GL_FLOAT, 32, BUFFER_OFFSET(0));
	glNormalPointer(GL_FLOAT, 32, BUFFER_OFFSET(12));
	glTexCoordPointer(2, GL_FLOAT, 32, BUFFER_OFFSET(24));
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);//*/

//	glDrawElements(GL_TRIANGLES,(int)indices.size(),GL_UNSIGNED_SHORT,&indices[0]);
	glDrawArrays(GL_TRIANGLES,0,mNumVerts);
	glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	glBindBuffer(GL_ARRAY_BUFFER, 0);//*/
}
void OBJ::RenderLerpBuffer(bool bmat, unsigned int VBO2ID)
{
	if(bmat)
	gpEng->mMatMgr.SetMaterialAttributes(mMaterials[mFaces.front().mattID].pMat,BASE_PASS,0);
	glBindBuffer(GL_ARRAY_BUFFER, miVBOID);
	glVertexPointer(3, GL_FLOAT, 32, BUFFER_OFFSET(0));
	glNormalPointer(GL_FLOAT, 32, BUFFER_OFFSET(12));
	glClientActiveTextureARB(GL_TEXTURE0_ARB); 
	glTexCoordPointer(2, GL_FLOAT, 32, BUFFER_OFFSET(24));
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);//*/

	glBindBuffer(GL_ARRAY_BUFFER, VBO2ID);
	glClientActiveTextureARB(GL_TEXTURE1_ARB); 
	glTexCoordPointer(3, GL_FLOAT, 32, BUFFER_OFFSET(0));
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);//*/
	glClientActiveTextureARB(GL_TEXTURE2_ARB); 
	glTexCoordPointer(3, GL_FLOAT, 32, BUFFER_OFFSET(12));
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);//*/

//	glDrawElements(GL_TRIANGLES,(int)indices.size(),GL_UNSIGNED_SHORT,&indices[0]);
	glDrawArrays(GL_TRIANGLES,0,mNumVerts);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glClientActiveTextureARB(GL_TEXTURE1_ARB); 
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glClientActiveTextureARB(GL_TEXTURE0_ARB); 


	glBindBuffer(GL_ARRAY_BUFFER, miVBOID);
	glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);

	glBindBuffer(GL_ARRAY_BUFFER, 0);//*/
}

void OBJ::ParseFile(CFileIO &fIO, char* dir)
{
	gLog.OutPut("Parsing World Data...\n");

	char* line;
	while(line=fIO.ReadLine())
	{
		char* tok;
		tok = strtok(line," \n");
		//find out what we are parsing
		if(!strcmp(tok,"#"))//its comment, skip it
			continue;
		if(!strcmp(tok,"mtllib"))//its material file, skip it for now
			continue;
		if(!strcmp(tok,"s"))//no idea, skip it
			continue;

		if(!strcmp(tok,"v"))//vertex, parse it
		{
			Vec3 v;
			v.x = (float)atof(strtok(NULL," "));
			v.y = (float)atof(strtok(NULL," "));
			v.z = (float)atof(strtok(NULL,"\n"));
			mVerts.push_back(v);
		}
		if(!strcmp(tok,"vt"))//texture coord, parse it
		{
			Vec2 v;
			v.x = (float)atof(strtok(NULL," "));
			v.y = (float)atof(strtok(NULL,"\n"));
			mUVs.push_back(v);
		}
		if(!strcmp(tok,"vn"))//normal, parse it
		{
			Vec3 v;
			v.x = (float)atof(strtok(NULL," "));
			v.y = (float)atof(strtok(NULL," "));
			v.z = (float)atof(strtok(NULL,"\n"));
			mNormals.push_back(v);
		}
		if(!strcmp(tok,"usemtl"))//material, skip it for now
//			continue;
		{
			Material_t m;
			tok = strtok(NULL,"_");
			tok = strtok(NULL,"\n");
			strcpy(tok,tok);
//			if(!(m.texID = gpEng->mTexMgr.LoadTexture(dir,tok,GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT)))
//				continue;
			m.pMat = NULL;
			char texname[32];
			strncpy(texname,tok,(int)strlen(tok)-4);
			texname[(int)strlen(tok)-4]='\0';
			if(!(m.pMat = gpEng->mMatMgr.LoadMaterial("Data/Materials",texname)))
				continue;
			m.ID = mCurrentMaterialCounter++;
			mMaterials.push_back(m);
		}//*/
		if(!strcmp(tok,"f"))//face, parse it
		{
			Face_t f;
			f.mattID = mMaterials.back().ID;
			f.vertex[0] = atoi(strtok(NULL,"/")) - 1;
			f.uv[0] = atoi(strtok(NULL,"/")) - 1;
			f.normal[0] = atoi(strtok(NULL," ")) - 1;

			f.vertex[1] = atoi(strtok(NULL,"/")) - 1;
			f.uv[1] = atoi(strtok(NULL,"/")) - 1;
			f.normal[1] = atoi(strtok(NULL," ")) - 1;

			f.vertex[2] = atoi(strtok(NULL,"/")) - 1;
			f.uv[2] = atoi(strtok(NULL,"/")) - 1;
			f.normal[2] = atoi(strtok(NULL,"\n")) - 1;

			mFaces.push_back(f);
		}
	}
}

void OBJ::Render(bool mat)
{
	if(!mFaces.empty())
	{
		std::list<Face_t>::iterator f;
		unsigned int curID = mFaces.front().mattID;
		if(mat)
		{
		//	UTIL_GL::BindMultiTexture(GL_TEXTURE0,curID);
			gpEng->mMatMgr.SetMaterialAttributes(mMaterials[mFaces.front().mattID].pMat,BASE_PASS,0);
		}
		glBegin(GL_TRIANGLES);
		for(f=mFaces.begin(); f!=mFaces.end(); f++)
		{
			if(mat)
			{
				if(curID != (*f).mattID)
				{
					curID = (*f).mattID;
					glEnd();
					//UTIL_GL::BindMultiTexture(GL_TEXTURE0,curID);
					gpEng->mMatMgr.SetMaterialAttributes(mMaterials[(*f).mattID].pMat,BASE_PASS,0);
					glBegin(GL_TRIANGLES);	
				}
			}//*/
			glNormal3fv(mNormals[(*f).normal[0]].v);
			glTexCoord2fv(mUVs[(*f).uv[0]].v);
			glVertex3fv(mVerts[(*f).vertex[0]].v);
			glNormal3fv(mNormals[(*f).normal[1]].v);
			glTexCoord2fv(mUVs[(*f).uv[1]].v);
			glVertex3fv(mVerts[(*f).vertex[1]].v);
			glNormal3fv(mNormals[(*f).normal[2]].v);
			glTexCoord2fv(mUVs[(*f).uv[2]].v);
			glVertex3fv(mVerts[(*f).vertex[2]].v);

		}
		glEnd();
	}
}