#pragma once
#include "Vector.h"
#include <vector>
#include <list>
#include "FileIO.h"
#include "VertexArray.h"
#include "Material.h"



struct Material_t
{
	unsigned int ID;//my id
	Material* pMat;//pointer to a material

};

struct Face_t
{
	unsigned  vertex[3];
	unsigned int uv[3];
//	unsigned int normal[3];
	unsigned int normal[3];//we are using flat shading so we only need to store one normal per face
	unsigned int mattID;//so we can find its texture

	unsigned char refCounter;//since I will not split faces, instead I will use a reference counter to discard allready drawn faces
	unsigned short indices[3];
};

class OBJ
{
public:
	bool Load(char* dir, char* name, float scale);
	void BuildVertexArray();
	void Render(bool mat);

	Vec3 mMins, mMaxs;
	std::vector<Vec3> mVerts;
	std::list<Face_t> mFaces;
	std::vector<Vec2> mUVs;
	std::vector<Vec3> mNormals;
	std::vector<Material_t>	mMaterials;

	VertexData* vArray;
	int mNumVerts;//vertex array size
	unsigned int miVBOID;
//	void RenderBuffer(std::vector<unsigned short> indices);
	void RenderBuffer(bool bmat);
	void RenderLerpBuffer(bool bmat, unsigned int VBO2ID);
private:
	void ParseFile(CFileIO &fIO, char* dir);

	unsigned int mCurrentMaterialCounter;
};

bool SortFacesByMaterial(Face_t f1, Face_t f2);
bool SortFacesMaterial(Face_t* f1, Face_t* f2);
