#include "Engine.h"

bool PruneParticleSystems(ParticleSystem* ps)
{
	if(ps->bRemove)
	{
		ps->mParticles.clear();
		delete ps;
		ps = NULL;
		//gLog.OutPut("Particle system removed!\n");
		return true;
	}
	ps->Update();
	return false;
}
void ParticleSystem::RemoveDeadParticle(std::vector<Particle>::iterator i)
{
	std::swap((*i),mParticles.back());
	mParticles.pop_back();
//	gParticleCount--;
}

void ParticleEngine::Update()
{
	mSystems3D.remove_if(PruneParticleSystems);
	mSystems3DAlpha.remove_if(PruneParticleSystems);
	mSystems2D.remove_if(PruneParticleSystems);
}
void ParticleEngine::Render3D(const float interp)
{
	if(!mSystems3D.empty())
	{
		GLfloat tempMatrix[16];
		glGetFloatv(GL_MODELVIEW_MATRIX,tempMatrix);
		Vec3 camRight = Vec3(tempMatrix[0],tempMatrix[4],tempMatrix[8]);
		Vec3 camUp = Vec3(tempMatrix[1],tempMatrix[5],tempMatrix[9]);
		Vec3 camFwd = Vec3(tempMatrix[2],tempMatrix[6],tempMatrix[10]);

		Vec2 UV, UVS, s;
		UVS = gpEng->mSprites[SPR_PARTICLES].mUVsize;
		UTIL_GL::BindTexture(GL_TEXTURE_2D, gpEng->mSprites[SPR_PARTICLES].mImg);
		Vec3 p;
	//	float r;
		glBegin(GL_QUADS);
		std::list<ParticleSystem*>::iterator it;
		for(it=mSystems3D.begin(); it!=mSystems3D.end(); it++)
		{
			std::vector<Particle>::iterator pt;
			for(pt=(*it)->mParticles.begin(); pt<(*it)->mParticles.end(); pt++)
			{
				UV = gpEng->mSprites[SPR_PARTICLES].GetTileUV((*pt).ID);
				p = UTIL_Misc::Interpolate((*pt).P, (*pt).oP, interp);
				s = UTIL_Misc::Interpolate((*pt).S, (*pt).oS, interp);
				
				glColor3fv((*pt).Col.v);
				glTexCoord2f(UV.x, UV.y);					glVertex3fv((p-camRight*s.x - camUp*s.y).v);
				glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex3fv((p+camRight*s.x - camUp*s.y).v);
				glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex3fv((p+camRight*s.x + camUp*s.y).v);
				glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex3fv((p-camRight*s.x + camUp*s.y).v);
			}
		}
		glEnd();
		UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		UTIL_GL::BindTexture(GL_TEXTURE_2D, gpEng->mSprites[SPR_PARTICLESALPHA].mImg);
		glBegin(GL_QUADS);
		for(it=mSystems3DAlpha.begin(); it!=mSystems3DAlpha.end(); it++)
		{
			std::vector<Particle>::iterator pt;
			for(pt=(*it)->mParticles.begin(); pt<(*it)->mParticles.end(); pt++)
			{
				UV = gpEng->mSprites[SPR_PARTICLES].GetTileUV((*pt).ID);
				p = UTIL_Misc::Interpolate((*pt).P, (*pt).oP, interp);
				s = UTIL_Misc::Interpolate((*pt).S, (*pt).oS, interp);
				
				glColor4f((*pt).Col.x,(*pt).Col.y,(*pt).Col.z,(*pt).fLife);
				glTexCoord2f(UV.x, UV.y);					glVertex3fv((p-camRight*s.x - camUp*s.y).v);
				glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex3fv((p+camRight*s.x - camUp*s.y).v);
				glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex3fv((p+camRight*s.x + camUp*s.y).v);
				glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex3fv((p-camRight*s.x + camUp*s.y).v);
			}
		}
		glEnd();
		glDisable(GL_TEXTURE_2D);
	}
}
void ParticleEngine::Render2D(const float interp)
{
	if(!mSystems2D.empty())
	{
		UTIL_GL::BindTexture(GL_TEXTURE_2D, gpEng->mSprites[SPR_PARTICLES].mImg);
		Vec2 UV, UVS, s;
		UVS = gpEng->mSprites[SPR_PARTICLES].mUVsize;
		Vec3 p;
		float r;
		glBegin(GL_QUADS);
		std::list<ParticleSystem*>::iterator it;
		for(it=mSystems2D.begin(); it!=mSystems2D.end(); it++)
		{
			std::vector<Particle>::iterator pt;
			for(pt=(*it)->mParticles.begin(); pt<(*it)->mParticles.end(); pt++)
			{
				UV = gpEng->mSprites[SPR_PARTICLES].GetTileUV((*pt).ID);
				p = UTIL_Misc::Interpolate((*pt).P, (*pt).oP, interp);
				s = UTIL_Misc::Interpolate((*pt).S, (*pt).oS, interp);
				
				glColor3fv((*pt).Col.v);
				Vec2 locUp,locRgt;
				r = UTIL_Misc::Interpolate((*pt).R, (*pt).oR, interp);
				UTIL_Misc::MakeVectors(r-90,locUp,locRgt);

				glTexCoord2f(UV.x, UV.y);					glVertex2fv((Vec2(p.x,p.y)-locRgt*s.x - locUp*s.y).v);
				glTexCoord2f(UV.x+UVS.x, UV.y);				glVertex2fv((Vec2(p.x,p.y)+locRgt*s.x - locUp*s.y).v);
				glTexCoord2f(UV.x+UVS.x, UV.y+UVS.y);		glVertex2fv((Vec2(p.x,p.y)+locRgt*s.x + locUp*s.y).v);
				glTexCoord2f(UV.x, UV.y+UVS.y);				glVertex2fv((Vec2(p.x,p.y)-locRgt*s.x + locUp*s.y).v);

			}
		}
		glEnd();
		glDisable(GL_TEXTURE_2D);
	}
}

void ParticleEngine::CleanUp()
{
	std::list<ParticleSystem*>::iterator it;
	for(it=mSystems3D.begin(); it!=mSystems3D.end(); it++)
	{
		delete (*it);
		(*it) = NULL;
	}
	mSystems3D.clear();
	for(it=mSystems3DAlpha.begin(); it!=mSystems3DAlpha.end(); it++)
	{
		delete (*it);
		(*it) = NULL;
	}
	mSystems3DAlpha.clear();
	for(it=mSystems2D.begin(); it!=mSystems2D.end(); it++)
	{
		delete (*it);
		(*it) = NULL;
	}
	mSystems2D.clear();
}
void ParticleEngine::InsertSystem(ParticleSystem* PS, bool Is2D)
{
	Is2D?
		mSystems2D.push_back(PS):
		mSystems3D.push_back(PS);
}
void ParticleEngine::InsertAlphaSystem(ParticleSystem* PS)
{
	mSystems3DAlpha.push_back(PS);
}