#include "ParticleSystem.h"
#include "Engine.h"

/******************************************/
/* Particle fade effect 1	(2d)		  */
/******************************************/
void PS_Fade1::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	fVerticalBar = -64;
	bRemove = false;
	Emit(2 + UTIL_Misc::Rand()%2);
	mParticles.reserve(1500);
}
void PS_Fade1::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		Particle p;
		p.Col = Vec3(1,1,1);
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(0.5)+1.5f;
		p.ID = 1;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(1,1);
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(2.5);
		p.Vel = Vec3(UTIL_Misc::RandFloat(5.0)-2.5,UTIL_Misc::RandFloat(3.0)+2.0,0);
		p.P = p.oP = Vec3(UTIL_Misc::Rand()%640,fVerticalBar + UTIL_Misc::Rand()%64, 0);
		mParticles.push_back(p);
	}
}
void PS_Fade1::Update()
{
	fVerticalBar += 3.0f;
	if(fVerticalBar >= 740)
	{
		bRemove = true;
		mParticles.clear();
		return;
	}
//	if(fVerticalBar >= 640)
//		fVerticalBar = -64;

	Emit(2 + UTIL_Misc::Rand()%2);

	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		(*pt).oP = (*pt).P;
		(*pt).Vel *= 0.095f;
		(*pt).P += (*pt).Vel;
		(*pt).oS = (*pt).S;
		(*pt).S += Vec2(0.3f,0.3f);
//		if((*pt).S.x > 32) (*pt).S.x = 32;
//		if((*pt).S.y > 32) (*pt).S.y = 32;
		(*pt).oR = (*pt).R;
		(*pt).R += (*pt).vR;
		(*pt).fLife -= TIME_STEP;
		(*pt).Col.x = (*pt).Col.y = (*pt).Col.z =  UTIL_Misc::TimeRamp(1,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}
}

/******************************************/
/* Glittery star effect					  */
/******************************************/
void PS_Stars::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	bRemove = false;
	//Emit(5 + UTIL_Misc::Rand()%25);
	bAlive = true;
	iEmitTime = 0;
	mParticles.reserve(100);
}
void PS_Stars::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		Particle p;
		p.Col = Vec3(1,1,0);
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(0.5)+0.5f;
		p.ID = 1;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(0.03,0.03);
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(2.5);
		p.Vel = up * (UTIL_Misc::RandFloat(0.01f) + 0.002f);
		p.P = p.oP = mPos + Vec3(UTIL_Misc::RandFloat(0.5f)-0.25f, UTIL_Misc::RandFloat(0.5f)-0.25f, UTIL_Misc::RandFloat(0.5f)-0.25f) + up * 0.1f;
		mParticles.push_back(p);
	}
	iEmitTime = UTIL_Misc::Rand()%30;
}
void PS_Stars::Update()
{
	if(bAlive)
	{
		iEmitTime--;
		if(iEmitTime<=0)
			Emit(5 + UTIL_Misc::Rand()%3);
	}

	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		(*pt).oP = (*pt).P;
		(*pt).Vel *= 0.95f;
		(*pt).P += (*pt).Vel;
		(*pt).oS = (*pt).S;
		(*pt).S += Vec2(0.003f,0.003f);
//		if((*pt).S.x > 32) (*pt).S.x = 32;
//		if((*pt).S.y > 32) (*pt).S.y = 32;
		(*pt).oR = (*pt).R;
		(*pt).R += (*pt).vR;
		(*pt).fLife -= TIME_STEP;
		(*pt).Col.x = UTIL_Misc::TimeRamp(fStarColor.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp(fStarColor.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp(fStarColor.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(!bAlive && mParticles.empty())
		bRemove = true;
}

/******************************************/
/* Ball exploding						  */
/******************************************/
void PS_BallExplode::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	bRemove = false;
	mPos = p;
	Color = c;
	IDtype = id;
	iChildEmit = 0;
	mParticles.reserve(1000);
	Emit(1000);
}
void PS_BallExplode::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		Particle p;
		p.Col = p.MaxColor = Color* (UTIL_Misc::RandFloat(0.5)+0.5);
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(1.5)+0.5f;
		p.ID = IDtype;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(0.3,0.3);
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(2.5);
		p.Vel = gpEng->mPlayer.mUp * (((UTIL_Misc::RandFloat(0.01f) + 0.002f)-0.006)*UTIL_Misc::RandFloat(15)) + 
			gpEng->mPlayer.mRight * (((UTIL_Misc::RandFloat(0.01f) + 0.002f)-0.006)*UTIL_Misc::RandFloat(15)) + 
			gpEng->mPlayer.mForward * (((UTIL_Misc::RandFloat(0.01f) + 0.002f)-0.006)*UTIL_Misc::RandFloat(15));
		p.P = p.oP = mPos + Normalize(p.Vel) * 0.25f;
		mParticles.push_back(p);
	}
}
void PS_BallExplode::Update()
{
	std::vector<Particle>::iterator pt;
/*	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		if((*pt).vR != -1)
		{
			iChildEmit++;
			if((iChildEmit%10) == 0)
			{
				Particle p = (*pt);
				p.Vel = Vec3(0,0,0);
				p.oS = p.S;
				p.oR = p.R;
				p.vR = -1;
				p.ID = 0;
				mParticles.push_back(p);
			}
		}
	}*/
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		iChildEmit++;
/*		if((*pt).vR == -1)
		{
			(*pt).fLife -= TIME_STEP;
			(*pt).oP = (*pt).P;
			//(*pt).Vel += Up * 0.0001f;
			//(*pt).Vel *= 0.95f;
			//(*pt).P += (*pt).Vel;
		}
		else
*/		{
			(*pt).oP = (*pt).P;
			(*pt).Vel += Up * 0.0001f;
			(*pt).Vel *= 0.985f;
			(*pt).P += (*pt).Vel;
			(*pt).oS = (*pt).S;
			(*pt).S -= Vec2(0.003f,0.003f);
	//		if((*pt).S.x > 32) (*pt).S.x = 32;
	//		if((*pt).S.y > 32) (*pt).S.y = 32;
			(*pt).oR = (*pt).R;
			(*pt).R += (*pt).vR;
			(*pt).fLife -= TIME_STEP;
		}
		(*pt).Col.x = UTIL_Misc::TimeRamp((*pt).MaxColor.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp((*pt).MaxColor.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp((*pt).MaxColor.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(mParticles.empty())
		bRemove = true;
}
/******************************************/
/* Ball jump dust						  */
/******************************************/
void PS_BallJumpDust::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	bRemove = false;
	mPos = p;
	DustCol = c;
	up = v;
//	mParticles.reserve(30);
	Emit(20);
}
void PS_BallJumpDust::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		Particle p;
		p.Col = DustCol * UTIL_Misc::RandFloat(0.5f)+0.5f;
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(0.5f)+0.25f;
		p.ID = 2;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(0.13,0.13);
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(2.5);
		p.Vel = gpEng->mPlayer.mRight * (((UTIL_Misc::RandFloat(0.01f) + 0.002f)-0.006)*UTIL_Misc::RandFloat(7)) + 
			gpEng->mPlayer.mForward * (((UTIL_Misc::RandFloat(0.01f) + 0.002f)-0.006)*UTIL_Misc::RandFloat(7));
		p.P = p.oP = mPos;
		mParticles.push_back(p);
	}
}
void PS_BallJumpDust::Update()
{

	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		(*pt).oP = (*pt).P;
		(*pt).Vel -= up * 0.0002f;
		(*pt).Vel *= 0.95f;
		(*pt).P += (*pt).Vel;
		(*pt).oS = (*pt).S;
		(*pt).S += Vec2(0.007f,0.007f);
//		if((*pt).S.x > 32) (*pt).S.x = 32;
//		if((*pt).S.y > 32) (*pt).S.y = 32;
		(*pt).oR = (*pt).R;
		(*pt).R += (*pt).vR;
		(*pt).fLife -= TIME_STEP;
		(*pt).Col.x = UTIL_Misc::TimeRamp(DustCol.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp(DustCol.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp(DustCol.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(mParticles.empty())
		bRemove = true;
}
/******************************************/
/* collected a star						  */
/******************************************/
void PS_CollectedStar::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	mPos = p;
	bRemove = false;
	iEmitTime = 0;
	iStarCounter = 0;
	mParticles.reserve(500);
	Color = c;
	Emit(5 + UTIL_Misc::Rand()%25);
	iChildEmit = 0;
	bCircle = false;
	iLifeTime = 150;
}
void PS_CollectedStar::Emit(float count)
{

	for(int n=0; n<count; n++)
	{
		iStarCounter++;
		if(iStarCounter>=25)
			break;
		Particle p;
		p.Col = p.MaxColor = UTIL_Misc::Interpolate(Color,Vec3(1,1,1),UTIL_Misc::RandFloat(1));
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(0.5)+0.5f;
		p.ID = 1;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(1,1);
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(7.5);
		p.Vel = (mTarget-mPos)*0.0001f + Vec3(sin((float)(UTIL_Misc::Rand()%360)),cos((float)(UTIL_Misc::Rand()%360)),0)* UTIL_Misc::RandFloat(13);
		p.P = p.oP = mPos + Vec3((UTIL_Misc::Rand()%50)-25,(UTIL_Misc::Rand()%50)-25,0);
		mParticles.push_back(p);
	}
	iEmitTime = UTIL_Misc::Rand()%15;
}
void PS_CollectedStar::Update()
{
	iEmitTime--;
	if(iEmitTime<=0)
		if(iStarCounter<25)
			Emit(5 + UTIL_Misc::Rand()%3);

	if(bCircle)
	{
		iLifeTime--;
		mTarget = Vec3(105,35,0) + Vec3(sin((float)iLifeTime*0.075f)*25,cos((float)iLifeTime*0.075f)*25,0);
	}
	else
	{
		mTarget += Normalize((Vec3(105,35,0)-mTarget)) * 2.5f;
		mTarget.x += sin((float)iChildEmit)* (Distance(mTarget,Vec3(105,35,0)) * 0.05f);
	}
	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		if((*pt).ID == 1)
		{
			iChildEmit++;
			if((iChildEmit%5) == 0)
			{
				Particle p = (*pt);
				p.Vel = Vec3(0,UTIL_Misc::RandFloat(3),0);
				p.oS = p.S;
				p.oR = p.R;
				p.ID = 0;
				mParticles.push_back(p);
			}
		}
	}
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		iChildEmit++;
		if((*pt).ID == 0)
		{
			(*pt).fLife -= TIME_STEP;
			(*pt).oP = (*pt).P;
			(*pt).Vel.y += 0.05f;
			(*pt).Vel *= 0.95f;
			(*pt).P += (*pt).Vel;
		}
		else
		{
			(*pt).oP = (*pt).P;
//			if(bCircle)
//				(*pt).Vel += (mTarget - (*pt).P)*0.1f;
//			else
				(*pt).Vel += (mTarget - (*pt).P)*0.01f;//(mTarget - (*pt).P)*0.001f + Vec3(sin((float)iChildEmit),0,0) *(Distance(mTarget,(*pt).P) * 0.05);
			(*pt).Vel *= 0.95f;
			(*pt).P += (*pt).Vel;
			(*pt).oS = (*pt).S;
			(*pt).S += Vec2(0.1f,0.1f);
			if((*pt).S.x > 8) (*pt).S.x = 8;
			if((*pt).S.y > 8) (*pt).S.y = 8;
			(*pt).oR = (*pt).R;
			(*pt).R += (*pt).vR;
			if(!bCircle)
			if(Distance((*pt).P,Vec3(105,35,0))<100)
				bCircle = true;
			if(iLifeTime<=0)
				(*pt).fLife -= TIME_STEP;
		}
		(*pt).Col.x = UTIL_Misc::TimeRamp((*pt).MaxColor.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp((*pt).MaxColor.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp((*pt).MaxColor.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(mParticles.empty())
		bRemove = true;
}

/******************************************/
/* collected a Gem						  */
/******************************************/
void PS_CollectedGem::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	mPos = p;
	bRemove = false;
	iEmitTime = 0;
	iPartCounter = 0;
	mParticles.reserve(500);
	Color = c;
	Emit(5 + UTIL_Misc::Rand()%25);
	iChildEmit = 0;
}
void PS_CollectedGem::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		iPartCounter++;
		if(iPartCounter>=25)
			break;
		Particle p;
		p.Col = p.MaxColor = UTIL_Misc::Interpolate(Color,Vec3(1,1,1),UTIL_Misc::RandFloat(1));
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(1.0)+0.5f;
		p.ID = 0;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(1,1);
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(7.5);
		p.Vel = (gpEng->mPlayer.mPos-mPos)*0.0001f + Vec3(sin((float)(UTIL_Misc::Rand()%360)),cos((float)(UTIL_Misc::Rand()%360)),sin((float)(UTIL_Misc::Rand()%360)))* UTIL_Misc::RandFloat(0.05f);
		p.P = p.oP = mPos;
		mParticles.push_back(p);
	}
	iEmitTime = UTIL_Misc::Rand()%15;
}
void PS_CollectedGem::Update()
{
	iEmitTime--;
	if(iEmitTime<=0)
		if(iPartCounter<25)
			Emit(5 + UTIL_Misc::Rand()%3);

	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		if((*pt).vR != -1)
		{
			iChildEmit++;
			if((iChildEmit%10) == 0)
			{
				Particle p = (*pt);
				p.Vel = Vec3(0,0,0);
				p.oS = p.S;
				p.oR = p.R;
				p.vR = -1;
				mParticles.push_back(p);
			}
		}
	}
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		iChildEmit++;
		if((*pt).vR == -1)
		{
			(*pt).fLife -= TIME_STEP;
			(*pt).oP = (*pt).P;
			(*pt).P += (*pt).Vel;
		}
		else
		{
			(*pt).oP = (*pt).P;
			(*pt).Vel += (gpEng->mPlayer.mPos - (*pt).P)*0.005f;
			(*pt).Vel *= 0.95f;
			(*pt).P += (*pt).Vel;
			(*pt).oS = (*pt).S;
			(*pt).S += Vec2(0.1f,0.1f);
			if((*pt).S.x > 0.5f) (*pt).S.x = 0.5f;
			if((*pt).S.y > 0.5f) (*pt).S.y = 0.5f;
			(*pt).oR = (*pt).R;
			(*pt).R += (*pt).vR;
			//if(Distance((*pt).P,gpEng->mPlayer.mPos)<1)
				(*pt).fLife -= TIME_STEP;
		}
		if(gpEng->mPlayer.mpPlatform)
		{
			(*pt).P += (gpEng->mPlayer.mpPlatform->mPos-gpEng->mPlayer.mpPlatform->mOPos);
		}
		(*pt).Col.x = UTIL_Misc::TimeRamp((*pt).MaxColor.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp((*pt).MaxColor.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp((*pt).MaxColor.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(mParticles.empty())
		bRemove = true;
}
/******************************************/
/* Projectile dust						  */
/******************************************/
void PS_ProjectileDust::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	bRemove = false;
	mPos = p;
	DustCol = c;
	up = v;
	mParticles.reserve(41);
	Emit(40);
}
void PS_ProjectileDust::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		Particle p;
		p.Col = DustCol * UTIL_Misc::RandFloat(0.5f)+0.5f;
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(0.5f)+0.25f;
		p.ID = 2;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(0.43,0.43);
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(2.5);
		p.Vel = mRight * (((UTIL_Misc::RandFloat(0.01f) + 0.002f)-0.006)*UTIL_Misc::RandFloat(7)) + 
			mForward * (((UTIL_Misc::RandFloat(0.01f) + 0.002f)-0.006)*UTIL_Misc::RandFloat(7)) +
			up * (((UTIL_Misc::RandFloat(0.01f) + 0.002f)-0.006)*UTIL_Misc::RandFloat(7));
		p.P = p.oP = mPos;
		mParticles.push_back(p);
	}
}
void PS_ProjectileDust::Update()
{

	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		(*pt).oP = (*pt).P;
		(*pt).Vel -= up * 0.0002f;
		(*pt).Vel *= 0.95f;
		(*pt).P += (*pt).Vel;
		(*pt).oS = (*pt).S;
		(*pt).S += Vec2(0.007f,0.007f);
//		if((*pt).S.x > 32) (*pt).S.x = 32;
//		if((*pt).S.y > 32) (*pt).S.y = 32;
		(*pt).oR = (*pt).R;
		(*pt).R += (*pt).vR;
		(*pt).fLife -= TIME_STEP;
		(*pt).Col.x = UTIL_Misc::TimeRamp(DustCol.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp(DustCol.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp(DustCol.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(mParticles.empty())
		bRemove = true;
}

/******************************************/
/* snow effect					  */
/******************************************/
int iSnowCount, iOldSnowCount;
void PS_Snow::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	bRemove = false;
	//Emit(5 + UTIL_Misc::Rand()%25);
	bAlive = true;
	iEmitTime = 0;
	iSnowCount = iOldSnowCount = 0;
	mParticles.reserve(3000);
	Emit(5);
}
void PS_Snow::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		Particle p;
		p.Col = Vec3(1,1,1);
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(5)+3;
		p.ID = 2;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(0.1,0.1) * (1+UTIL_Misc::RandFloat(0.5));
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(2.5);
		p.Vel = Vec3((UTIL_Misc::RandFloat(0.0025f) - 0.00125f),-0.01f+UTIL_Misc::RandFloat(0.005f),(UTIL_Misc::RandFloat(0.0025f) - 0.00125f));
		p.P = p.oP = gpEng->mPlayer.mPos + Vec3(UTIL_Misc::RandFloat(20)-10, 2+UTIL_Misc::RandFloat(2), UTIL_Misc::RandFloat(20)-10);
		mParticles.push_back(p);
	}
	iEmitTime = UTIL_Misc::Rand()%3;
	iSnowCount += count;
}
void PS_Snow::Update()
{
//	if(bAlive)
//	{
//		iEmitTime--;
//		if(iEmitTime<=0)
			Emit(3 + UTIL_Misc::Rand()%6);
//	}

	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		(*pt).oP = (*pt).P;
		(*pt).Vel.x = sin((*pt).fLife)*0.01f;
		(*pt).Vel.z = cos((*pt).fLife)*0.01f;
		(*pt).Vel.y *= 0.95f;
		(*pt).Vel.y -= 0.001f;
		(*pt).P += (*pt).Vel;
		(*pt).fLife -= TIME_STEP;
	//	(*pt).Col.x = (*pt).Col.y = (*pt).Col.z = UTIL_Misc::TimeRamp(1,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
		{
			iSnowCount--;
			RemoveDeadParticle(pt);
		}
		if(gpEng->GetMapTile((*pt).P)!=0)
		{
			(*pt).P = (*pt).oP = gpEng->mPlayer.mPos + Vec3(UTIL_Misc::RandFloat(20)-10, 2+UTIL_Misc::RandFloat(2), UTIL_Misc::RandFloat(20)-10);
		}

	}

//	if(!bAlive && mParticles.empty())
//		bRemove = true;
/*	if(iSnowCount> iOldSnowCount+100)
	{
		iOldSnowCount = iSnowCount;
		char sz[64];
		sprintf(sz,"Snow Count: %i\n",iSnowCount);
		gLog.OutPut(sz);

	}*/
}

/******************************************/
/* area clear effect					  */
/******************************************/
void PS_AreaClear::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	mPos = p;
	bRemove = false;
	iEmitTime = 0;
	iStarCounter = 0;
	mParticles.reserve(500);
	Emit(5 + UTIL_Misc::Rand()%25);
	iChildEmit = 0;
	iLifeTime = 300;
}
void PS_AreaClear::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		iStarCounter++;
		if(iStarCounter>=25)
			break;
		Particle p;
		p.Col = p.MaxColor = Vec3(1,1,0);
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(0.5)+0.5f;
		p.ID = 1;
		p.R = p.oR = 0;
		p.S = p.oS = Vec2(1,1);
		p.vR = 0;//(UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(7.5);
		//p.Vel = (mTarget-mPos)*0.0001f + Vec3(sin((float)(UTIL_Misc::Rand()%360)),cos((float)(UTIL_Misc::Rand()%360)),0)* UTIL_Misc::RandFloat(13);
		p.P = p.oP = mPos;
		mParticles.push_back(p);
	}
	iEmitTime = UTIL_Misc::Rand()%15;
}
void PS_AreaClear::Update()
{
	Vec3 Cycle[6] = {Vec3(1,0.5,0.5),
					Vec3(1,1,0),
					Vec3(0.5,1,0.5),
					Vec3(0,1,1),
					Vec3(0.5,0.5,1),
					Vec3(1,0.5,1)};
						
	iLifeTime--;
	iEmitTime--;
	if(iEmitTime<=0)
		if(iStarCounter<25)
			Emit(5 + UTIL_Misc::Rand()%3);

		int Col = iLifeTime/5;
		while(Col>4)Col-=5;
		if(Col<0)Col=0;

	mTarget = Vec3(320,160,0)+Vec3(sin((float)iLifeTime*0.075f)*240,cos((float)iLifeTime*0.075f)*60,0);
	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		if((*pt).ID == 1)
		{
			iChildEmit++;
			if((iChildEmit%5) == 0)
			{
				Particle p = (*pt);
				p.Vel = Vec3(0,UTIL_Misc::RandFloat(3),0);
				p.oS = p.S;
				p.oR = p.R;
				p.ID = 0;
				mParticles.push_back(p);
			}
		}
	}
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		iChildEmit++;
		if((*pt).ID == 0)
		{
			(*pt).fLife -= TIME_STEP;
			(*pt).oP = (*pt).P;
			(*pt).Vel.y += 0.05f;
			(*pt).Vel *= 0.95f;
			(*pt).P += (*pt).Vel;
		}
		else
		{
			(*pt).oP = (*pt).P;
			(*pt).Vel += (mTarget - (*pt).P)*0.1f;
			(*pt).Vel *= 0.95f;
			(*pt).P += (*pt).Vel;
			(*pt).oS = (*pt).S;
			(*pt).S += Vec2(0.1f,0.1f);
			if((*pt).S.x > 16) (*pt).S.x = 16;
			if((*pt).S.y > 16) (*pt).S.y = 16;
			(*pt).oR = (*pt).R;
			(*pt).R += (*pt).vR;
			if(iLifeTime<=0)
				(*pt).fLife -= TIME_STEP;

			(*pt).MaxColor = Cycle[Col];

		}
		(*pt).Col.x = UTIL_Misc::TimeRamp((*pt).MaxColor.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp((*pt).MaxColor.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp((*pt).MaxColor.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);

/*		(*pt).Col.x = UTIL_Misc::TimeRamp(UTIL_Misc::RandFloat(1),0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp(UTIL_Misc::RandFloat(1),0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp(UTIL_Misc::RandFloat(1),0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
*/		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(mParticles.empty())
		bRemove = true;
}

/******************************************/
/* simple firework effect				  */
/******************************************/
void PS_Firework::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	bRemove = false;
	mParticles.reserve(40);
	mPos = p;
	switch(UTIL_Misc::Rand()%5)
	{
	case 0:	mEffectCol = Vec3(1,0,0); break;
	case 1:	mEffectCol = Vec3(1,1,0); break;
	case 2:	mEffectCol = Vec3(0.3f,1,0.3f); break;
	case 3:	mEffectCol = Vec3(0,1,1); break;
	case 4:	mEffectCol = Vec3(0,0.3f,1); break;
	}
	Emit(40);
}
void PS_Firework::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		Particle p;
		p.Col = p.MaxColor = mEffectCol * UTIL_Misc::RandFloat(0.5f)+0.5f;
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(0.5f)+2.25f;
		p.ID = 0;
		p.R = p.oR = UTIL_Misc::Rand()%360;
		p.S = p.oS = Vec2(24,24);
		p.vR = (UTIL_Misc::Rand()%2? -1: 1) * UTIL_Misc::RandFloat(2.5);
		p.Vel = Vec3( (((UTIL_Misc::RandFloat(1.00f) + 0.2f)-0.6)*UTIL_Misc::RandFloat(7)), (((UTIL_Misc::RandFloat(1.00f) + 0.2f)-0.6)*UTIL_Misc::RandFloat(7)),0);
		p.Vel.y -= UTIL_Misc::RandFloat(4);
		p.P = p.oP = mPos;
		mParticles.push_back(p);
	}
}
void PS_Firework::Update()
{

	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		(*pt).oP = (*pt).P;
		(*pt).Vel *= 0.95f;
		(*pt).Vel.y += 0.01f;
		(*pt).P += (*pt).Vel;
		(*pt).oS = (*pt).S;
		(*pt).S -= Vec2(0.07f,0.07f);
//		if((*pt).S.x > 32) (*pt).S.x = 32;
//		if((*pt).S.y > 32) (*pt).S.y = 32;
		(*pt).oR = (*pt).R;
		(*pt).R += (*pt).vR;
		(*pt).fLife -= TIME_STEP;
		(*pt).Col.x = UTIL_Misc::TimeRamp((*pt).MaxColor.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp((*pt).MaxColor.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp((*pt).MaxColor.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(mParticles.empty())
		bRemove = true;
}


/******************************************/
/* Bonus ring effect					  */
/******************************************/
void PS_BonusRing::Spawn(Vec3 p, Vec3 v, Vec2 s, Vec3 c, float r, unsigned int id, float life)
{
	mPos = p;
	bRemove = false;
	iEmitTime = 0;
	iStarCounter = 0;
	mParticles.reserve(500);
	Emit(5 + UTIL_Misc::Rand()%25);
	iChildEmit = 0;
	iLifeTime = 100;
}
void PS_BonusRing::Emit(float count)
{
	for(int n=0; n<count; n++)
	{
		iStarCounter++;
		if(iStarCounter>=10)
			break;
		Particle p;
		p.Col = p.MaxColor = Vec3(1,1,0);
		p.fLife = p.fMaxLife = UTIL_Misc::RandFloat(0.5)+0.5f;
		p.ID = 1;
		p.R = p.oR = 0;
		p.S = p.oS = Vec2(1,1);
		p.vR = 0;
		//p.Vel = (mTarget-mPos)*0.0001f + Vec3(sin((float)(UTIL_Misc::Rand()%360)),cos((float)(UTIL_Misc::Rand()%360)),0)* UTIL_Misc::RandFloat(13);
		p.P = p.oP = mPos;
		mParticles.push_back(p);
	}
	iEmitTime = UTIL_Misc::Rand()%15;
}
void PS_BonusRing::Update()
{
	Vec3 Cycle[6] = {Vec3(1,0.5,0.5),
					Vec3(1,1,0),
					Vec3(0.5,1,0.5),
					Vec3(0,1,1),
					Vec3(0.5,0.5,1),
					Vec3(1,0.5,1)};
						
		iLifeTime--;
	iEmitTime--;
	if(iEmitTime<=0)
		if(iStarCounter<10)
			Emit(5 + UTIL_Misc::Rand()%3);

		int Col = iLifeTime/10;
		if(Col<0)Col= -Col;
		while(Col>4)Col-=5;
		if(Col<0)Col=0;

	mTarget = Vec3(245,35,0)+Vec3(sin((float)iLifeTime*0.125f)*30,cos((float)iLifeTime*0.125f)*30,0);
	std::vector<Particle>::iterator pt;
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		if((*pt).ID == 1)
		{
			iChildEmit++;
			if((iChildEmit%5) == 0)
			{
				Particle p = (*pt);
				p.Vel = -((*pt).P - Vec3(245,25,0)) * 0.03f;//Vec3(0,UTIL_Misc::RandFloat(3),0);
				p.oS = p.S = Vec2(8,8);
				p.oR = p.R;
				p.ID = 0;
				mParticles.push_back(p);
			}
		}
	}
	for(pt=mParticles.begin(); pt<mParticles.end(); pt++)
	{
		iChildEmit++;
		if((*pt).ID == 0)
		{
			(*pt).fLife -= TIME_STEP;
			(*pt).oP = (*pt).P;
		//	(*pt).Vel.y += 0.015f;
			(*pt).Vel *= 0.95f;
			(*pt).P += (*pt).Vel;
		}
		else
		{
			(*pt).oP = (*pt).P;
			(*pt).Vel += (mTarget - (*pt).P)*0.1f;
			(*pt).Vel *= 0.95f;
			(*pt).P += (*pt).Vel;
			(*pt).oS = (*pt).S;
			(*pt).S += Vec2(0.1f,0.1f);
			if((*pt).S.x > 16) (*pt).S.x = 16;
			if((*pt).S.y > 16) (*pt).S.y = 16;
			(*pt).oR = (*pt).R;
			(*pt).R += (*pt).vR;
			if(gpEng->mPlayer.mbTimeOut || gpEng->mPlayer.mbFallOut || gpEng->mPlayer.mbLostBall)
			if(iLifeTime<=0)
				(*pt).fLife -= TIME_STEP;

			(*pt).MaxColor = Cycle[Col];

		}
		(*pt).Col.x = UTIL_Misc::TimeRamp((*pt).MaxColor.x,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp((*pt).MaxColor.y,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp((*pt).MaxColor.z,0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);

/*		(*pt).Col.x = UTIL_Misc::TimeRamp(UTIL_Misc::RandFloat(1),0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.y = UTIL_Misc::TimeRamp(UTIL_Misc::RandFloat(1),0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
		(*pt).Col.z = UTIL_Misc::TimeRamp(UTIL_Misc::RandFloat(1),0,0,(*pt).fMaxLife, (*pt).fMaxLife - (*pt).fLife);
*/		if((*pt).fLife <=0)
			RemoveDeadParticle(pt);
	}

	if(mParticles.empty())
		bRemove = true;
}