#include "Platform.h"
#include "Engine.h"
#include "Spikes.h"

char gPlatName[16];
bool TriggerPlatformByGName(Platform* p)
{
	if(!strcmp(p->Name, gPlatName))
		p->Trigger();
	return false;
}
bool FreezePlatforms(Platform* e)
{
	e->Freeze();
	return false;
}

Platform::Platform()
{
	char out[64];
	sprintf(out,"Platform Constructor %i\n",iLogicDebugStep);
	gLog.OutPut(out);
}
Platform::Platform(const Platform& p)
{
	gLog.OutPut("Platform copy constructor called.\n");
	this->bActivated = p.bActivated;
	this->bHasReturned = p.bHasReturned;
	this->bPlayOnce = p.bPlayOnce;
	this->bReturnStart = p.bReturnStart;
	this->bTriggered = p.bTriggered;
	this->bTriggerTouch = p.bTriggerTouch;
	this->bVisible = p.bVisible;
	this->bWaiting = p.bWaiting;
	this->iFadeTime	= p.iFadeTime;
	this->iLogicDebugStep = p.iLogicDebugStep;
	this->iMaxMoveTime = p.iMaxMoveTime;
	this->iMaxWaitTime = p.iMaxWaitTime;
	this->iMoveTime = p.iMoveTime;
	this->iWaitTime = p.iWaitTime;
	this->mbMoveDir = p.mbMoveDir;
	this->mbPlayerIsRiding = p.mbPlayerIsRiding;
	this->mChilds = p.mChilds;
	this->mMaxs = p.mMaxs;
	this->mMins = p.mMins;
	this->mMoveDest = p.mMoveDest;
	this->mMoveSpeed = p.mMoveSpeed;
	this->mMoveStart = p.mMoveStart;
	this->mOPos = p.mOPos;
	this->mpBlocks = p.mpBlocks;
	this->mPos = p.mPos;
	this->mSize = p.mSize;
	mBlocks.clear();
	for(int X=0;X<(int)mSize.x; X++)
	for(int Y=0;Y<(int)mSize.y; Y++)
	for(int Z=0;Z<(int)mSize.z; Z++)
	{
		if(mpBlocks[Z*(int)mSize.y*(int)mSize.x+Y*(int)mSize.x+X]>0)
		{
			mBlocks.push_back(*mpBlocks[Z*(int)mSize.y*(int)mSize.x+Y*(int)mSize.x+X]);
		}
	}
//	this->mBlocks = p.mBlocks;
	strcpy(this->Name,p.Name);
	strcpy(this->TargetName,p.TargetName);
}

void Platform::Generate(Vec3 mins, Vec3 maxs)
{
	gLog.OutPut("Generating Platform!\n");
	Vec3 center;


	center = mins + ((maxs - mins)/2);

	mMaxs = center - maxs;//local coordinates
	mMins = center - mins;//local

	mMoveStart = mPos = mOPos = center;
	mSize = maxs-mins;

	mpBlocks = new Block*[(int)(mSize.x*mSize.y*mSize.z)];
	char sz[256];
	sprintf(sz,"PlatSize: X%i Y%i Z%i\n", (int)(mSize.x),(int)(mSize.y),(int)(mSize.z));
	gLog.OutPut(sz);
//	memset(mpBlocks,0,sizeof(Block*)*(int)(mSize.x*mSize.y*mSize.z));
	for(int X=0;X<(int)mSize.x; X++)
	for(int Y=0;Y<(int)mSize.y; Y++)
	for(int Z=0;Z<(int)mSize.z; Z++)
	//	mpBlocks[Z*(int)mSize.y*(int)mSize.x+Y*(int)mSize.x+X] = 0;
		mpBlocks[UTIL_Misc::ArrayBounds(Z*(int)mSize.y*(int)mSize.x+Y*(int)mSize.x+X, (int)(mSize.x*mSize.y*mSize.z), "platform")] = 0;

	//remove blocks from global array and lists
	std::list<Block>::iterator b;
	for(b=gpEng->mMapBlocks.begin(); b!=gpEng->mMapBlocks.end(); b++)
	{
		if(BlockInBounds((*b),mins,maxs))
		{
			Vec3 p = (*b).pos;
			gpEng->cMap[(int)(p.z)*64*64+(int)(p.y)*64+(int)(p.x)] = 0;
			(*b).bTagged = true;
			mBlocks.push_back((*b));//<<insert into local list
			mBlocks.back().bTagged = false;
		}
	}
	gpEng->mMapBlocks.remove_if(RemoveTaggedBlock);
	//insert blocks into local array
	for(b=mBlocks.begin(); b!=mBlocks.end(); b++)
	{
		(*b).bTagged = false;
		(*b).mbPlatform = true;
		(*b).pPlat = this;
		Vec3 p = ((mPos - mMins) + mSize)-(*b).pos;
		sprintf(sz,"Creating block at: X%i Y%i Z%i\n", (int)(p.x),(int)(p.y),(int)(p.z));
		gLog.OutPut(sz);
		//mpBlocks[(int)(p.z)*(int)mSize.y*(int)mSize.x+(int)(p.y)*(int)mSize.x+(int)(p.x)] = &(*b);
		mpBlocks[UTIL_Misc::ArrayBounds((int)(p.z)*(int)mSize.y*(int)mSize.x+(int)(p.y)*(int)mSize.x+(int)(p.x), (int)(mSize.x*mSize.y*mSize.z), "Platform")] = &(*b);
	}
	sprintf(sz,"Num Blocks: %i\n",(int)mBlocks.size());
	gLog.OutPut(sz);
	//link any entities
	std::list<Entity*>::iterator e;
	for(e=gpEng->mGameEnts.begin(); e!=gpEng->mGameEnts.end(); e++)
	{
		if((*e)->BoxOverlap(mins, maxs))
		{
			(*e)->mpPlatform = this;
			mChilds.push_back((*e));
		}
	}


	mMoveSpeed = Vec3(0,0,0);

	iMoveTime = 0;
	iWaitTime = 0;
	bWaiting = false;
	mbPlayerIsRiding = false;
	bHasReturned = false;
	bTriggered = false;
	if(bVisible)
		iFadeTime = 100;
	else
		iFadeTime = -1;
	gLog.OutPut("Completed!\n");

	iLogicDebugStep = 0;
}

void Platform::Cleanup()
{
//	mChilds.remove_if(CleanEnts);
	gLog.OutPut("Platform cleared\n");
	mChilds.clear();
	if(mpBlocks)
		delete [] mpBlocks;
	mpBlocks = NULL;
}

void Platform::Update()
{
	if(mBlocks.empty())
	{
		char szOut[256];
		sprintf(szOut,"No blocks on step %i\n",iLogicDebugStep);
		gLog.OutPut(szOut);
		//rebuild the blocks
		for(int X=0;X<(int)mSize.x; X++)
		for(int Y=0;Y<(int)mSize.y; Y++)
		for(int Z=0;Z<(int)mSize.z; Z++)
		{
			if(mpBlocks[Z*(int)mSize.y*(int)mSize.x+Y*(int)mSize.x+X]>0)
			{
				mBlocks.push_back(*mpBlocks[Z*(int)mSize.y*(int)mSize.x+Y*(int)mSize.x+X]);
			}
		}

	}
	iLogicDebugStep++;

	mOPos = mPos;
	if(gpEng->mPlayer.mbTimeOut || gpEng->mPlayer.mbStageWon)
		return;


	if(bVisible)
		if(iFadeTime<100 && iFadeTime>=0)
			iFadeTime++;
	if(bActivated)
	{
		if(bWaiting)
		{
			iWaitTime++;
			if(iWaitTime>=iMaxWaitTime)
			{
				iWaitTime = 0;
				if(bReturnStart || bPlayOnce && bReturnStart && !bHasReturned)//if plat can return to start
				{
					if(bPlayOnce && bHasReturned || !bReturnStart)
					{
					//	gLog.OutPut("Platform reached end.\n");
						if(bPlayOnce && bHasReturned)
						{
						strcpy(gPlatName, TargetName);
						strcpy(gEntName, TargetName);
						gpEng->mPlatforms.remove_if(TriggerPlatformByGName);
						gpEng->mGameEnts.remove_if(TriggerEntityByGName);
						bHasReturned = false;
						bActivated  = false;
						bTriggered = false;
						bWaiting = false;
						}
					}
					else
					{
					//	gLog.OutPut("Platform has returned.\n");
						bWaiting = false;
						bHasReturned = true;
					}
				}
			}
		}
		else
		{
			iMoveTime++;
			if(iMoveTime>=iMaxMoveTime)
			{
				mbMoveDir = !mbMoveDir;
				iMoveTime = 0;
				if(iMaxWaitTime>0)
				{
					bWaiting = true;
					mMoveSpeed = Vec3(0,0,0);
				}
			}
		}

		float maxTime = (float)(iMaxMoveTime) / (1.0 / TIME_STEP);
		float time = (float)(iMoveTime) / (1.0 / TIME_STEP);
		float interp = time / maxTime;
		mOPos = mPos;

		if(mbMoveDir)
		{
			mPos = UTIL_Misc::Interpolate(mMoveStart+mMoveDest,mMoveStart,interp);
			mMoveSpeed = mPos - mOPos;
		}
		else
		{
			mPos = UTIL_Misc::Interpolate(mMoveStart,mMoveStart+mMoveDest,interp);
			mMoveSpeed = mPos - mOPos;
		}

		Vec3 offset = mPos - mOPos;
		if(!mChilds.empty())
		{
			std::list<Entity*>::iterator e;
			for(e=mChilds.begin(); e!=mChilds.end(); e++)
			{
				//(*e)->mOPos += offset;
				(*e)->mPos += offset;
				(*e)->mMins += offset;
				(*e)->mMaxs += offset;
				if((*e)->iBaseType == ENT_SPIKES)
				{
					((Spikes*)(*e))->mStartPos += offset;
				}
			}
		}
		//if(!mBlocks.empty())
		{
			std::list<Block>::iterator b;
			for(b=mBlocks.begin(); b!=mBlocks.end(); b++)
			{
				(*b).pos += offset;
			}
		}//*/

		if(mbPlayerIsRiding)
		{
			gpEng->mPlayer.mPos += offset;
			gpEng->mCameraPos += offset;
			//TODO there is a problem with this, needs to be fixed
	/*		Block* b= gpEng->GetMapTile(gpEng->mPlayer.mPos+gpEng->mPlayer.mMoveUp);
			if(b)
			{
				bool noMatch = true;
				std::list<Block>::iterator it;
				for(it=mBlocks.begin(); it!=mBlocks.end(); it++)
				{
					if(b == &(*it))
					{
						noMatch = false;
						break;
					}
				}
				if(noMatch)
				{
					mbPlayerIsRiding = false;
					gpEng->mPlayer.mpPlatform = NULL;
				}
			}
			else
			{
				mbPlayerIsRiding = false;
				gpEng->mPlayer.mpPlatform = NULL;
			}//*/
		}
		else//player is not riding this platform, we should push him
		{
			if(PointInPlat(gpEng->mPlayer.mPos) || PointInPlat(gpEng->mPlayer.mPos+gpEng->mPlayer.mMoveRight*0.2f)
				 || PointInPlat(gpEng->mPlayer.mPos-gpEng->mPlayer.mMoveRight*0.2f)
				  || PointInPlat(gpEng->mPlayer.mPos+gpEng->mPlayer.mMoveForward*0.2f)
				  || PointInPlat(gpEng->mPlayer.mPos-gpEng->mPlayer.mMoveForward*0.2f))
			{
				if(gpEng->GetMapTile(gpEng->mPlayer.mPos-mMoveSpeed))
					gpEng->mPlayer.mPos += mMoveSpeed;
				//break;
			}

		}
	}
}
void Platform::Trigger()
{
	if(!bTriggered)
	{
		//gLog.OutPut("Triggered!\n");
		if(!bActivated)
		if((int)strlen(TriggerMessage)>6)
		{
			PopMenu* menu = new PopMenu;
			menu->iExternalCode = PopMenu::POP_TUTORIAL;
			menu->Spawn(Vec2(320,240));
			strcpy(menu->szMessage,TriggerMessage);
			gpEng->mMenus.push_back(menu);
		}
		bActivated = true;
		if(!bVisible)
		{
			bVisible = true;
			iFadeTime =  0;
		}
		bTriggered = true;
//		strcpy(gPlatName, TargetName);
//		gpEng->mPlatforms.remove_if(TriggerPlatformByGName);


	}
}
void Platform::Render(const float interp)
{
	if(!bVisible)
		return;

	float alpha = iFadeTime>=0?UTIL_Misc::TimeRamp(0,1,0,100,iFadeTime):1;

	Vec3 p = UTIL_Misc::Interpolate(mPos, mOPos, interp)-mPos;

	if(!mBlocks.empty())
	{
		glColor4f(1,1,1,1);
		glPushMatrix();
		glTranslatef(p.x, p.y, p.z);

		if(fabs(alpha)<1.0)
		{
			UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glColor4f(1,1,1,alpha);
		}
		GLint AtLocmins = gpEng->mShdrMgr.GetAttribLocation("inmins");
		GLint AtLocmaxs = gpEng->mShdrMgr.GetAttribLocation("inmaxs");
		glBegin(GL_QUADS);
		std::list<Block>::iterator b;
		for(b=mBlocks.begin(); b!=mBlocks.end(); b++)
		{
			gpEng->RenderCube((*b), (*b).pos, p, (*b).scale, AtLocmins, AtLocmaxs);
		}
		glEnd();
		glColor4f(1,1,1,1);
		glDisable(GL_BLEND);
		glPopMatrix();

/*		p = mPos;
		glBegin(GL_LINE_LOOP);
		glVertex3fv((p - Vec3(1,0,0) * (mSize.x*0.5) - Vec3(0,0,1) * (mSize.z*0.5) - Vec3(0,1,0) * (mSize.y*0.5) ).v);
		glVertex3fv((p + Vec3(1,0,0) * (mSize.x*0.5) - Vec3(0,0,1) * (mSize.z*0.5) - Vec3(0,1,0) * (mSize.y*0.5) ).v);
		glVertex3fv((p + Vec3(1,0,0) * (mSize.x*0.5) + Vec3(0,0,1) * (mSize.z*0.5) - Vec3(0,1,0) * (mSize.y*0.5) ).v);
		glVertex3fv((p - Vec3(1,0,0) * (mSize.x*0.5) + Vec3(0,0,1) * (mSize.z*0.5) - Vec3(0,1,0) * (mSize.y*0.5) ).v);
		glEnd();

		glBegin(GL_LINE_LOOP);
		glVertex3fv((p - Vec3(1,0,0) * (mSize.x*0.5) - Vec3(0,0,1) * (mSize.z*0.5) + Vec3(0,1,0) * (mSize.y*0.5) ).v);
		glVertex3fv((p + Vec3(1,0,0) * (mSize.x*0.5) - Vec3(0,0,1) * (mSize.z*0.5) + Vec3(0,1,0) * (mSize.y*0.5) ).v);
		glVertex3fv((p + Vec3(1,0,0) * (mSize.x*0.5) + Vec3(0,0,1) * (mSize.z*0.5) + Vec3(0,1,0) * (mSize.y*0.5) ).v);
		glVertex3fv((p - Vec3(1,0,0) * (mSize.x*0.5) + Vec3(0,0,1) * (mSize.z*0.5) + Vec3(0,1,0) * (mSize.y*0.5) ).v);
		glEnd();
*/
	}
//	else gLog.OutPut("No visible blocks for platform!\n");
}

bool Platform::PointInPlat(Vec3 p)
{
	if(p.x<mPos.x-mMins.x || p.x>mPos.x-mMaxs.x)
		return false;
	if(p.y<mPos.y-mMins.y || p.y>mPos.y-mMaxs.y)
		return false;
	if(p.z<mPos.z-mMins.z || p.z>mPos.z-mMaxs.z)
		return false;

	return true;
}

Block* Platform::GetTileInPlat(Vec3 p)
{
	if(!bVisible)
		return 0;
//	Vec3 t = (mPos - mMaxs) - p;
	Vec3 t = ((mPos - mMins) + mSize) - p;
	int X,Y,Z;
	X = (int)t.x;
	Y = (int)t.y;
	Z = (int)t.z;

	if(X<0 || t.x >= mSize.x)
		return 0;
	if(Y<0 || t.y >= mSize.y)
		return 0;
	if(Z<0 || t.z >= mSize.z)
		return 0;

//	Block* b = mpBlocks[Z*(int)mSize.y*(int)mSize.x+Y*(int)mSize.x+X];
	Block* b = mpBlocks[UTIL_Misc::ArrayBounds(Z*(int)mSize.y*(int)mSize.x+Y*(int)mSize.x+X, (int)(mSize.x*mSize.y*mSize.z), "platform")];
	if(b)return b;
	return 0;
}
void Platform::Freeze()
{
	mOPos = mPos;
}
