#include "Recording.h"
#include "Engine.h"
#include "FileIO.h"

void RecordBetaState(std::list<BetaRecording>& record)
{

	BetaRecording br;
	BetaRecording oldbr = record.back();

	if(oldbr.STATE_KEY_ACTION2 != gpEng->KEY_ACTION2.state ||
		oldbr.STATE_KEY_BACK != gpEng->KEY_BACK.state ||
		oldbr.STATE_KEY_FORWARD != gpEng->KEY_FORWARD.state ||
		oldbr.STATE_KEY_JUMP != gpEng->KEY_JUMP.state ||
		oldbr.STATE_KEY_LEFT != gpEng->KEY_TURN_LEFT.state ||
		oldbr.STATE_KEY_PAUSE != gpEng->KEY_PAUSE.state ||
		oldbr.STATE_KEY_QUIT != gpEng->KEY_QUIT.state ||
		oldbr.STATE_KEY_RIGHT != gpEng->KEY_TURN_RIGHT.state)
	{

		br.iTime = gpEng->miInternalClock;
		br.STATE_KEY_ACTION2 = gpEng->KEY_ACTION2.state;
		br.STATE_KEY_BACK = gpEng->KEY_BACK.state;
		br.STATE_KEY_FORWARD = gpEng->KEY_FORWARD.state;
		br.STATE_KEY_JUMP = gpEng->KEY_JUMP.state;
		br.STATE_KEY_LEFT = gpEng->KEY_TURN_LEFT.state;
		br.STATE_KEY_PAUSE = gpEng->KEY_PAUSE.state;
		br.STATE_KEY_QUIT = gpEng->KEY_QUIT.state;
		br.STATE_KEY_RIGHT = gpEng->KEY_TURN_RIGHT.state;
		record.push_back(br);
	}
}

bool SaveBetaRecord(char* demoname, std::list<BetaRecording>& record)
{
	CFileIO fIO;
	unsigned int numRecord = (int)record.size();
	if(!fIO.GetFile(demoname,"wb"))
		return false;

	if(!fIO.WriteBinary(&numRecord))
		return false;

	while(!record.empty())
	{
		BetaRecording br = record.front();

		if(!fIO.WriteBinary(&br.iTime))
			return false;
		if(!fIO.WriteBinary(&br.STATE_KEY_ACTION2))
			return false;
		if(!fIO.WriteBinary(&br.STATE_KEY_BACK))
			return false;
		if(!fIO.WriteBinary(&br.STATE_KEY_FORWARD))
			return false;
		if(!fIO.WriteBinary(&br.STATE_KEY_JUMP))
			return false;
		if(!fIO.WriteBinary(&br.STATE_KEY_LEFT))
			return false;
		if(!fIO.WriteBinary(&br.STATE_KEY_PAUSE))
			return false;
		if(!fIO.WriteBinary(&br.STATE_KEY_QUIT))
			return false;
		if(!fIO.WriteBinary(&br.STATE_KEY_RIGHT))
			return false;

		record.pop_front();
	}
	fIO.ShutFile();
	return true;
}

bool LoadBetaRecord(char* demoname, std::list<BetaRecording>& record)
{
	CFileIO fIO;
	if(!fIO.GetFile(demoname,"rb"))
		return false;

	record.clear();

	unsigned int numRecord = 0;
	if(!fIO.ReadBinary(&numRecord))
		return false;

	for(unsigned int n=0; n<numRecord; n++)
	{
		BetaRecording br;

		if(!fIO.ReadBinary(&br.iTime))
			return false;
		if(!fIO.ReadBinary(&br.STATE_KEY_ACTION2))
			return false;
		if(!fIO.ReadBinary(&br.STATE_KEY_BACK))
			return false;
		if(!fIO.ReadBinary(&br.STATE_KEY_FORWARD))
			return false;
		if(!fIO.ReadBinary(&br.STATE_KEY_JUMP))
			return false;
		if(!fIO.ReadBinary(&br.STATE_KEY_LEFT))
			return false;
		if(!fIO.ReadBinary(&br.STATE_KEY_PAUSE))
			return false;
		if(!fIO.ReadBinary(&br.STATE_KEY_QUIT))
			return false;
		if(!fIO.ReadBinary(&br.STATE_KEY_RIGHT))
			return false;

		record.push_back(br);
	}

	return true;
}

void UpdateBetaRecordState(std::list<BetaRecording>& record)
{
	BetaRecording br = record.front();
	if(br.iTime == gpEng->miInternalClock)
	{
		gpEng->KEY_ACTION2.state = br.STATE_KEY_ACTION2;
		gpEng->KEY_BACK.state = br.STATE_KEY_BACK;
		gpEng->KEY_FORWARD.state = br.STATE_KEY_FORWARD;
		gpEng->KEY_JUMP.state = br.STATE_KEY_JUMP;
		gpEng->KEY_TURN_LEFT.state = br.STATE_KEY_LEFT;
		gpEng->KEY_PAUSE.state = br.STATE_KEY_PAUSE;
		gpEng->KEY_QUIT.state = br.STATE_KEY_QUIT;
		gpEng->KEY_TURN_RIGHT.state = br.STATE_KEY_RIGHT;
		record.pop_front();
	}
}