#include "TextureManager.h"
#include "UTIL_openGL.h"
//================================================================================================//
						/*****************************
						** Texture Manager			**	
						******************************/
//================================================================================================//
TextureManager::TextureManager()
{
	iNumTextures=0;
	iStartIndex=0;
	mTextures.clear();
}
//================================================================================================//
						/*****************************
						** cleanup					**	
						******************************/
//================================================================================================//
void TextureManager::CleanUp()
{
	std::map<std::string, unsigned int>::iterator it;
	for(it=mTextures.begin(); it!=mTextures.end(); it++)
	{
		glDeleteTextures(1,&(*it).second);
	}
	mTextures.clear();
	mDimensions.clear();
}
//================================================================================================//
						/*****************************
						** get a texture name by id	**	
						******************************/
//================================================================================================//
bool TextureManager::GetTextureName(unsigned int id, char* name)
{
	std::map<unsigned int, std::string>::iterator it;
	it=mTexNames.find(id);
	if(it != mTexNames.end())
	{
		strcpy(name,(char*)(*it).second.c_str());
		gLog.OutPut("Found Texture ID for name: ");
		gLog.OutPut(name);
		gLog.OutPut("\n");
		return true;
	}
	gLog.OutPut("Failed to find texture name\n");
	return false;
}
//================================================================================================//
						/*****************************
						** free specific texture	**	
						******************************/
//================================================================================================//
void TextureManager::FreeTexture(std::string name)
{
	std::map<std::string, unsigned int>::iterator it;
	it=mTextures.find(name);
	if(it == mTextures.end())
		return;

	std::map<unsigned int, std::pair<unsigned int, unsigned int> >::iterator n;
	n=mDimensions.find((*it).second);
	if(n != mDimensions.end())
		mDimensions.erase(n);

	glDeleteTextures(1,&(*it).second);
	mTextures.erase(it);
}
//================================================================================================//
						/*****************************
						** load a texture			**	
						******************************/
//================================================================================================//
unsigned int TextureManager::LoadTexture(bool aniso, std::string dir, std::string name, const unsigned int minfilter, const unsigned int magfilter , const unsigned int wrapX, const unsigned int wrapY )
{
	std::map<std::string, unsigned int>::iterator it;
	it=mTextures.find(name);
	if(it != mTextures.end())
		return (*it).second;

	char file[256];
	sprintf(file,"%s/%s",dir.c_str(),name.c_str());
	unsigned int ID, w, h;
//	if(!(ID=UTIL_GL::Image::LoadImage(file,minfilter,magfilter,wrapX,wrapY)))
//		return 0;
	if(gAnisotropy==0 || !aniso)
	{
		if(!(ID=UTIL_GL::Image::LoadImageMipMapped(file)))
//		if(!(ID=UTIL_GL::Image::LoadImage(file,minfilter,magfilter,wrapX,wrapY)))
//		if(!(ID=UTIL_GL::Image::LoadImage(file)))
			return 0;
		UTIL_GL::BindTexture(GL_TEXTURE_2D, ID);
		UTIL_GL::TextureWrap(wrapX,wrapY);
		UTIL_GL::TextureFilter(minfilter,magfilter);
	}
	else
	{
	//	if(!(ID=UTIL_GL::Image::LoadImage(file,w, h)))//TODO: we are storing the same texture twice - its broken, we should only do this once
		if(!(ID=UTIL_GL::Image::LoadImage(file)))
			return 0;
		UTIL_GL::BindTexture(GL_TEXTURE_2D, ID);
		UTIL_GL::TextureWrap(wrapX,wrapY);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, gAnisotropy);
	}

	glDisable(GL_TEXTURE_2D);

	mTextures[name] = ID;
	mTexNames[ID] = name;
	mDimensions[ID] = std::pair<unsigned int, unsigned int>(w,h);
	return mTextures[name];
}
//================================================================================================//
						/***********************************
						** Get dimensions of a texture ID **	
						***********************************/
//================================================================================================//
void TextureManager::GetDimensions(unsigned int ID, unsigned int& w, unsigned int& h)
{
	std::map<unsigned int, std::pair<unsigned int, unsigned int> >::iterator n;
	n=mDimensions.find(ID);
	if(n == mDimensions.end())
		return;

	w = (*n).second.first;
	h = (*n).second.second;
}
