#include "UTIL_Misc.h"
#include <time.h>
#include <stdio.h>
#include "log.h"
namespace UTIL_Misc
{
//================================================================================================//
						/*****************************************************
						 ** outputs a value between src & dst based on time **
						 *****************************************************/
//================================================================================================//
	float TimeRamp(float src, float dst, float startTime, float length, float curTime)

	{
		float t = ((curTime-startTime)/length);
		float l;
		src>dst?
			l=src-((src-dst)*t):
		l=src+((dst-src)*t);
		//clamp it
		if(src>dst)
		{
			if(l<dst)
				return dst;
			else if(l>src)
				return src;
		}
		else if(src<dst)
		{
			if(l>dst)
				return dst;
			else if(l<src)
				return src;
		}
		return l;
	}
//================================================================================================//
						/**********************************************
						 ** converts seconds to hours mins & seconds **
						 **********************************************/
//================================================================================================//
	void ConvertSecondsToHrsMinsSecs(int secondsIn, int& hoursOut, int& minsOut, int& secsOut)
	{
		hoursOut = (int)(secondsIn/(60*60));
		minsOut = (int)(secondsIn/60)-(60*hoursOut);
		secsOut = secondsIn>59?(int)secondsIn%60:(int)secondsIn;
	}
	void GetDate(char* date)
	{
		char cTimeBuffer[9];
		char cDateBuffer[9];
		_tzset();
		_strtime(cTimeBuffer);
		_strdate(cDateBuffer);
		sprintf(date, "%s %s", cDateBuffer, cTimeBuffer);
 	}
//================================================================================================//
						/*******************************************************
						 ** get the z axis angle in degrees from a 4x4 matrix **
						 *******************************************************/
//================================================================================================//
	float GetZangleFrom4x4(float* mat)
	{
		float Y,Z,C,trX,trY;
		Y = asin( mat[2]);
		C = cos( Y );
		if ( fabs( C ) > 0.000 )
		{
			trX = mat[0] / C;
			trY = -mat[1] / C;
		}
		else
		{
			trX = mat[5];
			trY = mat[4];
		}

		Z = RAD2DEG(atan2( trY, trX ));
		Z = Wrap(Z,360);
		return Z;
	}
//================================================================================================//
						/********************************
						 ** clamp a value to a min/max **
						 ********************************/
//================================================================================================//
	float Clamp(float in,float mins, float maxs)
	{
		if(in<mins)
			in = mins;
		if(in>maxs)
			in = maxs;
		return in;
	}
//================================================================================================//
						/*******************************
						 ** wrap a value to a min/max **
						 *******************************/
//================================================================================================//
	float Wrap(float in, float maxs)
	{
		if(in>maxs)
			in-=maxs;
		if(in<0)
			in+=maxs;
		return in;
	}
//================================================================================================//
						/*****************************
						 ** interpolation functions **
						 *****************************/
//================================================================================================//
	Vec2 Interpolate(Vec2 cur, Vec2 old, float interp)
	{
		Vec2 out;
		out = old + (cur-old) * interp;
		return out;
	}
	Vec3 Interpolate(Vec3 cur, Vec3 old, float interp)
	{
		Vec3 out;
		out = old + (cur-old) * interp;
		return out;
	}
	float Interpolate(float cur, float old, float interp)
	{
		float out;
		out = old + (cur-old) * interp;
		return out;
	}
//================================================================================================//
						/***********************************************
						 ** take a degree and return up/right vectors **
						 ***********************************************/
//================================================================================================//
	void MakeVectors(float rotation, Vec2& up, Vec2& right)
	{
		scalar cosYaw = (scalar)cos(DEG2RAD(rotation));
		scalar sinYaw = (scalar)sin(DEG2RAD(rotation));
		up = Vec2(cosYaw,sinYaw);
		right = Vec2(scalar(cos(DEG2RAD(rotation + 90.0))),scalar(sin(DEG2RAD(rotation + 90.0))));
		up.Normalize();
		right.Normalize();		
	}
//================================================================================================//
						/***********************************************
						 ** take a degree and return up/right vectors **
						 ***********************************************/
//================================================================================================//
	Vec2 CalculatePointOnCurve(Vec2* points, float d, int numPoints)
	{
		Vec2 p,v;
		float a = 1-d;
		float b = 1-a;
		int n;
		for(n=0;n<numPoints;n++)
		{
			v=points[n];
			if(n>0 && n<(numPoints-1)) v*=(float)numPoints-1;
			v*=pow(a,(float)numPoints-(n+1));
			v*=pow(b,(float)n);
			p+=v;
		}
		return p;
	}
	Vec3 CalculatePointOnCurve(Vec3* points, float d, int numPoints)
	{
		Vec3 p,v;
		float a = 1-d;
		float b = 1-a;
		int n;
		for(n=0;n<numPoints;n++)
		{
			v=points[n];
			if(n>0 && n<(numPoints-1)) v*=(float)numPoints-1;
			v*=pow(a,(float)numPoints-(n+1));
			v*=pow(b,(float)n);
			p+=v;
		}
		return p;
	}
//================================================================================================//
						/***********************************************
						 ** Random seed **
						 ***********************************************/
//================================================================================================//
	int gRndSeed;
	void Seed(int seed)
	{
		gRndSeed = seed;
	}
//================================================================================================//
						/***********************************************
						 ** Random number generator **
						 ***********************************************/
//================================================================================================//
	int Rand()
	{
		int k1;
		int ix = gRndSeed;
		
		k1 = ix / 127773;
		ix = 16807 * (ix - k1 * 127773) - k1 * 2836;
		if (ix < 0)
			ix += 2147483647;
		gRndSeed = ix;
		return gRndSeed;
	}
	float RandFloat(float range)//3 digits after decimal should be plenty
	{
		int iRange = (range* 1000);
		float result = (float)(Rand()%iRange);
		return result * 0.001f;
	}

//================================================================================================//
						/***********************************************
						 ** debugging arrays **
						 ***********************************************/
//================================================================================================//
	unsigned int ArrayBounds(unsigned int idx, unsigned int bounds, char* name)
	{
		if(idx>bounds)
		{
			char sz[256];
			sprintf(sz,"Array Out of bounds: %s\n", name);
			gLog.OutPut(sz);
		}
		return idx;
	}
}